 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.guimy.ctrls;

import name.mgplldz.guimy.listeners.CmbActList;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import name.mgplldz.tracemystack.TraceMyStack;

public abstract class CmbMy extends JComboBox {

    private int ignoreActListIdx = 0;
    private Component frame;

    public abstract String askForNewEntry(String oldEntry);

    public abstract boolean entryExists(String entry);

    public abstract boolean entryInvalid(String entry);

    public CmbMy(int x, int y, int w, int h, Object items[],
            Font fnt, JPanel owner, String name, Component fr,
            double xSc, double ySc) {
        super();
        frame = fr;
        setBackground(Color.white);
        setFont(fnt);
        setName(name);
        setBounds((int) (x * xSc), (int) (y * ySc), (int) (w * xSc), (int) (h * ySc));
        owner.add(this);
    }

    /*
     *  public
     */
    public String addEntry() {
        String newName = askForNewEntry("");
        if (newName != null) {
            setIgnoreActListIdx(1);
            addItem(newName);
            setIgnoreActListIdx(1);
            setSelectedItem(newName);
        }
        return newName;
    }

    public String renameEntry(int idx) {
        String newName = "";
        if (idx >= 0 && idx < getItemCount()) {
            newName = askForNewEntry(getItemAt(idx).toString());
            if (newName != null) {
                if (getItemCount() > 1) {
                    setIgnoreActListIdx(1);
                } else {
                    setIgnoreActListIdx(2);
                }
                removeItemAt(idx);
                insertItemAt(newName, idx);
                setSelectedIndex(idx);
            }
        }
        return newName;
    }

    public void delEntryByIdx(int idx) {
        int cnt = getItemCount();
        if (cnt > 0 && idx >= 0) {
            if (cnt > 1) {
                setIgnoreActListIdx(1);
            } else {
                setIgnoreActListIdx(2);
            }
            removeItemAt(idx);
        } else {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    "Can't remove item [" + new Integer(idx).toString() + "]", 3);
        }
    }

    public void setIgnoreActListIdx(int i) {
        ignoreActListIdx = i;
    }

    public boolean getIgnoreActListIdx() {
        if (ignoreActListIdx > 0) {
            ignoreActListIdx--;
            return true;
        }
        return false;
    }

    @Override
    public void removeAllItems() {
        setIgnoreActListIdx(1);
        super.removeAllItems();
    }

    /*
     * Test
     */
    public static void main(String args[]) {

        JFrame frm = new JFrame();
        JPanel pan = new JPanel();
        pan.setLayout(null);
        Integer items[] = new Integer[]{0, 1, 2};
        final CmbMy cmb = new CmbMy(10, 10, 120, 30, items,
                null, pan, CmbMyEnum.CMB_TEST.name(), null,
                1, 1) {

            @Override
            public String askForNewEntry(String oldEntry) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            @Override
            public boolean entryExists(String entry) {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            @Override
            public boolean entryInvalid(String entry) {
                throw new UnsupportedOperationException("Not supported yet.");
            }
        };
        cmb.addActionListener(new CmbActList());
        ButMy btAdd = new ButMy(10, 45, 120, 30, "Add",
                null, pan, ButMyEnum.BUT_TEST.name(), null, 1, 1);
        btAdd.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                int cnt = cmb.getItemCount();
                if (cnt == 0) {
                    cmb.setIgnoreActListIdx(1);
                }
                cmb.addItem(cmb.getItemCount());
                cmb.setSelectedIndex(cmb.getItemCount() - 1);
            }
        });
        ButMy btDel = new ButMy(10, 80, 120, 30, "Del",
                null, pan, ButMyEnum.BUT_TEST.name(), null, 1, 1);
        btDel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                int cnt = cmb.getItemCount();
                int idx = cmb.getSelectedIndex();
                if (cnt > 0 && idx >= 0) {
                    cmb.setIgnoreActListIdx(1);
                    cmb.removeItemAt(idx);
                    if (cnt > 1) {
                        cmb.setSelectedIndex(0);
                    } else {
                        cmb.setIgnoreActListIdx(1);
                    }

                }
            }
        });

        ButMy btReselect = new ButMy(10, 115, 120, 30, "Reselect",
                null, pan, ButMyEnum.BUT_TEST.name(), null, 1, 1);
        btReselect.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                cmb.setSelectedIndex(cmb.getSelectedIndex());
            }
        });
        pan.add(cmb);
        frm.setContentPane(pan);
        frm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frm.pack();
        frm.setBounds(100, 100, 200, 200);
        frm.setVisible(true);


    }

    private enum CmbMyEnum {

        CMB_TEST;
    }

    private enum ButMyEnum {

        BUT_TEST;
    }

    @Override
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        G2DQual.q(g2d, 1);
        super.paintComponent(g);
    }
}
