 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.guimy.ctrls;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.TimeZone;

public enum DateFormatEnum {

    NO_DATE(null, ""),
    HHMMSS(new SimpleDateFormat("HH:mm:ss"), ":"),
    HHMM(new SimpleDateFormat("HH:mm"), ":"),
    YYYYMMDD(new SimpleDateFormat("yyyy-MM-dd"), "-");
    /*
     * End enum
     */
    private SimpleDateFormat dateFormat;
    private String separator;
    public static final int MLS_SEC = 1000;
    public static final int MLS_MIN = 1000 * 60;
    public static final int MLS_HOUR = 1000 * 60 * 60;

    DateFormatEnum(SimpleDateFormat df, String separ) {
        dateFormat = df;
        separator = separ;
        if (dateFormat != null) {
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
        }
    }

    public SimpleDateFormat getDateFormat() {
        return dateFormat;
    }

    public String getSeparator() {
        return separator;
    }

    public static String formatDateStr(String inStr, DateFormatEnum dtFrmtEnu)
            throws ParseException {
        SimpleDateFormat dF = dtFrmtEnu.dateFormat;
        long time = dF.parse(inStr).getTime();
        return formatDateTime(time, dtFrmtEnu);
    }

    public static String formatDateTime(long time, DateFormatEnum dtFrmtEnu)
            throws ParseException {
        SimpleDateFormat dF = dtFrmtEnu.dateFormat;
        //String outStr = "";
        StringBuilder outStr = new StringBuilder();
        switch (dtFrmtEnu) {
            case HHMMSS:
            case HHMM:
                long hour = 3600000;
                long minute = 60000;
                long second = 1000;
                Long hours = time / hour;
                Long minutes = time % hour / minute;
                Long seconds = time % minute / second;
                hours = Math.abs(hours);
                minutes = Math.abs(minutes);
                seconds = Math.abs(seconds);
                if (time < 0) {
                    outStr.append("-");
                }
                outStr.append(hours < 10 ? "0" + hours.toString()
                        : hours.toString()).append(dtFrmtEnu.separator).append(
                        minutes < 10 ? "0" + minutes.toString()
                        : minutes.toString()).append(dtFrmtEnu.separator).append(
                        seconds < 10 ? "0" + seconds.toString() : seconds.toString());
                break;
            case YYYYMMDD:
                outStr.append(dF.format(time));
                break;
            default:
                break;
        }
        return outStr.toString();
    }

    public static int monthNb(int month) {
        switch (month) {
            case GregorianCalendar.JANUARY:
                return 1;
            case GregorianCalendar.FEBRUARY:
                return 2;
            case GregorianCalendar.MARCH:
                return 3;
            case GregorianCalendar.APRIL:
                return 4;
            case GregorianCalendar.MAY:
                return 5;
            case GregorianCalendar.JUNE:
                return 6;
            case GregorianCalendar.JULY:
                return 7;
            case GregorianCalendar.AUGUST:
                return 8;
            case GregorianCalendar.SEPTEMBER:
                return 9;
            case GregorianCalendar.OCTOBER:
                return 10;
            case GregorianCalendar.NOVEMBER:
                return 11;
            case GregorianCalendar.DECEMBER:
                return 12;
            default:
                assert false : "Wrong month number: " + month;
        }
        return 0;
    }

    public static void main(String args[]) {
    }
}
