 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.guimy.ctrls;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JColorChooser;
import name.mgplldz.tracemystack.TraceMyStack;

public class MousePointerMy {

    private static final MousePointerMy instance = new MousePointerMy();
    private final int POINTER_W = 32;
    private final int POINTER_H = 32;
    private final int POINTER_SHIFT = (POINTER_W - 1) / 2;
    private Cursor cursor;
    private BufferedImage buffImgCurIco = new BufferedImage(POINTER_W,
            POINTER_H, BufferedImage.BITMASK);
    private Graphics2D g2d = (Graphics2D) buffImgCurIco.getGraphics();
    private ArrayList<CurAndIco> pointers = new ArrayList<CurAndIco>();
    private Color ptrColor = Color.BLUE;
    private Point curShift = new Point(POINTER_SHIFT, POINTER_SHIFT);
    private BasicStroke lineStroke =
            new BasicStroke(0, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
    private BasicStroke lineStrokeBold =
            new BasicStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

    private MousePointerMy() {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        preparePointers();
    }

    public static MousePointerMy getInstance() {
        return instance;
    }

    interface PtrAndIcoArr {

        void makePtrAndIco();
    }
    private PtrAndIcoArr ptrsAndIcos[] = new PtrAndIcoArr[]{
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrC1s();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrC2s();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrS1s();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrS2s();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrC1b();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrC2b();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrS1b();
                packPtrAndIco();
            }
        },
        new PtrAndIcoArr() {

            public void makePtrAndIco() {
                ptrS2b();
                packPtrAndIco();
            }
        }
    };

    /*
     * Public
     */
    public int getIcoIdx(ImageIcon ico) {
        int rows = pointers.size();
        for (int i = 0; i < rows; ++i) {
            if (ico.equals(pointers.get(i).ico)) {
                return i;
            }
        }
        return -1;
    }

    public BufferedImage getBuffImgCur() {
        return buffImgCurIco;
    }

    public void setPtrCol() {
        ptrColor = JColorChooser.showDialog(null, "Mouse pointer color",
                g2d.getColor());
        preparePointers();
    }

    public ImageIcon[] getPtrsIcos() {
        int rows = pointers.size();
        ImageIcon icos[] = new ImageIcon[rows];
        for (int i = 0; i < rows; ++i) {
            icos[i] = pointers.get(i).ico;
        }
        return icos;
    }

    public Cursor getPtrByIdx(int idx) {
        return pointers.get(idx).cur;
    }

    public Cursor getPointerByIco(ImageIcon ico) {
        int rows = pointers.size();
        Cursor cur = null;
        for (int i = 0; i < rows; ++i) {
            if (ico.equals(pointers.get(i).ico)) {
                cur = pointers.get(i).cur;
                break;
            }
        }
        return cur;
    }

    public int getPtrsNb() {
        return pointers.size();
    }

    /*
     * Private
     */
    private void preparePointers() {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        pointers.clear();
        int rows = ptrsAndIcos.length;
        for (int i = 0; i < rows; ++i) {
            ptrsAndIcos[i].makePtrAndIco();
        }
    }

    private void packPtrAndIco() {
        //TraceMyStack.print(Thread.currentThread().getStackTrace(), "");
        ImageIcon ico = prepareIco(buffImgCurIco);
        cursor = Toolkit.getDefaultToolkit().createCustomCursor(
                ico.getImage(), curShift, "");
        pointers.add(new CurAndIco(ico, cursor));
    }

    private void ptrC2s() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawOval(5, 5, 20, 20);
        g2d.setStroke(lineStroke);
        G2DQual.q(g2d, 0);
        g2d.drawLine(6, 6, 24, 24);
        g2d.drawLine(24, 6, 6, 24);
        buffImgCurIco.flush();
    }

    private void ptrC2b() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawOval(2, 2, 26, 26);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(4, 4, 26, 26);
        g2d.drawLine(26, 4, 4, 26);
        buffImgCurIco.flush();
    }

    private void ptrC1s() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawOval(5, 5, 20, 20);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(3, 15, 27, 15);
        g2d.drawLine(15, 3, 15, 27);
        buffImgCurIco.flush();
    }

    private void ptrC1b() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawOval(2, 2, 26, 26);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(0, 15, 30, 15);
        g2d.drawLine(15, 0, 15, 30);
        buffImgCurIco.flush();
    }

    private void ptrS2s() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawRect(6, 6, 18, 18);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(4, 4, 26, 26);
        g2d.drawLine(26, 4, 4, 26);
        buffImgCurIco.flush();
    }

    private void ptrS2b() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawRect(2, 2, 26, 26);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(0, 0, 30, 30);
        g2d.drawLine(30, 0, 0, 30);
        buffImgCurIco.flush();
    }

    private void ptrS1s() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawRect(6, 6, 18, 18);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(15, 4, 15, 26);
        g2d.drawLine(4, 15, 26, 15);
        buffImgCurIco.flush();
    }

    private void ptrS1b() {
        clearBuffImg();
        G2DQual.q(g2d, 1);
        g2d.setColor(ptrColor);
        g2d.setStroke(lineStrokeBold);
        g2d.drawRect(2, 2, 26, 26);
        G2DQual.q(g2d, 0);
        g2d.setStroke(lineStroke);
        g2d.drawLine(15, 0, 15, 30);
        g2d.drawLine(0, 15, 30, 15);
        buffImgCurIco.flush();
    }

    private ImageIcon prepareIco(BufferedImage bI) {
        //TraceMyStack.print(Thread.currentThread().getStackTrace(), "");
        int w = bI.getWidth();
        int h = bI.getHeight();
        BufferedImage bICopy = new BufferedImage(w, h, BufferedImage.BITMASK);
        for (int i = 0; i < h; ++i) {
            for (int k = 0; k < w; ++k) {
                bICopy.setRGB(i, k, bI.getRGB(i, k));
            }
        }
        ImageIcon ico = new ImageIcon(bICopy);
        return ico;
    }

    private void clearBuffImg() {
        g2d.setBackground(new Color(0, 0, 0, 0));
        g2d.clearRect(0, 0, POINTER_W, POINTER_H);
        buffImgCurIco.flush();
    }

    /*
     * class
     */
    private class CurAndIco {

        private ImageIcon ico;
        private Cursor cur;

        public CurAndIco(ImageIcon i, Cursor c) {
            ico = i;
            cur = c;
        }
    }

    /*
     * Test
     */
    public static void main(String args[]) {
    }
}
