 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.guimy.ctrls;

import java.awt.Color;
import java.awt.Component;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

public class MsgBox {

    static final Color COL_GRAY = new Color(240, 240, 240);

    /*
     * Public
     */
    public static void msgPlainNoTh(final String text, final Component parent,
            final int messageType) {
        msgPlain(text, parent, messageType);
    }

    public static void msgPlainTh(final String text, final Component parent,
            final int messageType) {
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                msgPlain(text, parent, messageType);
            }
        });
    }

    public Integer msgAskTh(String text, Component owner, int messageType) {
        MsgAsk ma = new MsgAsk(text, owner, messageType);
        ma.run();
        return ma.getRetVal();
    }

    public Object msgCmbTh(String info, Object values[], Object oldValue,
            Component owner) {
        MsgCmb mc = new MsgCmb(info, values, oldValue, owner);
        mc.run();
        return mc.getRetVal();
    }

    public Integer getIntTh(Integer oldInt, String info, Component owner) {
        GetInt gi = new GetInt(oldInt, info, owner);
        gi.run();
        return gi.getRetVal();
    }

    public Double getDblTh(Double oldDbl, String info, Component owner) {
        GetDbl gd = new GetDbl(oldDbl, info, owner);
        gd.run();
        return gd.getRetVal();
    }

    public String getStrTh(String oldTxt, String info, Component owner) {
        GetStr gs = new GetStr(oldTxt, info, owner);
        gs.run();
        return gs.getRetVal();
    }

    public String getDateStrTh(String oldTxt, DateFormatEnum frmtEnum,
            String info, Component owner) {
        GetDateStr gds = new GetDateStr(oldTxt, frmtEnum, info, owner);
        gds.run();
        return gds.getRetVal();
    }

    /*
     * Private
     */
    private static void msgPlain(String text, Component owner,
            int messageType) {
        JTextArea txtArea = new JTextArea();
        txtArea.setText(text);
        txtArea.setEditable(false);
        txtArea.setBackground(COL_GRAY);
        txtArea.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        JOptionPane.showMessageDialog(owner, txtArea, "OOoDigitizer",
                messageType);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
    }

    private static Integer msgAsk(final String text, Component owner,
            int messageType) {
        JTextArea txtArea = new JTextArea();
        txtArea.setText(text);
        txtArea.setEditable(false);
        txtArea.setBackground(COL_GRAY);
        txtArea.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        int retVal = JOptionPane.showConfirmDialog(owner, txtArea, "OOoDigitizer",
                JOptionPane.YES_NO_OPTION, messageType);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
        return retVal;
    }

    private static Object msgCmb(String info, Object values[],
            Object oldValue, Component owner) {
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        Object sel = JOptionPane.showInputDialog(
                owner, info, "OOoDigitizer", -1, null,
                values, oldValue);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
        return (sel == null) ? oldValue : sel;
    }

    private static Integer getInt(Integer oldInt, String info, Component owner) {
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        String tmpStr = JOptionPane.showInputDialog(owner, info, oldInt);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
        oldInt = null;
        boolean okFl = false;
        while (!okFl) {
            if (tmpStr != null) {
                try {
                    oldInt = Integer.parseInt(tmpStr);
                    okFl = true;
                } catch (NumberFormatException ex1) {
                    try {
                        tmpStr = tmpStr.replace(",", ".");
                        oldInt = (int) Double.parseDouble(tmpStr);
                        okFl = true;
                    } catch (NumberFormatException ex) {
                        if (frm != null) {
                            topFl = frm.isAlwaysOnTop();
                            if (topFl) {
                                frm.setAlwaysOnTop(false);
                            }
                        }
                        MsgBox.msgPlainTh("It's not a number.", owner,
                                JOptionPane.INFORMATION_MESSAGE);
                        tmpStr = JOptionPane.showInputDialog(owner, info, tmpStr);
                        if (frm != null) {
                            if (topFl) {
                                frm.setAlwaysOnTop(true);
                            }
                        }
                    }
                }
            } else {
                okFl = true;
            }
        }
        return oldInt;
    }

    private static Double getDouble(Double oldDbl, String info, Component owner) {
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        String tmpStr = JOptionPane.showInputDialog(owner, info, oldDbl);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
        oldDbl = null;
        boolean okFl = false;
        while (!okFl) {
            if (tmpStr != null) {
                try {
                    tmpStr = tmpStr.replace(",", ".");
                    oldDbl = (double) Double.parseDouble(tmpStr);
                    okFl = true;
                } catch (NumberFormatException ex) {
                    if (frm != null) {
                        topFl = frm.isAlwaysOnTop();
                        if (topFl) {
                            frm.setAlwaysOnTop(false);
                        }
                    }
                    MsgBox.msgPlainTh("It's not a number.", owner,
                            JOptionPane.INFORMATION_MESSAGE);
                    tmpStr = JOptionPane.showInputDialog(owner, info, tmpStr);
                    if (frm != null) {
                        if (topFl) {
                            frm.setAlwaysOnTop(true);
                        }
                    }
                }
            } else {
                okFl = true;
            }
        }
        return oldDbl;
    }

    private static String getString(String oldTxt, String info, Component owner) {
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        String tmpStr = JOptionPane.showInputDialog(owner, info, oldTxt);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
        return tmpStr;
    }

    private static String getStringDate(String oldTxt, DateFormatEnum frmtEnum,
            String info, Component owner) {
        DateFormatEnum dtFrmtEnumTimeNoSec = DateFormatEnum.HHMM;
        SimpleDateFormat dtFrmtTimeNoSec =
                dtFrmtEnumTimeNoSec.getDateFormat();
        SimpleDateFormat dtFrmt = frmtEnum.getDateFormat();
        boolean topFl = false;
        JFrame frm = (JFrame) owner;
        if (frm != null) {
            topFl = frm.isAlwaysOnTop();
            if (topFl) {
                frm.setAlwaysOnTop(false);
            }
        }
        String tmpStr = JOptionPane.showInputDialog(owner, info, oldTxt);
        if (frm != null) {
            if (topFl) {
                frm.setAlwaysOnTop(true);
            }
        }
        oldTxt = null;
        boolean okFl = false;
        while (!okFl) {
            if (tmpStr != null) {
                try {
                    switch (frmtEnum) {
                        case HHMMSS:
                            try {
                                dtFrmt.parse(tmpStr);
                                oldTxt = DateFormatEnum.formatDateStr(tmpStr, frmtEnum);
                                okFl = true;
                            } catch (ParseException ex) {
                                dtFrmtTimeNoSec.parse(tmpStr);
                                oldTxt = DateFormatEnum.formatDateStr(tmpStr,
                                        dtFrmtEnumTimeNoSec);
                                okFl = true;
                            }
                            break;
                        default:
                            dtFrmt.parse(tmpStr);
                            oldTxt = DateFormatEnum.formatDateStr(tmpStr, frmtEnum);
                            okFl = true;
                            break;
                    }
                } catch (ParseException ex) {
                    if (frm != null) {
                        topFl = frm.isAlwaysOnTop();
                        if (topFl) {
                            frm.setAlwaysOnTop(false);
                        }
                    }
                    MsgBox.msgPlainTh("Wrong format.", owner,
                            JOptionPane.INFORMATION_MESSAGE);
                    tmpStr = JOptionPane.showInputDialog(owner, info, tmpStr);
                    if (frm != null) {
                        if (topFl) {
                            frm.setAlwaysOnTop(true);
                        }
                    }
                }
            } else {
                okFl = true;
            }
        }
        return oldTxt;
    }

    public static void main(String args[]) {
        int i = (Integer) MsgBox.msgCmb(
                "Line width:",
                new Integer[]{1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21,
                    23, 25, 27, 29, 31, 33, 35, 37, 39, 41},
                new Integer(3), null);
        System.out.println(i);
    }


    /*
     * Class
     */
    private class MsgAsk implements Runnable {

        private String text;
        private int messageType;
        private Component owner;
        private Integer retVal;

        public MsgAsk(final String t, final Component own,
                final int msgType) {
            text = t;
            messageType = msgType;
            owner = own;
        }

        public void run() {
            retVal = msgAsk(text, owner, messageType);
        }

        public Integer getRetVal() {
            return retVal;
        }
    }

    private class MsgCmb implements Runnable {

        private String info;
        private Object values[];
        private Object oldValue;
        private Component owner;
        private Object retVal;

        public MsgCmb(String i, Object v[], Object oldV, Component own) {
            info = i;
            values = v;
            oldValue = oldV;
            owner = own;
        }

        public void run() {
            retVal = msgCmb(info, values, oldValue, owner);
        }

        public Object getRetVal() {
            return retVal;
        }
    }

    private class GetInt implements Runnable {

        private Integer oldInt;
        private String info;
        private Component owner;
        private Integer retVal;

        public GetInt(Integer old, String i, Component own) {
            oldInt = old;
            info = i;
            owner = own;
        }

        public void run() {
            retVal = getInt(oldInt, info, owner);
        }

        public Integer getRetVal() {
            return retVal;
        }
    }

    private class GetDbl implements Runnable {

        private Double oldDbl;
        private String info;
        private Component owner;
        private Double retVal;

        public GetDbl(Double old, String i, Component own) {
            oldDbl = old;
            info = i;
            owner = own;
        }

        public void run() {
            retVal = getDouble(oldDbl, info, owner);
        }

        public Double getRetVal() {
            return retVal;
        }
    }

    private class GetStr implements Runnable {

        private String oldTxt;
        private String info;
        private Component owner;
        private String retVal = "";

        public GetStr(String old, String i, Component own) {
            oldTxt = old;
            info = i;
            owner = own;
        }

        public void run() {
            retVal = getString(oldTxt, info, owner);
        }

        public String getRetVal() {
            return retVal;
        }
    }

    private class GetDateStr implements Runnable {

        private String oldTxt;
        private DateFormatEnum frmtEnum;
        private String info;
        private Component owner;
        private String retVal;

        public GetDateStr(String old, DateFormatEnum fe,
                String i, Component own) {
            oldTxt = old;
            frmtEnum = fe;
            info = i;
            owner = own;
        }

        public void run() {
            retVal = getStringDate(oldTxt, frmtEnum, info, owner);
        }

        public String getRetVal() {
            return retVal;
        }
    }
}
//private static void showErrorPane(String message, String title) {
//   JOptionPane pane = new JOptionPane(message, JOptionPane.ERROR_MESSAGE);
//   JDialog dialog = pane.createDialog(title);
//   dialog.setAlwaysOnTop(true);
//   dialog.setVisible(true);
//}

