 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.guimy.ctrls;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.ToolTipManager;

public class SpinMy extends JSpinner {

    enum SpinMyEnum {

        T0_SPIN_DEG_MIN,
        T0_SPIN_DEG_MAX,
        T1_SPIN_SMOOTH
    }
    private final int minVal;
    private final int maxVal;
    private boolean ignoreSpinListFl = false;

    public SpinMy(int x, int y, int w, int h,
            Object value, Font font,
            JPanel ownerPanel, Color color, String name,
            int min, int max, boolean editable, double xSc, double ySc) {
        super();
        minVal = min;
        maxVal = max;
        setName(name);
        setFont(font);
        setBounds((int) (x * xSc), (int) (y * ySc), (int) (w * xSc), (int) (h * ySc));
        setValue(value);
        setBackground(color);
        ((JSpinner.DefaultEditor) getEditor()).getTextField().setEditable(editable);
        ((JSpinner.DefaultEditor) getEditor()).getTextField().setBackground(color);
        ownerPanel.add(this);
    }

    public int getMin() {
        return minVal;
    }

    public int getMax() {
        return maxVal;
    }

    public void on(boolean doOn) {
        Color col;
        ignoreSpinListFl = (!doOn);
        setEnabled(doOn);
        if (doOn) {
            col = GUIMy.COL_WHITE;
        } else {
            col = GUIMy.COL_GRAY;
        }
        ((JSpinner.DefaultEditor) getEditor()).getTextField().setBackground(col);
    }

    public void registerToolTip(boolean doRegister) {
        if (doRegister) {
            ToolTipManager.sharedInstance().registerComponent(this);
            ToolTipManager.sharedInstance().registerComponent(
                    ((JSpinner.DefaultEditor) this.getEditor()).getTextField());
            int cmpsNb = this.getComponentCount();
            for (int i = 0; i < cmpsNb; ++i) {
                ToolTipManager.sharedInstance().registerComponent(
                        (JComponent) this.getComponent(i));
            }
        } else {
            ToolTipManager.sharedInstance().unregisterComponent(this);
            ToolTipManager.sharedInstance().unregisterComponent(
                    ((JSpinner.DefaultEditor) this.getEditor()).getTextField());
            int cmpsNb = this.getComponentCount();
            for (int i = 0; i < cmpsNb; ++i) {
                ToolTipManager.sharedInstance().unregisterComponent(
                        (JComponent) this.getComponent(i));
            }
        }
    }

    public boolean getIgnoreSpinListFl() {
        return ignoreSpinListFl;
    }

    @Override
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        G2DQual.q(g2d, 1);
        super.paintComponent(g);
    }
}
