 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.guimy.ctrls;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JPanel;
import javax.swing.JTextField;
import name.mgplldz.tracemystack.TraceMyStack;

public class TxtFldMy extends JTextField {

    private boolean ignoreMouseListFl = false;
    private Object value = null;
    public static final boolean EDITABLE = true;
    public static final boolean FOCUSABLE = true;
    private String stringFormat = "%s";
    private String integerFormat = "%d";
    private String doubleFormat = "%g";
    private boolean dontUseFl = true;

    public TxtFldMy(int x, int y, int w, int h,
            Object value, Font font, int alignment, JPanel ownerPanel,
            Color color, String name,
            boolean editable, boolean focusable, double xSc, double ySc) {
        super();
        setName(name);
        setFont(font);
        setBounds((int) (x * xSc), (int) (y * ySc), (int) (w * xSc), (int) (h * ySc));
        setValue(value);
        setBackground(color);
        setEditable(editable);
        setFocusable(focusable);
        setHorizontalAlignment(alignment);
        ownerPanel.add(this);
    }

    public void onList(boolean doON) {
        ignoreMouseListFl = !doON;
        if (doON) {
            setBackground(GUIMy.COL_WHITE);
        } else {
            setBackground(GUIMy.COL_GRAY);
        }
    }

    public void onListAndON(boolean doON) {
        setEnabled(doON);
        onList(doON);
    }

    public boolean getIgnoreMouseListFl() {
        return ignoreMouseListFl;
    }

    public final void setValue(Object val) {
        value = val;
        if (value instanceof java.lang.String) {
            dontUseFl = false;
            setText(String.format(stringFormat, value));
        } else if (value instanceof java.lang.Number) {
            if (value instanceof java.lang.Float
                    || value instanceof java.lang.Double) {
                dontUseFl = false;
                setText(String.format(doubleFormat, value).replace(',', '.'));
            } else if (value instanceof java.lang.Integer
                    || value instanceof java.lang.Long) {
                dontUseFl = false;
                setText(String.format(integerFormat, value));
            }
        }
    }

    public Object getValue() {
        return value;
    }

    public void setStringFormat(String f) {
        stringFormat = f;
    }

    public void setIntegerFormat(String f) {
        integerFormat = f;
    }

    public void setDoubleFormat(String f) {
        doubleFormat = f;
    }
    /*
     * Override
     */

    @Override
    public void setText(String text) {
        if (dontUseFl) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    "Use setValue(Object val, String printFormat", 3);
            System.exit(1);
        }
        dontUseFl = true;
        super.setText(text);
    }

    @Override
    public String getText() {
        if (dontUseFl) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    "Use getValue()", 3);
            System.exit(1);
        }
        dontUseFl = true;
        return super.getText();
    }

    @Override
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        G2DQual.q(g2d, 1);
        super.paintComponent(g);
    }
}
