 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Scanner;
import name.mgplldz.tracemystack.TraceMyStack;

public class ConfigGUI {

    private String userDir = System.getProperty("user.home");
    private String nL = System.getProperty("line.separator");
    private String fS = System.getProperty("file.separator");
    private String filePath = userDir + fS + "ooodigitizerconf.txt";
    private int fntSize = 0;
    private double xScale = 0;
    private double yScale = 0;
    private final int DBG_LEV = 3;

    ConfigGUI(int fnSz, double xS, double yS) {
        fntSize = fnSz;
        xScale = xS;
        yScale = yS;
    }

    public void readConfigFile() {
        Scanner scan = null;
        ArrayList<String> vals = new ArrayList<String>();
        try {
            scan = new Scanner(new FileInputStream(filePath));
            while (scan.hasNext()) {
                vals.add(scan.nextLine());
            }
        } catch (IOException e) {
        } finally {
            scan.close();
        }
        fntSize = Integer.parseInt(vals.get(1));
        xScale = Double.parseDouble(vals.get(3));
        yScale = Double.parseDouble(vals.get(5));
    }

    public void saveConfigFile() throws ConfigGUIFileWriteException {
        checkFileWrite();
        Writer out = null;
        try {
            out = new OutputStreamWriter(new FileOutputStream(filePath), "UTF-8");
            out.write("Font size:" + nL);
            out.write(((Integer) (fntSize)).toString() + nL);
            out.write("Width scale:" + nL);
            out.write(((Double) (xScale)).toString() + nL);
            out.write("Height scale:" + nL);
            out.write(((Double) (yScale)).toString() + nL);
        } catch (IOException e) {
        } finally {
            try {
                out.close();
            } catch (IOException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            }
        }

    }

    public void setValues(int fnSz, double xSc, double ySc) throws ConfigGUIFileWriteException {
        fntSize = fnSz;
        xScale = xSc;
        yScale = ySc;
        saveConfigFile();
    }

    public int getFntSize() {
        return fntSize;
    }

    public double getXSc() {
        return xScale;
    }

    public double getYSc() {
        return yScale;
    }

    public String getFilePath() {
        return filePath;
    }

    public void checkFileExists() throws ConfigGUIFileExistsException {

        File file = new File(filePath);
        boolean exists = file.exists();
        if (!exists) {
            throw new ConfigGUIFileExistsException("Config file doesn't exist.");
        }
    }

    public void checkFileWrite() throws ConfigGUIFileWriteException {

        File file = new File(filePath);
        boolean exists = file.canWrite();
        if (!exists) {
            throw new ConfigGUIFileWriteException("Can't write config file.");
        }
    }

    public void checkFileRead() throws ConfigGUIFileReadException {

        File file = new File(filePath);
        boolean exists = file.canRead();
        if (!exists) {
            throw new ConfigGUIFileReadException("Can't read config file.");
        }
    }

    public void createEmptyFile() throws ConfigGUIFileWriteException {

        File file = new File(filePath);
        try {
            file.createNewFile();
        } catch (IOException ex) {
            throw new ConfigGUIFileWriteException("Can't create config file.");
        }
    }

    public class ConfigGUIFileExistsException extends Exception {

        public ConfigGUIFileExistsException(String msg) {
            super(msg);
        }
    }

    public class ConfigGUIFileReadException extends Exception {

        public ConfigGUIFileReadException(String msg) {
            super(msg);
        }
    }

    public class ConfigGUIFileWriteException extends Exception {

        public ConfigGUIFileWriteException(String msg) {
            super(msg);
        }
    }

    public static void main(String args[]) {
    }
}
