 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui;

import com.sun.star.uno.XComponentContext;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.geom.Point2D;
import java.text.ParseException;
import java.util.Locale;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import name.mgplldz.guimy.ctrls.DateFormatEnum;
import name.mgplldz.guimy.ctrls.MsgBox;
import name.mgplldz.guimy.ctrls.UIManagerSet;
import name.mgplldz.oood.gui.ConfigGUI.ConfigGUIFileExistsException;
import name.mgplldz.oood.gui.ConfigGUI.ConfigGUIFileReadException;
import name.mgplldz.oood.gui.ConfigGUI.ConfigGUIFileWriteException;
import name.mgplldz.oood.gui.menu.MenuBar;
import name.mgplldz.oood.gui.menu.series.ItmsMSeries;
import name.mgplldz.oood.gui.tab.TabPane;
import name.mgplldz.oood.gui.frame.Frame;
import name.mgplldz.oood.gui.menu.draw.lang.EnUsMDraw;
import name.mgplldz.oood.gui.menu.draw.lang.LngMDraw;
import name.mgplldz.oood.gui.menu.interp.lang.EnUsMInterp;
import name.mgplldz.oood.gui.menu.interp.lang.LngMInterp;
import name.mgplldz.oood.gui.menu.settings.lang.EnUsMSettings;
import name.mgplldz.oood.gui.menu.settings.lang.LngMSettings;
import name.mgplldz.oood.gui.tab.digit.lang.EnUsTDigit;
import name.mgplldz.oood.gui.tab.digit.lang.LngTDigit;
import name.mgplldz.oood.lang.Lng;
import name.mgplldz.oood.lang.Lng.LangEnum;
import name.mgplldz.oood.save.CSV;
import name.mgplldz.oood.save.Office;
import name.mgplldz.oood.scale.Scale.ScaleLogException;
import name.mgplldz.ooohelp.OfficeHelp;
import name.mgplldz.ooohelp.OfficeHelp.OOoNullContextException;
import name.mgplldz.tracemystack.TraceMyStack;

public class GUI {

    public final XComponentContext contextOOo;

    /*
     * Debug level for TraceMyStack class
     */
    public static final int DBG_LEV = 3;
    /*
     * Font
     */
    private int fntSize = 12;
    private double fnt17Scale = 1.42;
    Font fntNorm12 = new Font("", Font.PLAIN, fntSize);
    Font fntBold12 = new Font("", Font.BOLD, fntSize);
    Font fntBold17 = new Font("", Font.BOLD, (int) (fntSize * fnt17Scale));
    /*
     * Size, scale
     */
    double xSc = 1;
    double ySc = 1;
    int frmX = 100;
    int frmY = 100;
    public final ConfigGUI confGUI;
    /*
     * Some things
     */
    public final Insets margZero = new Insets(0, 0, 0, 0);
    public final int COLS_OUT_NB = 3;
    public final Color COL_GRAY = new Color(240, 240, 240);
    public final Color COL_WHITE = Color.white;
    public Color chartSerieColor = new Color(0, 0, 0);
    public String chartNameSelected;
    public final Frame frame;
    private final TabPane tabPane;
    private final MenuBar menuBar;
    /*
     *  Listeners
     */

    public GUI(XComponentContext c) {
        try {
            if (System.getProperty("os.name").toLowerCase().contains("linux")) {
                System.setProperty("awt.useSystemAAFontSettings", "on");
            }
        } catch (java.lang.SecurityException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        } catch (java.lang.NullPointerException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        } catch (java.lang.IllegalArgumentException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        }
        contextOOo = c;
        OfficeHelp offH = new OfficeHelp(contextOOo);
        try {
            String lang = offH.getLocaleStr();
            Lng.setLangEnu(LangEnum.valueOf(lang));
        } catch (OOoNullContextException e) {
            Lng.setLangEnu(Lng.LangEnum.en_US);
        }
        //Lng.setLangEnu(LangEnum.fr_FR);
        for (Locale loc : Locale.getAvailableLocales()) {
            if (loc.toString().toLowerCase().equals(Lng.getLangEnu().getStr().
                    toLowerCase())) {
                Locale.setDefault(loc);
                break;
            }
        }
        confGUI = new ConfigGUI(fntSize, xSc, ySc);
        config();
        UIManagerSet.set(fntNorm12, xSc, ySc);
        frame = new Frame(this);
        tabPane = new TabPane(this);
        menuBar = new MenuBar(this);
        frame.setBarAndPane(menuBar, tabPane);
        startState();
        toolTipsAdd();
        scaleFrame();
        addListeners();
        frame.frm.setVisible(true);
    }

    public final void toolTipsAdd() {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DBG_LEV);
        frame.tabPane.tImage.addToolTips();
        frame.tabPane.tDigit.addToolTips();
    }

    public final void toolTipsShow(boolean show) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DBG_LEV);
        frame.tabPane.tImage.toolTipsShow(show);
        frame.tabPane.tDigit.toolTipsShow(show);
    }

    public static void main(String params[]) {
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                GUI gui = new GUI(null);
            }
        });

    }

    public static void removeKeys(JComponent compon) {
        class ClearMap {

            void clear(InputMap m) {
                if (m != null) {
                    m.clear();
                }
                m = m.getParent();
                if (m != null) {
                    m.clear();
                }
            }
        }
        int type = JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT;
        InputMap inMap = compon.getInputMap(type);
        ClearMap clearMap = new ClearMap();
        clearMap.clear(inMap);
        type = JComponent.WHEN_FOCUSED;
        inMap = compon.getInputMap(type);
        clearMap.clear(inMap);
        type = JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT;
        inMap = compon.getInputMap(type);
        clearMap.clear(inMap);
    }

    public void setFntSize(int s) {
        fntSize = s;
        fntNorm12 = new Font("", Font.PLAIN, fntSize);
        fntBold12 = new Font("", Font.BOLD, fntSize);
        fntBold17 = new Font("", Font.BOLD, (int) (fntSize * fnt17Scale));
    }

    public int getFntSize() {
        return fntSize;
    }

    public void setXSc(double s) {
        xSc = s;
        frmX = (int) (frmX * xSc);
    }

    public double getXSc() {
        return xSc;
    }

    public void setYSc(double s) {
        ySc = s;
        frmY = (int) (frmY * ySc);
    }

    public double getYSc() {
        return ySc;
    }

    public void setScaleGrph()
            throws
            ScaleLogException {
        frame.tabPane.tImage.imgPan.getScale().recalcScr();
        frame.tabPane.tImage.imgPan.getScale().setGrph(
                frame.tabPane.tScale.txtXAxisP1XGraph.getValue(),
                (Double) frame.tabPane.tScale.txtXAxisP1YGraph.getValue(),
                frame.tabPane.tScale.txtXAxisP2XGraph.getValue(),
                (Double) frame.tabPane.tScale.txtXAxisP2YGraph.getValue(),
                frame.tabPane.tScale.txtYAxisP1XGraph.getValue(),
                (Double) frame.tabPane.tScale.txtYAxisP1YGraph.getValue(),
                frame.tabPane.tScale.txtYAxisP2XGraph.getValue(),
                (Double) frame.tabPane.tScale.txtYAxisP2YGraph.getValue());
        Point2D ptYAxisGrph = frame.tabPane.tImage.imgPan.getScale().scr2Grph(
                frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr());
        Point2D ptXAxisGrph = frame.tabPane.tImage.imgPan.getScale().scr2Grph(
                frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr());
        DateFormatEnum dFrmtEnu = frame.tabPane.tImage.imgPan.getScale().getDateFrmtEnu();
        String dateStr = null;
        switch (dFrmtEnu) {
            case HHMMSS:
            case YYYYMMDD:
                try {
                    dateStr = DateFormatEnum.formatDateTime(
                            Math.round(ptYAxisGrph.getX()), dFrmtEnu);
                } catch (ParseException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                }
                frame.tabPane.tScale.txtYAxisP1XGraph.setValue(dateStr);
                frame.tabPane.tScale.txtYAxisP2XGraph.setValue(dateStr);
                break;
            default:
                frame.tabPane.tScale.txtYAxisP1XGraph.setValue(
                        ptYAxisGrph.getX());
                frame.tabPane.tScale.txtYAxisP2XGraph.setValue(
                        ptYAxisGrph.getX());
                break;
        }
        frame.tabPane.tScale.txtXAxisP1YGraph.setValue(
                ptXAxisGrph.getY());
        frame.tabPane.tScale.txtXAxisP2YGraph.setValue(
                ptXAxisGrph.getY());
    }

    /*
     * Private
     */
    private void startState() {
        /*
         * Clear keyboard
         */
        removeKeys(frame.tabPane.pane);
        removeKeys(frame.tabPane.pane.getScrl());
        removeKeys(frame.tabPane.tImage.imgPan);
        removeKeys(frame.tabPane.tDigit.table);
        removeKeys(frame.tabPane.tDigit.comboSeries);
        /*
         * Menu
         */
        frame.menuBar.menuScale.menu.setEnabled(false);
        frame.menuBar.menuSeries.menu.setEnabled(false);
        frame.menuBar.menuInterp.menu.setEnabled(false);
        frame.menuBar.menuInterp.itMergePts.setEnabled(false);
        frame.menuBar.menuInterp.itDiscardPts.setEnabled(false);
        frame.menuBar.menuScale.chkClockwise.setEnabled(false);
        frame.menuBar.menuDraw.itPtSize.setText(
                LngMDraw.text(EnUsMDraw.IT_PT_SIZE.name())
                + " [" + frame.tabPane.tImage.imgPan.getPtSize().toString() + "]");
        frame.menuBar.menuDraw.itLineWidth.setText(
                LngMDraw.text(EnUsMDraw.IT_LINE_WIDTH.name())
                + " [" + frame.tabPane.tImage.imgPan.getLineWidth().toString() + "]");
        frame.menuBar.menuInterp.itMorePts.setText(
                LngMInterp.text(EnUsMInterp.IT_MORE_PTS.name())
                + " [" + frame.tabPane.tImage.imgPan.getMorePts().toString() + "]");
        CSV.setMode(ItmsMSeries.RB_SAVE_SERIES_MULTI_COL_CSV);
        Office.setMode(ItmsMSeries.RB_SAVE_SERIES_ONE_SHEET);
        frame.menuBar.menuDraw.chkDrawPts.setSelected(true);
        frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
        frame.menuBar.menuSeries.rbSaveSeriesMultiColCSV.setSelected(true);
        frame.menuBar.menuSeries.rbSaveSeriesOneSheet.setSelected(true);
        frame.menuBar.menuScale.rbCartesian.setSelected(true);
        frame.menuBar.menuScale.rbXNorm.setSelected(true);
        frame.menuBar.menuScale.rbYNorm.setSelected(true);
        frame.menuBar.menuSettings.chkToolTips.setSelected(true);
        frame.tabPane.tImage.butDel.setEnabled(false);
        frame.tabPane.tImage.radButCropImageP1.setSelected(true);
        frame.tabPane.tImage.txtRotateImg.onListAndON(false);
        frame.tabPane.tImage.chkScaleImg.setSelected(true);
        frame.tabPane.tImage.txtScaleImg.onListAndON(true);
        frame.tabPane.tImage.imgPan.scaleCropPoints(xSc, xSc);
        frame.tabPane.tImage.imgPan.resetScaleCartes(xSc, ySc);
        frame.tabPane.tScale.bordOptPolar.setVisible(false);
        frame.tabPane.tScale.txtXAxisP1XGraph.onList(true);
        frame.tabPane.tScale.txtXAxisP2XGraph.onList(false);
        frame.tabPane.tScale.txtYAxisP1YGraph.onList(false);
        frame.tabPane.tScale.txtYAxisP2YGraph.onList(false);
        frame.tabPane.tScale.radButPolar00.setSelected(true);
        frame.tabPane.tScale.radButYAxisP1.setSelected(true);
        frame.tabPane.tScale.radButXAxisP1.setSelected(true);
        frame.tabPane.tDigit.table.getColumnModel().getColumn(0).setHeaderValue(
                LngTDigit.text(EnUsTDigit.TABLE_LAB_X.name()));
        frame.tabPane.tDigit.table.getColumnModel().getColumn(1).setHeaderValue(
                LngTDigit.text(EnUsTDigit.TABLE_LAB_Y.name()));
    }

    private void scaleFrame() {
        int tableColWidth = (frame.tabPane.tDigit.table.getScrollPane().getViewport().getWidth()
                - 1) / 2;
        frame.tabPane.tDigit.table.getColumnModel().getColumn(0).setPreferredWidth(tableColWidth);
        frame.tabPane.tDigit.table.getColumnModel().getColumn(1).setPreferredWidth(tableColWidth);
        frame.tabPane.pane.setSelectedIndex(0);
        frame.tabPane.pane.setPreferredSize(new Dimension(
                /*
                 * w
                 */
                frame.tabPane.tImage.imgPan.getScrollPane().getX()
                + frame.tabPane.tImage.imgPan.getScrollPane().getWidth()
                + frame.tabPane.pane.getScrl().getVerticalScrollBar().getPreferredSize().width
                + frame.frm.getInsets().left + frame.frm.getInsets().right,
                /*
                 * h
                 */
                frame.tabPane.tImage.bordTunePoint.getY()
                + frame.tabPane.tImage.bordTunePoint.getHeight()
                + frame.menuBar.bar.getPreferredSize().height
                + frame.tabPane.pane.getScrl().getHorizontalScrollBar().getPreferredSize().height));

        frame.frm.setBounds(
                (int) (frmX * xSc), (int) (frmY * ySc),
                /*
                 * w
                 */
                frame.tabPane.pane.getPreferredSize().width
                + frame.tabPane.pane.getScrl().getVerticalScrollBar().getPreferredSize().width
                + frame.frm.getInsets().left
                + frame.frm.getInsets().right,
                /*
                 * h
                 */
                frame.frm.getInsets().top
                + frame.tabPane.pane.getPreferredSize().height
                + frame.menuBar.bar.getPreferredSize().height
                + frame.tabPane.pane.getScrl().getHorizontalScrollBar().getPreferredSize().height);
    }

    private void config() {
        try {
            confGUI.checkFileExists();
            confGUI.checkFileRead();
            confGUI.readConfigFile();
            setFntSize(confGUI.getFntSize());
            setXSc(confGUI.getXSc());
            setYSc(confGUI.getYSc());
        } catch (ConfigGUIFileExistsException e) {
            try {
                confGUI.createEmptyFile();
                confGUI.checkFileWrite();
                confGUI.saveConfigFile();
                MsgBox.msgPlainNoTh(LngMSettings.text(
                        EnUsMSettings.MSG_CREATED_CONFG_FILE.name())
                        + "\n" + confGUI.getFilePath(), null,
                        JOptionPane.INFORMATION_MESSAGE);
            } catch (ConfigGUIFileWriteException ex) {
                MsgBox.msgPlainNoTh(LngMSettings.text(
                        EnUsMSettings.MSG_CANT_WRITE_CONFI_FILE.name())
                        + "\n" + confGUI.getFilePath(), null,
                        JOptionPane.INFORMATION_MESSAGE);
            }
        } catch (ConfigGUIFileReadException e) {
            MsgBox.msgPlainNoTh(LngMSettings.text(
                    EnUsMSettings.MSG_CANT_READ_CONFIG_FILE.name())
                    + "\n" + confGUI.getFilePath(), null,
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }

    public Font getFnt() {
        return fntNorm12;
    }

    public Font getFntBld() {
        return fntBold12;
    }

    public Font getFnt17Bld() {
        return fntBold17;
    }

    private void addListeners() {
        frame.addListeners();
        tabPane.addListeners();
        tabPane.tImage.addListeners();
        tabPane.tScale.addListeners();
        tabPane.tDigit.addListeners();
        menuBar.menuAbout.addListeners();
        menuBar.menuDraw.addListeners();
        menuBar.menuImage.addListeners();
        menuBar.menuInterp.addListeners();
        menuBar.menuScale.addListeners();
        menuBar.menuSeries.addListeners();
        menuBar.menuSettings.addListeners();
        frame.addListeners();
    }

    /*
     * Exceptions
     */
    public static class OOoDException extends Exception {

        public OOoDException(String msg) {
            super(msg);
        }
    }
}
