 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui;

import java.awt.Font;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.text.ParseException;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import name.mgplldz.guimy.ctrls.DateFormatEnum;
import name.mgplldz.tracemystack.TraceMyStack;

public class TableMy extends JTable {

    enum TableMyEnum {

        T_DIGIT_TABLE
    }
    private final JScrollPane scrl = new JScrollPane(this);
    private String strFormatDbl = "%13.5e";
    private DateFormatEnum dateFrmtEnu = DateFormatEnum.NO_DATE;
    private final int DGB_LEV = 3;

    public TableMy(int x, int y, int w, int h,
            String name, Font fnt, double xSc, double ySc) {

        super(new DefaultTableModel() {

            @Override
            public boolean isCellEditable(int r, int c) {
                return false;
            }
        });
        scrl.setBounds((int) (x * xSc), (int) (y * ySc), (int) (w * xSc), (int) (h * ySc));
        scrl.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrl.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        ((DefaultTableModel) getModel()).setColumnIdentifiers(
                new Object[]{"col1", "col2"});
        setFont(fnt);
        setName(name);
        getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        getTableHeader().setReorderingAllowed(false);
        setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        //setPreferredScrollableViewportSize(new Dimension(195, 280));
        //setFillsViewportHeight(true);
    }

    public void setDateFrmtEnu(DateFormatEnum dE) {
        dateFrmtEnu = dE;
    }

    public DateFormatEnum getDateFrmtEnu() {
        return dateFrmtEnu;
    }

    public void addRow(Object row[]) {
        ((DefaultTableModel) getModel()).addRow(row);
    }

    public void insertRow2Unique(int idx, Object row[]) {
        if (idx < 0) {
            insertRow(-idx - 1, row);
            scrollRectToVisible(new Rectangle(getCellRect(-idx - 1, 0, true)));
            getSelectionModel().setSelectionInterval(-idx - 1, -idx - 1);
        }
    }

    public void insertRow2(int idx, Object row[]) {
        if (idx < 0) {
            insertRow(-idx - 1, row);
            scrollRectToVisible(new Rectangle(getCellRect(-idx - 1, 0, true)));
            getSelectionModel().setSelectionInterval(-idx - 1, -idx - 1);
        } else {
            insertRow(idx, row);
            scrollRectToVisible(new Rectangle(getCellRect(idx, 0, true)));
            getSelectionModel().setSelectionInterval(idx, idx);
        }
    }

    public void delAllRows() {
        ((DefaultTableModel) getModel()).setRowCount(0);
    }

    public void selectNextPrevRow(int nextPrev) {
        int max = getRowCount();
        int selectedRow = getSelectedRow();
        if (selectedRow < 0) {
            return;
        }
        if (nextPrev > 0) {
            selectedRow =
                    (selectedRow + nextPrev >= max)
                    ? selectedRow
                    : selectedRow + nextPrev;
        } else {
            selectedRow =
                    (selectedRow + nextPrev < 0)
                    ? selectedRow
                    : selectedRow + nextPrev;
        }
        setRowSelectionInterval(selectedRow, selectedRow);
        scrollRectToVisible(
                new Rectangle(getCellRect(selectedRow, 0, true)));
    }

    public void updateRow(int ptIdx, Point2D pt) {
        switch (dateFrmtEnu) {
            case HHMMSS:
            case YYYYMMDD:
                try {
                    getModel().setValueAt(DateFormatEnum.formatDateTime(
                            (long) pt.getX(), dateFrmtEnu), ptIdx, 0);
                } catch (ParseException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DGB_LEV);
                }
                break;
            default:
                getModel().setValueAt(String.format(strFormatDbl,
                        pt.getX()), ptIdx, 0);
                break;
        }
        getModel().setValueAt(String.format(strFormatDbl,
                pt.getY()), ptIdx, 1);
    }

    public void loadSerieIntoTable(Point2D serie[]) {
        delAllRows();
        int rows = serie.length;
        for (int i = 0; i < rows; ++i) {
            Point2D ptGrph = serie[i];
            String outStr = null;
            switch (dateFrmtEnu) {
                case HHMMSS:
                case YYYYMMDD:
                    try {
                        outStr = DateFormatEnum.formatDateTime(
                                Math.round(ptGrph.getX()), dateFrmtEnu);
                    } catch (ParseException ex) {
                        TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                                DGB_LEV);
                    }
                    break;
                default:
                    outStr = String.format(strFormatDbl, ptGrph.getX());
                    break;
            }
            Object row[] = new Object[]{outStr,
                String.format(strFormatDbl, ptGrph.getY())};
            addRow(row);
        }
        if (rows > 0) {
            setRowSelectionInterval(0, 0);
        }
    }

    public String getFormatString() {
        return strFormatDbl;
    }

    public JScrollPane getScrollPane() {
        return scrl;
    }

    public void setColumnName(int colIdx, String name) {
        getColumnModel().getColumn(colIdx).setHeaderValue(name);
        getTableHeader().repaint();
    }
    /*
     * Private
     */

    private void insertRow(int idx, Object row[]) {
        ((DefaultTableModel) getModel()).insertRow(idx, row);
    }
}
