 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.filechoose;

import java.io.File;
import javax.swing.filechooser.*;
import name.mgplldz.oood.gui.filechoose.lang.EnUsFileChoose;
import name.mgplldz.oood.gui.filechoose.lang.LngFileChoose;

public class FileFilterMy extends FileFilter {

    private filtersEnum filterType = null;

    public FileFilterMy(filtersEnum ft) {
        filterType = ft;
    }

    public enum filtersEnum {

        IMG_OPEN,
        PDF_OPEN,
        CSV_SAVE,
        DIRS_SAVE,
        IMG_SAVE;
    }

    enum imgSaveEnum {

        PNG
    }

    enum csvEnum {

        CSV
    }

    enum dirEnum {
    }

    enum imgEnum {

        JPEG,
        JPG,
        GIF,
        PNG
    }

    enum pdfEnum {

        PDF
    }

    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }
        String extension = getExtension(f);
        Enum fileTypes[] = null;
        if (extension != null) {
            switch (filterType) {
                case IMG_OPEN:
                    fileTypes = imgEnum.values();
                    break;
                case CSV_SAVE:
                    fileTypes = csvEnum.values();
                    break;
                case PDF_OPEN:
                    fileTypes = pdfEnum.values();
                    break;
                case IMG_SAVE:
                    fileTypes = imgSaveEnum.values();
            }
            try {
                for (Enum e : fileTypes) {
                    if (extension.equals(e.name())) {
                        return true;
                    }
                }
            } catch (Throwable ignore) {
            }
        }
        return false;
    }

    public String getDescription() {
        StringBuilder descr = new StringBuilder();
        Enum fileTypesArr[] = null;
        switch (filterType) {
            case IMG_OPEN:
                descr.append(LngFileChoose.text(EnUsFileChoose.IMAGES.name()));
                fileTypesArr = imgEnum.values();
                break;
            case CSV_SAVE:
                descr.append(LngFileChoose.text(EnUsFileChoose.TEXT_DATA.name()));
                fileTypesArr = csvEnum.values();
                break;
            case PDF_OPEN:
                descr.append(LngFileChoose.text(EnUsFileChoose.PDF_DOCS.name()));
                fileTypesArr = pdfEnum.values();
                break;
            case DIRS_SAVE:
                descr.append(LngFileChoose.text(EnUsFileChoose.DIRS.name()));
                fileTypesArr = dirEnum.values();
                break;
            case IMG_SAVE:
                descr.append(LngFileChoose.text(EnUsFileChoose.IMAGES.name()));
                fileTypesArr = imgSaveEnum.values();
            default:
                break;
        }
        int rows = fileTypesArr.length;
        for (int i = 0; i < rows; ++i) {
            descr.append(fileTypesArr[i]);
            if (i < rows - 1) {
                descr.append(", ");
            }
        }
        return descr.toString();
    }

    /*
     *  private
     */
    private static String getExtension(File f) {
        String ext = null;
        String fName = f.getName();
        int i = fName.lastIndexOf('.');
        if (i > 0 && i < fName.length() - 1) {
            ext = fName.substring(i + 1).toUpperCase();
        }
        return ext;
    }
}
