 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.filechoose;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFileChooser;

public class ImgFilterPreview extends JComponent
        implements PropertyChangeListener {

    ImageIcon imgPreview = null;
    File file = null;

    public ImgFilterPreview(JFileChooser fc) {
        setPreferredSize(new Dimension(200, 100));
        fc.addPropertyChangeListener(this);
    }

    public void loadImage() {
        if (file == null) {
            imgPreview = null;
            return;
        }
        ImageIcon tmpIcon = new ImageIcon(file.getPath());
        if (tmpIcon != null) {
            if (tmpIcon.getIconWidth() > 200) {
                imgPreview = new ImageIcon(tmpIcon.getImage().
                        getScaledInstance(200, -1, Image.SCALE_SMOOTH));
            } else {
                imgPreview = tmpIcon;
            }
        }
    }

    public void propertyChange(PropertyChangeEvent e) {
        boolean update = false;
        String prop = e.getPropertyName();
        if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(prop)) {
            file = null;
            update = true;
        } else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(prop)) {
            file = (File) e.getNewValue();
            update = true;
        }
        if (update) {
            imgPreview = null;
            if (isShowing()) {
                loadImage();
                repaint();
            }
        }
    }

    @Override
    protected void paintComponent(Graphics g) {
        int marginX = 2;
        int marginY = 2;

        if (imgPreview == null) {
            loadImage();
        }
        if (imgPreview != null) {
            int x = getWidth() / 2 - imgPreview.getIconWidth() / 2;
            int y = getHeight() / 2 - imgPreview.getIconHeight() / 2;
            if (x < marginX) {
                x = marginX;
            }
            if (y < marginY) {
                y = marginY;
            }
            imgPreview.paintIcon(this, g, x, y);
        }
    }
}
