 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.filechoose;

import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import name.mgplldz.guimy.ctrls.MsgBox;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.gui.tab.image.ImgPanel;
import name.mgplldz.oood.gui.filechoose.lang.EnUsFileChoose;
import name.mgplldz.oood.gui.filechoose.lang.LngFileChoose;
import name.mgplldz.oood.gui.tab.image.lang.EnUsTImage;
import name.mgplldz.oood.gui.tab.image.lang.LngTImage;
import name.mgplldz.oood.lang.Lng;
import name.mgplldz.tracemystack.TraceMyStack;

public class LoadImage {

    private GUI gui;
    private final int DBG_LEV = 3;

    public LoadImage(GUI g) {
        gui = g;
    }

    public void load() {
        JFileChooser fc = new JFileChooser();
        FileFilterMy filterPDF = new FileFilterMy(FileFilterMy.filtersEnum.PDF_OPEN);
        FileFilterMy filterImg = new FileFilterMy(FileFilterMy.filtersEnum.IMG_OPEN);
        fc.addChoosableFileFilter(filterImg);
        fc.addChoosableFileFilter(filterPDF);
        fc.setFileFilter(filterImg);
        fc.setAcceptAllFileFilterUsed(false);
        fc.setAccessory(new ImgFilterPreview(fc));
        JFrame frm = (JFrame) SwingUtilities.getRoot(gui.frame.frm);
        boolean onTopFl = frm.isAlwaysOnTop();
        if (onTopFl) {
            frm.setAlwaysOnTop(false);
        }
        int returnVal = fc.showDialog(gui.frame.frm,
                LngFileChoose.text(EnUsFileChoose.LOAD.name()));
        if (onTopFl) {
            frm.setAlwaysOnTop(true);
        }
        File imgFile = null;
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            imgFile = fc.getSelectedFile();
        } else {
            imgFile = null;
        }
        fc.setSelectedFile(null);
        try {
            if (fc.getFileFilter().equals(filterImg)) {
                gui.frame.tabPane.tImage.imgPan.loadFile(
                        imgFile, ImgPanel.ImgPanFileTypeEnum.IMAGE);
                gui.frame.tabPane.tImage.txtPDFPageNumber.setValue("0/0");
            } else if (fc.getFileFilter().equals(filterPDF)) {
                gui.frame.tabPane.tImage.imgPan.loadFile(
                        imgFile, ImgPanel.ImgPanFileTypeEnum.PDF);
                gui.frame.tabPane.tImage.txtPDFPageNumber.setValue(
                        gui.frame.tabPane.tImage.imgPan.getPDFPageIdx() + "/"
                        + gui.frame.tabPane.tImage.imgPan.getPDFDocPagesNb());
            }
        } catch (OOoDException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        }
        imgFile = gui.frame.tabPane.tImage.imgPan.getImgFile();
        String namePath = (imgFile == null)
                ? LngTImage.text(EnUsTImage.TXT_IMG_NAME.name())
                : imgFile.getName();
        gui.frame.tabPane.tImage.txtImgNamePath.setValue(namePath);
        namePath = (imgFile == null)
                ? Lng.text(EnUsTImage.TIP_T_IMG_FULL_PATH.name())
                : imgFile.getPath();
        gui.frame.tabPane.tImage.txtImgNamePath.setToolTipText(namePath);
    }
}
