 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.filechoose;

import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.Component;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.gui.tab.image.ImgPanel;
import name.mgplldz.oood.gui.menu.image.lang.EnUsMImage;
import name.mgplldz.oood.gui.menu.image.lang.LngMImage;
import name.mgplldz.oood.gui.menu.series.lang.EnUsMSeries;
import name.mgplldz.oood.gui.menu.series.lang.LngMSeries;
import name.mgplldz.oood.lang.EnUs;
import name.mgplldz.oood.lang.Lng;
import name.mgplldz.tracemystack.TraceMyStack;

public class SaveImage {

    private ImgPanel imgPan;
    private Component parentForMsgBox;
    private final int DBG_LEV = 3;

    public SaveImage(ImgPanel iP, Component parentFr) {
        imgPan = iP;
        parentForMsgBox = parentFr;

    }

    public void save() throws OOoDException {
        if (imgPan.getBuffImg() == null) {
            throw new OOoDException(LngMSeries.text(
                    EnUsMSeries.MSG_NOTHING_TO_SAVE.name()));
        }
        JFileChooser fileChoos = new JFileChooser();
        FileFilterMy fileFilt = new FileFilterMy(FileFilterMy.filtersEnum.IMG_SAVE);
        int returnVal = -1;
        boolean fileExtOKFl = false;
        String filePathName = null;
        fileChoos.addChoosableFileFilter(fileFilt);
        fileChoos.setAcceptAllFileFilterUsed(false);
        fileChoos.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChoos.setFileFilter(fileFilt);
        JFrame frm = (JFrame) parentForMsgBox;
        boolean onTopFl = frm.isAlwaysOnTop();
        if (onTopFl) {
            frm.setAlwaysOnTop(false);
        }
        returnVal = fileChoos.showSaveDialog(parentForMsgBox);
        if (onTopFl) {
            frm.setAlwaysOnTop(true);
        }
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fileChoos.getSelectedFile();
            fileExtOKFl = file.getPath().toLowerCase().endsWith(".png");
            if (!fileExtOKFl) {
                file = new File(file.getPath() + ".png");
            }
            if (file.exists()) {
                if (new MsgBox().msgAskTh(LngMImage.text(
                        EnUsMImage.MSG_IMG_EXISTS.name()), parentForMsgBox,
                        JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            try {
                filePathName = file.getCanonicalPath();
                ImageIO.write(imgPan.getBuffImg(), "png", file);
            } catch (FileNotFoundException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                throw new OOoDException(
                        Lng.text(EnUs.CANT_WRITE_FILE.name())
                        + "\n" + filePathName);
            } catch (IOException ex1) {
                TraceMyStack.print(ex1.getStackTrace(), ex1.getMessage(), DBG_LEV);
            } catch (NullPointerException ex2) {
                TraceMyStack.print(ex2.getStackTrace(), ex2.getMessage(), DBG_LEV);
                throw new OOoDException(""
                        + Lng.text(EnUs.CANT_WRITE_FILE.name())
                        + "\n" + filePathName);
            }
            MsgBox.msgPlainTh(LngMImage.text(EnUsMImage.MSG_IMAGE_SAVED.name())
                    + "\n" + filePathName,
                    parentForMsgBox, JOptionPane.INFORMATION_MESSAGE);
        }
    }
}
