 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.menu.axes;

import name.mgplldz.guimy.ctrls.DateFormatEnum;
import java.awt.event.ActionEvent;
import java.util.Date;
import javax.swing.JMenuItem;
import name.mgplldz.guimy.listeners.MenuActList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.tab.axes.lang.EnUsTAxes;
import name.mgplldz.oood.gui.tab.axes.lang.LngTAxes;
import name.mgplldz.oood.scale.Scale.ScaleLogException;
import name.mgplldz.oood.scale.Scale.ScaleTypeEnum;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListMenuActMAxes extends MenuActList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListMenuActMAxes(GUI g) {
        gui = g;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        JMenuItem mIt = (JMenuItem) e.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(), mIt.getName(),
                DBG_LEV);
        ItAxes mEn = ItAxes.valueOf(mIt.getName());
        switch (mEn) {

            case RB_X_NORM:
            case RB_X_LOG_10:
            case RB_X_DATE_HHMMSS:
            case RB_X_DATE_YYYYMMDD:
            case RB_Y_NORM:
            case RB_Y_LOG_10:
                axisScaleFormat(mEn);
                break;
            default:
                break;
        }
    }

    /*
     *  private
     */
    private void axisScaleFormat(ItAxes menuItEnu) {
        Date dt = new Date();
        xyLog10();
        switch (menuItEnu) {
            case RB_X_NORM:
                gui.frame.tabPane.tScale.labXAxisX.setText(
                        LngTAxes.text(EnUsTAxes.LAB_X_AXIS_X.name()));
                gui.frame.tabPane.tScale.labYAxisX.setText(
                        LngTAxes.text(EnUsTAxes.LAB_Y_AXIS_X.name()));
                gui.frame.tabPane.tImage.imgPan.getScale().setXScaleTypeEnum(
                        ScaleTypeEnum.PLAIN);
                gui.frame.tabPane.tDigit.table.setDateFrmtEnu(DateFormatEnum.NO_DATE);
                gui.frame.tabPane.tScale.txtXAxisP1XGraph.setValue(new Double(0));
                gui.frame.tabPane.tScale.txtXAxisP2XGraph.setValue(new Double(100));
                gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(new Double(0));
                gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(new Double(100));
                break;
            case RB_X_LOG_10:
                gui.frame.tabPane.tScale.labXAxisX.setText(
                        LngTAxes.text(EnUsTAxes.LAB_X_AXIS_X_LOG10.name()));
                gui.frame.tabPane.tScale.labYAxisX.setText(
                        LngTAxes.text(EnUsTAxes.LAB_Y_AXIS_X_LOG10.name()));
                gui.frame.tabPane.tImage.imgPan.getScale().setXScaleTypeEnum(
                        ScaleTypeEnum.LOG10);
                gui.frame.tabPane.tDigit.table.setDateFrmtEnu(DateFormatEnum.NO_DATE);
                gui.frame.tabPane.tScale.txtXAxisP1XGraph.setValue(new Double(1));
                gui.frame.tabPane.tScale.txtXAxisP2XGraph.setValue(new Double(1000));
                gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(new Double(1));
                gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(new Double(1000));
                break;
            case RB_X_DATE_HHMMSS:
                gui.frame.tabPane.tScale.labXAxisX.setText(LngTAxes.text(
                        EnUsTAxes.LAB_X_AXIS_X_TIME.name()));
                gui.frame.tabPane.tScale.labYAxisX.setText(LngTAxes.text(
                        EnUsTAxes.LAB_Y_AXIS_X_TIME.name()));
                gui.frame.tabPane.tImage.imgPan.getScale().setXScaleTypeEnum(
                        ScaleTypeEnum.TIME);
                gui.frame.tabPane.tDigit.table.setDateFrmtEnu(
                        DateFormatEnum.HHMMSS);
                gui.frame.tabPane.tScale.txtXAxisP1XGraph.setValue(
                        DateFormatEnum.HHMMSS.getDateFormat().format(dt));
                gui.frame.tabPane.tScale.txtXAxisP2XGraph.setValue(
                        DateFormatEnum.HHMMSS.getDateFormat().format(dt));
                gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(
                        DateFormatEnum.HHMMSS.getDateFormat().format(dt));
                gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(
                        DateFormatEnum.HHMMSS.getDateFormat().format(dt));
                break;
            case RB_X_DATE_YYYYMMDD:
                gui.frame.tabPane.tScale.labXAxisX.setText(
                        LngTAxes.text(EnUsTAxes.LAB_X_AXIS_X_DATE.name()));
                gui.frame.tabPane.tScale.labYAxisX.setText(
                        LngTAxes.text(EnUsTAxes.LAB_Y_AXIS_X_DATE.name()));
                gui.frame.tabPane.tImage.imgPan.getScale().setXScaleTypeEnum(
                        ScaleTypeEnum.DATE);
                gui.frame.tabPane.tDigit.table.setDateFrmtEnu(
                        DateFormatEnum.YYYYMMDD);
                gui.frame.tabPane.tScale.txtXAxisP1XGraph.setValue(
                        DateFormatEnum.YYYYMMDD.getDateFormat().format(dt));
                gui.frame.tabPane.tScale.txtXAxisP2XGraph.setValue(
                        DateFormatEnum.YYYYMMDD.getDateFormat().format(dt));
                gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(
                        DateFormatEnum.YYYYMMDD.getDateFormat().format(dt));
                gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(
                        DateFormatEnum.YYYYMMDD.getDateFormat().format(dt));
                break;
            case RB_Y_NORM:
                gui.frame.tabPane.tScale.labXAxisY.setText(
                        LngTAxes.text(EnUsTAxes.LAB_X_AXIS_Y.name()));
                gui.frame.tabPane.tScale.labYAxisY.setText(
                        LngTAxes.text(EnUsTAxes.LAB_Y_AXIS_Y.name()));
                gui.frame.tabPane.tImage.imgPan.getScale().setYScaleTypeEnum(
                        ScaleTypeEnum.PLAIN);
                gui.frame.tabPane.tScale.txtXAxisP1YGraph.setValue(new Double(0));
                gui.frame.tabPane.tScale.txtXAxisP2YGraph.setValue(new Double(100));
                gui.frame.tabPane.tScale.txtYAxisP1YGraph.setValue(new Double(0));
                gui.frame.tabPane.tScale.txtYAxisP2YGraph.setValue(new Double(100));
                break;
            case RB_Y_LOG_10:
                gui.frame.tabPane.tScale.labXAxisY.setText(
                        LngTAxes.text(EnUsTAxes.LAB_X_AXIS_Y_LOG10.name()));
                gui.frame.tabPane.tScale.labYAxisY.setText(
                        LngTAxes.text(EnUsTAxes.LAB_Y_AXIS_Y_LOG10.name()));
                gui.frame.tabPane.tImage.imgPan.getScale().setYScaleTypeEnum(
                        ScaleTypeEnum.LOG10);
                gui.frame.tabPane.tScale.txtXAxisP1YGraph.setValue(new Double(1));
                gui.frame.tabPane.tScale.txtXAxisP2YGraph.setValue(new Double(1000));
                gui.frame.tabPane.tScale.txtYAxisP1YGraph.setValue(new Double(1));
                gui.frame.tabPane.tScale.txtYAxisP2YGraph.setValue(new Double(1000));
                break;
            default:
                break;
        }
        try {
            gui.setScaleGrph();
        } catch (ScaleLogException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                    DBG_LEV);
        }
    }

    private void xyLog10() {
        gui.frame.tabPane.tImage.imgPan.getScale().setXYLog10(
                gui.frame.menuBar.menuScale.rbXLog10.isSelected(),
                gui.frame.menuBar.menuScale.rbYLog10.isSelected());
    }
}
