 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.menu.axes;

import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.event.ItemEvent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.listeners.MenuItList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.menu.series.lang.EnUsMSeries;
import name.mgplldz.oood.gui.menu.series.lang.LngMSeries;
import name.mgplldz.oood.gui.tab.digit.lang.EnUsTDigit;
import name.mgplldz.oood.gui.tab.digit.lang.LngTDigit;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListMenuItMAxes extends MenuItList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListMenuItMAxes(GUI g) {
        gui = g;
    }

    @Override
    public void itemStateChanged(ItemEvent e) {
        JMenuItem source = (JMenuItem) (e.getSource());
        ItAxes menuEnum =
                ItAxes.valueOf(source.getName());
        boolean selectedFl = (e.getStateChange() == ItemEvent.SELECTED) ? true : false;
        TraceMyStack.print(Thread.currentThread().getStackTrace(),
                source.getName() + " selected: " + selectedFl, DBG_LEV);
        switch (menuEnum) {
            case RB_CARTESIAN:
                if (selectedFl) {
                    if (gui.frame.menuBar.menuScale.rbCartesian.getIgnoreItemListIdx()) {
                        break;
                    }
                    if (!cancelChangeCartesianPolar(menuEnum, selectedFl)) {
                        guiStateCartesian(true);
                    }
                }
                break;
            case RB_POLAR:
                if (selectedFl) {
                    if (gui.frame.menuBar.menuScale.rbPolar.getIgnoreItemListIdx()) {
                        break;
                    }
                    if (!cancelChangeCartesianPolar(menuEnum, selectedFl)) {
                        guiStateCartesian(false);
                    }
                }
                break;
            case CHK_CLOCKWISE:
                gui.frame.tabPane.tImage.imgPan.getScale().setChangeDirectionFl(!selectedFl);
                break;
            default:
                break;
        }
    }

    public void menuAlwaysOnTop(boolean onTop) {
        gui.frame.frm.setAlwaysOnTop(onTop);
    }

    public void showToolTips(boolean showFl) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DBG_LEV);
        if (showFl) {
        } else {
        }
    }

    private void delAllSeries() {
        gui.frame.tabPane.tImage.imgPan.setSelectedSeriesIdx(-1);
        gui.frame.tabPane.tImage.imgPan.getSeriesBox().delAllSeries();
        gui.frame.tabPane.tDigit.table.delAllRows();
        gui.frame.tabPane.tDigit.comboSeries.removeAllItems();
    }

    private void guiStateCartesian(boolean cartFl) {
        gui.frame.tabPane.tScale.bordOptCartes.setVisible(cartFl);
        gui.frame.tabPane.tScale.bordOptPolar.setVisible(!cartFl);
        gui.frame.menuBar.menuScale.rbXNorm.setEnabled(cartFl);
        gui.frame.menuBar.menuScale.rbXLog10.setEnabled(cartFl);
        gui.frame.menuBar.menuScale.rbYNorm.setEnabled(cartFl);
        gui.frame.menuBar.menuScale.rbYLog10.setEnabled(cartFl);
        gui.frame.menuBar.menuScale.rbXDateHHMMSS.setEnabled(cartFl);
        gui.frame.menuBar.menuScale.rbXDateYYYYMMDD.setEnabled(cartFl);
        gui.frame.menuBar.menuScale.chkClockwise.setEnabled(!cartFl);
        gui.frame.menuBar.menuSeries.itRemoveDuplicates.setEnabled(cartFl);
        gui.frame.tabPane.tScale.bordXAxis.setVisible(cartFl);
        gui.frame.tabPane.tScale.bordYAxis.setVisible(cartFl);
        gui.frame.tabPane.tScale.bordPolar.setVisible(!cartFl);
        delAllSeries();
        if (cartFl) {
            gui.frame.tabPane.tImage.imgPan.resetScaleCartes(
                    gui.getXSc(), gui.getYSc());
            gui.frame.tabPane.tDigit.table.setColumnName(0, LngTDigit.text(
                    EnUsTDigit.TABLE_LAB_X.name()));
            gui.frame.tabPane.tDigit.table.setColumnName(1, LngTDigit.text(
                    EnUsTDigit.TABLE_LAB_Y.name()));
            gui.frame.menuBar.menuSeries.itXName.setText(LngMSeries.text(
                    EnUsMSeries.IT_X_NAME_CARTES.name()));
            gui.frame.menuBar.menuSeries.itYName.setText(LngMSeries.text(
                    EnUsMSeries.IT_Y_NAME_CARTES.name()));
            gui.frame.menuBar.menuSeries.itXNameAll.setText(LngMSeries.text(
                    EnUsMSeries.IT_X_NAME_ALL_CARTES.name()));
            gui.frame.menuBar.menuSeries.itYNameAll.setText(LngMSeries.text(
                    EnUsMSeries.IT_Y_NAME_ALL_CARTES.name()));
        } else {
            gui.frame.tabPane.tImage.imgPan.resetScalePolar(
                    gui.getXSc(), gui.getYSc());
            gui.frame.tabPane.tDigit.table.setColumnName(0, LngTDigit.text(
                    EnUsTDigit.TABLE_LAB_LENGTH.name()));
            gui.frame.tabPane.tDigit.table.setColumnName(1, LngTDigit.text(
                    EnUsTDigit.TABLE_LAB_DEGREES.name()));
            gui.frame.menuBar.menuSeries.itXName.setText(LngMSeries.text(
                    EnUsMSeries.IT_X_NAME_POLAR.name()));
            gui.frame.menuBar.menuSeries.itYName.setText(LngMSeries.text(
                    EnUsMSeries.IT_Y_NAME_POLAR.name()));
            gui.frame.menuBar.menuSeries.itXNameAll.setText(LngMSeries.text(
                    EnUsMSeries.IT_X_NAME_ALL_POLAR.name()));
            gui.frame.menuBar.menuSeries.itYNameAll.setText(LngMSeries.text(
                    EnUsMSeries.IT_Y_NAME_ALL_POLAR.name()));
        }
        gui.frame.tabPane.tImage.imgPan.repaint();
    }

    private boolean cancelChangeCartesianPolar(ItAxes mEn,
            boolean selected) {
        boolean cancel = false;
        cancel = (new MsgBox().msgAskTh(LngMSeries.text(
                EnUsMSeries.MSG_ALL_SERIES_WILL_BE_DELETED.name()),
                gui.frame.frm, JOptionPane.WARNING_MESSAGE)
                == JOptionPane.YES_OPTION)
                ? false : true;
        switch (mEn) {
            case RB_CARTESIAN:
                if (selected && cancel) {
                    gui.frame.menuBar.menuScale.rbPolar.setIgnoreItemListIdx(1);
                    gui.frame.menuBar.menuScale.rbPolar.setSelected(true);
                } else {
                    gui.frame.tabPane.tScale.radButXAxisP1.doClick();
                }
                break;
            case RB_POLAR:
                if (selected && cancel) {
                    gui.frame.menuBar.menuScale.rbCartesian.setIgnoreItemListIdx(1);
                    gui.frame.menuBar.menuScale.rbCartesian.setSelected(true);
                } else {
                    gui.frame.tabPane.tScale.radButPolar00.doClick();
                }
                break;
            default:
                cancel = true;
                break;
        }
        return cancel;
    }
}
//DefaultFormatterFactory dff =
//new DefaultFormatterFactory(
//new DateFormatter(new SimpleDateFormat("HH:mm:ss")));
//gui.frame.tabPane.tScale.TxtXAxisP1XGraph.setFormatterFactory(dff);

