 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.menu.image;

import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.listeners.MenuActList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.gui.filechoose.LoadImage;
import name.mgplldz.oood.gui.filechoose.SaveImage;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListMenuActMImage extends MenuActList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListMenuActMImage(GUI g) {
        gui = g;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        JMenuItem mIt = (JMenuItem) e.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(), mIt.getName(),
                DBG_LEV);
        ItImage item = ItImage.valueOf(mIt.getName());
        switch (item) {
            case IT_IMG_LOAD:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                new LoadImage(gui).load();
                break;
            case IT_IMG_LOAD_CLIPBOARD:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                loadClipboard();
                break;
            case IT_IMG_RELOAD:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                reloadImg();
                break;
            case IT_IMG_SAVE:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                try {
                    saveImage();
                } catch (OOoDException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                    MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                            JOptionPane.INFORMATION_MESSAGE);
                }
                break;
            default:
                break;
        }
    }

    /*
     *  Private
     */
    private void saveImage() throws OOoDException {
        new SaveImage(gui.frame.tabPane.tImage.imgPan, gui.frame.frm).save();
    }

    private void reloadImg() {
        try {
            gui.frame.tabPane.tImage.imgPan.reloadImage();
        } catch (OOoDException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }

    private void loadClipboard() {
        try {
            gui.frame.frm.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            gui.frame.tabPane.tImage.imgPan.loadImageFromClipboard();
        } catch (OOoDException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        } catch (java.lang.Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        } finally {
            gui.frame.frm.setCursor(null);
        }
    }
}
