 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.menu.interp;

import name.mgplldz.oood.gui.menu.interp.lang.EnUsMInterp;
import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.listeners.MenuActList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.gui.tab.image.ImgPanel.ImgPanelNoSeriesException;
import name.mgplldz.oood.gui.menu.interp.lang.LngMInterp;
import name.mgplldz.oood.gui.tab.image.lang.EnUsTImage;
import name.mgplldz.oood.gui.tab.image.lang.LngTImage;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListMenuActMInterp extends MenuActList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListMenuActMInterp(GUI g) {
        gui = g;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        JMenuItem mIt = (JMenuItem) e.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(), mIt.getName(),
                DBG_LEV);
        ItInterp mEn = ItInterp.valueOf(mIt.getName());
        switch (mEn) {
            case IT_INTERP:
                if (!gui.frame.tabPane.tImage.imgPan.getInterpFl()) {
                    gui.frame.tabPane.tImage.imgPan.setInterpFl(true);
                    interp();
                }
                break;
            case IT_MERGE_PTS:
                if (gui.frame.tabPane.tImage.imgPan.getInterpFl()) {
                    mergeInterp(true);
                    gui.frame.tabPane.tImage.imgPan.setInterpFl(false);
                }
                break;
            case IT_DISCARD_PTS:
                if (gui.frame.tabPane.tImage.imgPan.getInterpFl()) {
                    mergeInterp(false);
                    gui.frame.tabPane.tImage.imgPan.setInterpFl(false);
                }
                break;
            case IT_MORE_PTS:
                morePts(mIt);
                break;
            default:
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        "Wrong menu item.", DBG_LEV);
                break;
        }
    }

    /*
     *  private
     */
    private void morePts(JMenuItem mIt) {
        Integer oldPtsNb = gui.frame.tabPane.tImage.imgPan.getMorePts();
        Integer newPtsNb = new MsgBox().getIntTh(oldPtsNb, LngMInterp.text(
                EnUsMInterp.MSG_MORE_PTS.name()), gui.frame.frm);
        newPtsNb = newPtsNb <= 0 ? 1 : newPtsNb;
        mIt.setText(LngMInterp.text(EnUsMInterp.IT_MORE_PTS.name())
                + "[" + newPtsNb.toString() + "]");
        gui.frame.tabPane.tImage.imgPan.setMorePts(newPtsNb.intValue());
        gui.frame.tabPane.tImage.imgPan.repaint();
    }

    private void interp() {
        int serieIdx = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx();
        if (serieIdx < 0) {
            gui.frame.tabPane.tImage.imgPan.setInterpFl(false);
            return;
        }
        guiStateInterp(true);
        gui.frame.tabPane.tImage.imgPan.setPtsNbBeforeInterp(
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().getPointsCount(
                gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()));
        gui.frame.tabPane.tDigit.table.delAllRows();
        try {
            gui.frame.tabPane.tImage.imgPan.interpolate2(serieIdx);
            MsgBox.msgPlainTh(LngTImage.text(
                    EnUsTImage.MSG_INTERPOLATION_MERGE_DISCARD.name()),
                    gui.frame.frm, JOptionPane.INFORMATION_MESSAGE);
        } catch (OOoDException ex) {
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
            guiStateInterp(false);
            gui.frame.tabPane.tImage.imgPan.setInterpFl(false);
            try {
                gui.frame.tabPane.tDigit.table.loadSerieIntoTable(
                        gui.frame.tabPane.tImage.imgPan.getSelectedSeriesGrph());
            } catch (ImgPanelNoSeriesException ex1) {
                TraceMyStack.print(ex1.getStackTrace(), ex1.getMessage(),
                        DBG_LEV);
            }
        }
    }

    private void mergeInterp(boolean mergeFl) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DBG_LEV);
        int serieIdx = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx();
        gui.frame.tabPane.tImage.imgPan.mergeInterp(serieIdx, mergeFl);
        try {
            gui.frame.tabPane.tDigit.table.loadSerieIntoTable(
                    gui.frame.tabPane.tImage.imgPan.getSelectedSeriesGrph());
        } catch (ImgPanelNoSeriesException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        }
        if (mergeFl) {
            if ((gui.frame.tabPane.tImage.imgPan.getSeriesBox().getPointsCount(
                    gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx())
                    - gui.frame.tabPane.tImage.imgPan.getPtsNbBeforeInterp())
                    == 0) {
                MsgBox.msgPlainTh(LngMInterp.text(
                        EnUsMInterp.MSG_NO_POINTS_ADDED_INTERP.name()),
                        gui.frame.frm, JOptionPane.INFORMATION_MESSAGE);
            }
        }
        guiStateInterp(false);
    }

    private void guiStateInterp(boolean interpFl) {
        gui.frame.tabPane.tDigit.butNextPt.setEnabled(!interpFl);
        gui.frame.tabPane.tDigit.butPrevPt.setEnabled(!interpFl);
        gui.frame.tabPane.tDigit.comboSeries.setEnabled(!interpFl);
        gui.frame.tabPane.tDigit.table.setEnabled(!interpFl);
        gui.frame.tabPane.tImage.butDel.setEnabled(!interpFl);
        gui.frame.tabPane.tImage.butDown.setEnabled(!interpFl);
        gui.frame.tabPane.tImage.butLeft.setEnabled(!interpFl);
        gui.frame.tabPane.tImage.butRight.setEnabled(!interpFl);
        gui.frame.tabPane.tImage.butUp.setEnabled(!interpFl);
        gui.frame.tabPane.pane.setEnabled(!interpFl);
        gui.frame.menuBar.menuSeries.menu.setEnabled(!interpFl);
        gui.frame.menuBar.menuDraw.chkDrawAllSeries.setEnabled(!interpFl);
        gui.frame.menuBar.menuInterp.itInterp.setEnabled(!interpFl);
        gui.frame.menuBar.menuInterp.itMorePts.setEnabled(!interpFl);
        gui.frame.menuBar.menuInterp.itMergePts.setEnabled(interpFl);
        gui.frame.menuBar.menuInterp.itDiscardPts.setEnabled(interpFl);
    }
}
