 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.menu.series;

import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.listeners.MenuActList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.tab.image.ImgPanel.ImgPanelNoSeriesException;
import name.mgplldz.oood.gui.menu.series.lang.EnUsMSeries;
import name.mgplldz.oood.gui.menu.series.lang.LngMSeries;
import name.mgplldz.oood.gui.tab.digit.lang.EnUsTDigit;
import name.mgplldz.oood.gui.tab.digit.lang.LngTDigit;
import name.mgplldz.oood.lang.EnUs;
import name.mgplldz.oood.lang.Lng;
import name.mgplldz.oood.save.CSV;
import name.mgplldz.oood.save.CSV.SaveSeriesHelpDirCreateException;
import name.mgplldz.oood.save.CSV.SaveSeriesHelpDirExistsException;
import name.mgplldz.oood.save.CSV.SaveSeriesHelpFileException;
import name.mgplldz.oood.save.CSV.SaveSeriesHelpNoDataException;
import name.mgplldz.oood.save.Office;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListMenuActMSeries extends MenuActList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListMenuActMSeries(GUI g) {
        gui = g;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        JMenuItem mIt = (JMenuItem) e.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(), mIt.getName(),
                DBG_LEV);
        ItmsMSeries mEn = ItmsMSeries.valueOf(mIt.getName());
        String strTmp = "";
        int intTmp = 0;
        switch (mEn) {
            case IT_ADD_SERIES:
                addSeries();
                break;
            case IT_RENAME_SERIES:
                renameSeries();
                break;
            case IT_DEL_SERIES:
                delSeries();
                break;
            case IT_DEL_ALL_SERIES:
                delAllSeries();
                break;
            case IT_REMOVE_DUPLICATES:
                if (new MsgBox().msgAskTh(LngMSeries.text(
                        EnUsMSeries.MSG_DELETE_DUPLICATES_ASK.name()),
                        gui.frame.frm, JOptionPane.QUESTION_MESSAGE)
                        == JOptionPane.YES_OPTION) {
                    gui.frame.tabPane.tImage.imgPan.removeDuplSelSeries();
                    try {
                        gui.frame.tabPane.tDigit.table.loadSerieIntoTable(
                                gui.frame.tabPane.tImage.imgPan.getSelectedSeriesGrph());
                    } catch (ImgPanelNoSeriesException ex) {
                        TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                                DBG_LEV);
                    }
                }
                break;
            case RB_SAVE_SERIES_MULTI_COL_CSV:
            case RB_SAVE_SERIES_ONE_COL_CSV:
            case RB_SAVE_SERIES_ONE_FILE_PER_SERIES_CSV:
                CSV.setMode(mEn);
                break;
            case RB_SAVE_SERIES_ONE_SHEET:
            case RB_SAVE_SERIES_ONE_SHEET_PER_SERIES:
                Office.setMode(mEn);
                break;
            case IT_SAVE_SERIES_CSV:
                saveSeries();
                break;
            case IT_SAVE_SERIES_OFFICE:
                new Office(gui.contextOOo, gui.frame.tabPane.tImage.imgPan, gui.frame.frm).save();
                break;
            case IT_X_NAME:
                if ((intTmp = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()) < 0) {
                    break;
                }
                strTmp = new MsgBox().getStrTh(
                        gui.frame.tabPane.tImage.imgPan.getSeriesBox().getXName(intTmp),
                        LngMSeries.text(EnUsMSeries.MSG_X_NAME.name()), gui.frame.frm);
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().setXName(intTmp, strTmp);
                gui.frame.tabPane.tDigit.table.setColumnName(0, strTmp);
                break;
            case IT_Y_NAME:
                if ((intTmp = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()) < 0) {
                    break;
                }
                strTmp = new MsgBox().getStrTh(
                        gui.frame.tabPane.tImage.imgPan.getSeriesBox().getYName(intTmp),
                        LngMSeries.text(EnUsMSeries.MSG_Y_NAME.name()), gui.frame.frm);
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().setYName(intTmp, strTmp);
                gui.frame.tabPane.tDigit.table.setColumnName(1, strTmp);
                break;
            case IT_X_NAME_ALL:
                if ((intTmp = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()) < 0) {
                    break;
                }
                strTmp = new MsgBox().getStrTh(
                        gui.frame.tabPane.tImage.imgPan.getSeriesBox().getXName(intTmp),
                        LngMSeries.text(EnUsMSeries.MSG_X_NAME.name()), gui.frame.frm);
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().setXNameAll(strTmp);
                gui.frame.tabPane.tDigit.table.setColumnName(0, strTmp);
                break;
            case IT_Y_NAME_ALL:
                if ((intTmp = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()) < 0) {
                    break;
                }
                strTmp = new MsgBox().getStrTh(
                        gui.frame.tabPane.tImage.imgPan.getSeriesBox().getYName(intTmp),
                        LngMSeries.text(EnUsMSeries.MSG_Y_NAME.name()), gui.frame.frm);
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().setYNameAll(strTmp);
                gui.frame.tabPane.tDigit.table.setColumnName(1, strTmp);
                break;
            default:
                break;
        }
    }

    /*
     *  private
     */
    private void addSeries() {
        String newSerieName = gui.frame.tabPane.tDigit.comboSeries.addEntry();
        if (newSerieName == null) {
            return;
        }
        if (gui.frame.tabPane.tImage.imgPan.getScale().getCartesianModeFl()) {
            gui.frame.tabPane.tImage.imgPan.getSeriesBox().addSeries(
                    newSerieName,
                    LngTDigit.text(EnUsTDigit.TABLE_LAB_X.name()),
                    LngTDigit.text(EnUsTDigit.TABLE_LAB_Y.name()));
        } else {
            gui.frame.tabPane.tImage.imgPan.getSeriesBox().addSeries(
                    newSerieName,
                    LngTDigit.text(EnUsTDigit.TABLE_LAB_LENGTH.name()),
                    LngTDigit.text(EnUsTDigit.TABLE_LAB_DEGREES.name()));
        }
        gui.frame.tabPane.tImage.imgPan.setSelectedSeriesIdx(
                gui.frame.tabPane.tDigit.comboSeries.getSelectedIndex());
        gui.frame.tabPane.tDigit.table.delAllRows();
        gui.frame.tabPane.tDigit.table.setColumnName(0,
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().getXName(
                gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()));
        gui.frame.tabPane.tDigit.table.setColumnName(1,
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().getYName(
                gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()));
        gui.frame.tabPane.tImage.imgPan.repaint();
    }

    private void delSeries() {
        int seriesIdx = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx();
        if (seriesIdx < 0) {
            return;
        }
        if (new MsgBox().msgAskTh(LngMSeries.text(
                EnUsMSeries.MSG_DEL_SERIES.name()), gui.frame.frm,
                JOptionPane.WARNING_MESSAGE) != JOptionPane.YES_OPTION) {
            return;
        }
        gui.frame.tabPane.tDigit.table.delAllRows();
        gui.frame.tabPane.tDigit.comboSeries.delEntryByIdx(seriesIdx);
        gui.frame.tabPane.tImage.imgPan.getSeriesBox().delByIdx(seriesIdx);
        if (gui.frame.tabPane.tImage.imgPan.getSeriesBox().getSeriesCount()
                > 0) {
            gui.frame.tabPane.tDigit.comboSeries.setSelectedIndex(0);
            gui.frame.tabPane.tImage.imgPan.setSelectedSeriesIdx(0);
        } else {
            gui.frame.tabPane.tImage.imgPan.setSelectedSeriesIdx(-1);
            gui.frame.tabPane.tDigit.txtPtsNb.setValue("0/0");
        }
    }

    private void delAllSeries() {
        int seriesNb =
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().getSeriesCount();
        if (seriesNb <= 0) {
            return;
        }
        if (new MsgBox().msgAskTh(LngMSeries.text(
                EnUsMSeries.MSG_DEL_ALL_SERIES.name()), gui.frame.frm,
                JOptionPane.WARNING_MESSAGE) != JOptionPane.YES_OPTION) {
            return;
        }
        gui.frame.tabPane.tImage.imgPan.setSelectedSeriesIdx(-1);
        gui.frame.tabPane.tImage.imgPan.getSeriesBox().delAllSeries();
        gui.frame.tabPane.tDigit.table.delAllRows();
        gui.frame.tabPane.tDigit.comboSeries.removeAllItems();
    }

    private void renameSeries() {
        int serieIdx = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx();
        if (serieIdx < 0) {
            return;
        }
        String newSerieName = gui.frame.tabPane.tDigit.comboSeries.renameEntry(serieIdx);
        if (newSerieName != null) {
            gui.frame.tabPane.tImage.imgPan.getSeriesBox().setSeriesName(
                    serieIdx, newSerieName);
            gui.frame.tabPane.tImage.imgPan.repaint();
        }
    }

    private void saveSeries() {
        try {
            new CSV(gui.frame.tabPane.tImage.imgPan, gui.frame.frm).save();
        } catch (SaveSeriesHelpNoDataException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(LngMSeries.text(
                    EnUsMSeries.MSG_NOTHING_TO_SAVE.name()),
                    gui.frame.frm, JOptionPane.INFORMATION_MESSAGE);
        } catch (SaveSeriesHelpDirCreateException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(
                    Lng.text(EnUs.CANT_CREATE_DIDR.name())
                    + "\n" + ex.getMessage(), gui.frame.frm, JOptionPane.ERROR_MESSAGE);
        } catch (SaveSeriesHelpFileException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(
                    Lng.text(EnUs.CANT_WRITE_FILE.name())
                    + "\n" + ex.getMessage(), gui.frame.frm, JOptionPane.ERROR_MESSAGE);
        } catch (SaveSeriesHelpDirExistsException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(
                    Lng.text(EnUs.DIR_DOESNT_EXISTS.name())
                    + "\n" + ex.getMessage(), gui.frame.frm, JOptionPane.ERROR_MESSAGE);
        }
    }
}
