 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab;

import name.mgplldz.guimy.ctrls.MsgBox;
import name.mgplldz.guimy.ctrls.DateFormatEnum.*;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import name.mgplldz.guimy.ctrls.TabPaneMy;
import name.mgplldz.guimy.listeners.TabPaneChngList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.tab.image.ImgPanel;
import name.mgplldz.oood.gui.tab.image.ImgPanel.ImgPanelNoSeriesException;
import name.mgplldz.oood.gui.tab.axes.lang.EnUsTAxes;
import name.mgplldz.oood.gui.tab.axes.lang.LngTAxes;
import name.mgplldz.oood.scale.Scale.ScaleLogException;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListTabPaneChng extends TabPaneChngList {

    private GUI gui;
    private final int DBG_LEV = 3;

    public ListTabPaneChng(GUI g) {
        gui = g;
    }

    @Override
    public void stateChanged(ChangeEvent e) {
        TabPaneMy tPane = (TabPaneMy) e.getSource();
        if (!tPane.getIgnoreTabChngIdx()) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    tPane.getName() + ": ignore", DBG_LEV);
        } else {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    tPane.getName(),
                    DBG_LEV);
        }
        int tabIdx = tPane.getSelectedIndex();
        switch (tabIdx) {
            case 0:
                gui.frame.tabPane.tImage.imgPan.setTabEnum(ImgPanel.ImgPanTabsEnum.IMAGE);
                gui.frame.tabPane.tImage.butDel.setEnabled(false);
                gui.frame.menuBar.menuImage.menu.setEnabled(true);
                gui.frame.menuBar.menuScale.menu.setEnabled(false);
                gui.frame.menuBar.menuSeries.menu.setEnabled(false);
                gui.frame.menuBar.menuInterp.menu.setEnabled(false);
                break;
            case 1:
                gui.frame.tabPane.tImage.imgPan.setTabEnum(ImgPanel.ImgPanTabsEnum.SCALE);
                gui.frame.tabPane.tImage.butDel.setEnabled(false);
                gui.frame.menuBar.menuImage.menu.setEnabled(false);
                gui.frame.menuBar.menuScale.menu.setEnabled(true);
                gui.frame.menuBar.menuSeries.menu.setEnabled(false);
                gui.frame.menuBar.menuInterp.menu.setEnabled(false);
                gui.frame.menuBar.menuDraw.menu.setEnabled(true);
                try {
                    gui.setScaleGrph();
                } catch (ScaleLogException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                }
                break;
            case 2:
                gui.frame.tabPane.tImage.imgPan.setTabEnum(
                        ImgPanel.ImgPanTabsEnum.DIGITIZE);
                gui.frame.tabPane.tImage.butDel.setEnabled(true);
                gui.frame.tabPane.tDigit.txtPtsNb.setValue(
                        (gui.frame.tabPane.tImage.imgPan.getSelPointIdx() + 1)
                        + "/"
                        + gui.frame.tabPane.tImage.imgPan.getSeriesBox().getPointsCount(
                        gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx()));
                gui.frame.menuBar.menuImage.menu.setEnabled(false);
                gui.frame.menuBar.menuScale.menu.setEnabled(false);
                gui.frame.menuBar.menuSeries.menu.setEnabled(true);
                gui.frame.menuBar.menuInterp.menu.setEnabled(
                        gui.frame.menuBar.menuScale.rbCartesian.isSelected());
                gui.frame.menuBar.menuDraw.menu.setEnabled(true);
                setScaleGrph(tPane);
                break;
        }
        gui.frame.tabPane.tImage.tab.remove(gui.frame.tabPane.tImage.imgPan.getScrollPane());
        gui.frame.tabPane.tScale.tab.remove(gui.frame.tabPane.tImage.imgPan.getScrollPane());
        gui.frame.tabPane.tDigit.tab.remove(gui.frame.tabPane.tImage.imgPan.getScrollPane());
        ((JPanel) tPane.getSelectedComponent()).add(
                gui.frame.tabPane.tImage.imgPan.getScrollPane());
        gui.frame.tabPane.tImage.tab.remove(gui.frame.tabPane.tImage.bordTunePoint);
        gui.frame.tabPane.tScale.tab.remove(gui.frame.tabPane.tImage.bordTunePoint);
        gui.frame.tabPane.tDigit.tab.remove(gui.frame.tabPane.tImage.bordTunePoint);
        ((JPanel) tPane.getSelectedComponent()).add(
                gui.frame.tabPane.tImage.bordTunePoint);
    }

    private void setScaleGrph(TabPaneMy tPane) {
        /*
         * Cartesian
         */
        if (gui.frame.menuBar.menuScale.rbCartesian.isSelected()) {
            gui.frame.tabPane.tImage.imgPan.getScale().recalcScr();
            try {
                gui.frame.tabPane.tImage.imgPan.getScale().setGrph(
                        gui.frame.tabPane.tScale.txtXAxisP1XGraph.getValue(),
                        (Double) gui.frame.tabPane.tScale.txtXAxisP1YGraph.getValue(),
                        gui.frame.tabPane.tScale.txtXAxisP2XGraph.getValue(),
                        (Double) gui.frame.tabPane.tScale.txtXAxisP2YGraph.getValue(),
                        gui.frame.tabPane.tScale.txtYAxisP1XGraph.getValue(),
                        (Double) gui.frame.tabPane.tScale.txtYAxisP1YGraph.getValue(),
                        gui.frame.tabPane.tScale.txtYAxisP2XGraph.getValue(),
                        (Double) gui.frame.tabPane.tScale.txtYAxisP2YGraph.getValue());
            } catch (ScaleLogException ex) {
                tPane.setIgnoreTabChngIdx(1);
                gui.frame.tabPane.pane.setSelectedIndex(1);
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        ex.getMessage(), DBG_LEV);
                MsgBox.msgPlainTh(LngTAxes.text(
                        EnUsTAxes.MSG_LOG_10_GREATER_ZERO.name()), gui.frame.frm,
                        JOptionPane.INFORMATION_MESSAGE);
            }
        }
        /*
         * Polar
         */
        if (gui.frame.menuBar.menuScale.rbPolar.isSelected()) {
            gui.frame.tabPane.tImage.imgPan.getScale().recalcScr();
            try {
                gui.frame.tabPane.tImage.imgPan.getScale().setGrph(
                        gui.frame.tabPane.tScale.txtPolarLength.getValue(),
                        (Double) gui.frame.tabPane.tScale.txtPolarDegrees.getValue(),
                        null, null,
                        null, null,
                        null, null);
                gui.frame.tabPane.tImage.imgPan.getSeriesBox().recalcSeries();
                gui.frame.tabPane.tDigit.table.loadSerieIntoTable(
                        gui.frame.tabPane.tImage.imgPan.getSelectedSeriesGrph());
            } catch (ScaleLogException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            } catch (ImgPanelNoSeriesException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            }
        }
        gui.frame.tabPane.tImage.imgPan.getSeriesBox().recalcSeries();
        try {
            gui.frame.tabPane.tDigit.table.loadSerieIntoTable(
                    gui.frame.tabPane.tImage.imgPan.getSelectedSeriesGrph());
        } catch (ImgPanelNoSeriesException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        }
    }
}
