 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.axes;

import java.awt.Point;
import java.awt.event.ItemEvent;
import name.mgplldz.guimy.ctrls.ChkBoxMy;
import name.mgplldz.guimy.listeners.ChkBoxItemList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.tab.axes.CtrlTAxes.*;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListChkBoxItemTAxes extends ChkBoxItemList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListChkBoxItemTAxes(GUI g) {
        gui = g;
    }

    @Override
    public void itemStateChanged(ItemEvent ev) {
        ChkBoxMy chkBox = (ChkBoxMy) ev.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(), chkBox.getName(),
                DBG_LEV);
        if (chkBox.getIgnoreItemListIdx()) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(), chkBox.getName()
                    + ": ignores item listener.", DBG_LEV);
            return;
        }
        boolean selectedFl =
                (ev.getStateChange() == ItemEvent.SELECTED) ? true : false;
        CtrlTAxes ctrl = CtrlTAxes.valueOf(chkBox.getName());
        switch (ctrl) {
            case CHK_P1_EQUALS_P1:
            case CHK_X_HORIZONTAL:
            case CHK_Y_VERTICAL:
                if (selectedFl) {
                    scaleCartesianOpt();
                    gui.frame.tabPane.tImage.imgPan.repaint();
                }
                break;
            case CHK_RADIUS_HORIZONTAL:
                int length = (int) Point.distance(
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x,
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y,
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x,
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y);
                if (selectedFl) {
                    gui.frame.tabPane.tScale.chkRadiusVertical.setSelected(false);
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x =
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x + length;
                    gui.frame.tabPane.tImage.imgPan.repaint();
                }
                break;
            case CHK_RADIUS_VERTICAL:
                length = (int) Point.distance(
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x,
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y,
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x,
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y);
                if (selectedFl) {
                    gui.frame.tabPane.tScale.chkRadiusHorizontal.setSelected(false);
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x =
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x;
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y
                            - length;
                    gui.frame.tabPane.tImage.imgPan.repaint();
                }
                break;
            default:
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        "Wrong check-box.", DBG_LEV);
                break;
        }
    }

    /*
     *  Private
     */
//    private void scalePolarOpt() {
//        if (gui.frame.tabPane.tScale.chkRadiusHorizontal.isSelected()) {
//            gui.frame.tabPane.tScale.chkRadiusVertical.setSelected(false);
//            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
//                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
//            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x =
//                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y;
//        }
//        if (gui.frame.tabPane.tScale.chkRadiusVertical.isSelected()) {
//            gui.frame.tabPane.tScale.chkRadiusHorizontal.setSelected(false);
//            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x =
//                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x;
//            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
//                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x;
//        }
//    }

    private void scaleCartesianOpt() {
        if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
            gui.frame.tabPane.tScale.txtXAxisP2YGraph.setValue(
                    gui.frame.tabPane.tScale.txtXAxisP1YGraph.getValue());
        }
        if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x =
                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
            gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(
                    gui.frame.tabPane.tScale.txtYAxisP1XGraph.getValue());
        }
        if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x =
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x;
            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().y =
                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
            gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(
                    gui.frame.tabPane.tScale.txtXAxisP1XGraph.getValue());
            gui.frame.tabPane.tScale.txtYAxisP1YGraph.setValue(
                    gui.frame.tabPane.tScale.txtXAxisP1YGraph.getValue());
            if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) { // redundant but independet of order
                gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                gui.frame.tabPane.tScale.txtXAxisP2YGraph.setValue(
                        gui.frame.tabPane.tScale.txtXAxisP1YGraph.getValue());
            }
            if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {// redundant but independet of order
                gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x =
                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
                gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(
                        gui.frame.tabPane.tScale.txtYAxisP1XGraph.getValue());
            }
        }
    }
}
