 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Godard
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.axes;

import name.mgplldz.guimy.ctrls.DateFormatEnum;
import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.event.MouseEvent;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.ctrls.TxtFldMy;
import name.mgplldz.guimy.listeners.TxtFldMouseList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.tab.axes.CtrlTAxes.*;
import name.mgplldz.oood.gui.tab.axes.lang.EnUsTAxes;
import name.mgplldz.oood.gui.tab.axes.lang.LngTAxes;
import name.mgplldz.oood.scale.Scale.ScaleLogException;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListTxtFldMouseTAxes extends TxtFldMouseList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListTxtFldMouseTAxes(GUI g) {
        gui = g;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        TxtFldMy fld = (TxtFldMy) e.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(),
                fld.getName(), DBG_LEV);
        if (fld.getIgnoreMouseListFl()) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    fld.getName() + ": ignores mouse listener.", DBG_LEV);
            return;
        }
        CtrlTAxes ctrl = CtrlTAxes.valueOf(fld.getName());
        Double valDbl;
        try {
            switch (ctrl) {
                case X_AXIS_P1_X_GRAPH:
                    scaleAxisValues(fld, ctrl, true,
                            LngTAxes.text(EnUsTAxes.MSG_CARTES_X_AXIS_P1_X.name()));
                    gui.setScaleGrph();
                    break;
                case X_AXIS_P1_Y_GRAPH:
                    break;
                case X_AXIS_P2_X_GRAPH:
                    scaleAxisValues(fld, ctrl, true,
                            LngTAxes.text(EnUsTAxes.MSG_CARTES_X_AXIS_P2_X.name()));
                    gui.setScaleGrph();
                    break;
                case Y_AXIS_P1_Y_GRAPH:
                    scaleAxisValues(fld, ctrl, false,
                            LngTAxes.text(EnUsTAxes.MSG_CARTES_Y_AXIS_P1_Y.name()));
                    gui.setScaleGrph();
                    break;
                case Y_AXIS_P2_Y_GRAPH:
                    scaleAxisValues(fld, ctrl, false,
                            LngTAxes.text(EnUsTAxes.MSG_CARTES_Y_AXIS_P2_Y.name()));
                    gui.setScaleGrph();
                    break;
                case R_AXIS_LENGTH:
                    valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                            LngTAxes.text(EnUsTAxes.MSG_POLAR_LENGTH.name()),
                            gui.frame.frm);
                    if (valDbl == null) {
                        return;
                    }
                    fld.setValue(valDbl);
                    break;
                case R_AXIS_DEGREES:
                    valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                            LngTAxes.text(EnUsTAxes.MSG_POLAR_DEGREES.name()),
                            gui.frame.frm);
                    if (valDbl == null) {
                        return;
                    }
                    valDbl = ((valDbl % 360) + 360) % 360;
                    fld.setValue(valDbl);
                    break;
                default:
                    TraceMyStack.print(Thread.currentThread().getStackTrace(),
                            "Wrong text field.", DBG_LEV);
                    break;
            }
        } catch (ScaleLogException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(LngTAxes.text(
                    EnUsTAxes.MSG_LOG_10_GREATER_ZERO.name()), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }

    @Override
    public void mousePressed(MouseEvent e) {
        return;
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        return;
    }

    @Override
    public void mouseEntered(MouseEvent e) {
        return;
    }

    @Override
    public void mouseExited(MouseEvent e) {
        return;
    }

    /*
     *  Private
     */
    private void txtFldScaleHorizVertEq(Object val,
            CtrlTAxes ctrl) {
        switch (ctrl) {
            case X_AXIS_P1_X_GRAPH:
                if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                    gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(val);
                    if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                        gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(val);
                    }
                }
                break;
            case X_AXIS_P1_Y_GRAPH:
                if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                    gui.frame.tabPane.tScale.txtXAxisP2YGraph.setValue(val);
                }
                if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                    gui.frame.tabPane.tScale.txtYAxisP1YGraph.setValue(val);
                }
                break;
            case X_AXIS_P2_X_GRAPH:
                break;
            case X_AXIS_P2_Y_GRAPH:
                if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                    gui.frame.tabPane.tScale.txtXAxisP1YGraph.setValue(val);
                    if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                        gui.frame.tabPane.tScale.txtYAxisP1YGraph.setValue(val);
                    }
                }
                break;
            case Y_AXIS_P1_X_GRAPH:
                if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                    gui.frame.tabPane.tScale.txtYAxisP2XGraph.setValue(val);
                }
                if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                    gui.frame.tabPane.tScale.txtXAxisP1XGraph.setValue(val);
                }
                break;
            case Y_AXIS_P1_Y_GRAPH:
                if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                    gui.frame.tabPane.tScale.txtXAxisP1YGraph.setValue(val);
                    if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                        gui.frame.tabPane.tScale.txtXAxisP2YGraph.setValue(val);
                    }
                }
                break;
            case Y_AXIS_P2_X_GRAPH:
                if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                    gui.frame.tabPane.tScale.txtYAxisP1XGraph.setValue(val);
                    if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                        gui.frame.tabPane.tScale.txtXAxisP1XGraph.setValue(val);
                    }
                }
                break;
            case Y_AXIS_P2_Y_GRAPH:
                break;
            default:
                break;
        }
    }

    private void scaleAxisValues(TxtFldMy fld, CtrlTAxes ctrl,
            boolean xCoord, String msgInfo) {
        String valStr;
        Double valDbl;
        DateFormatEnum dFrmtEnu =
                gui.frame.tabPane.tImage.imgPan.getScale().getDateFrmtEnu();
        switch (dFrmtEnu) {
            case HHMMSS:
                if (xCoord) {
                    msgInfo += (" "
                            + LngTAxes.text(EnUsTAxes.MSG_CARTES_X_TIME.name()));
                    valStr = (String) new MsgBox().getDateStrTh(
                            (String) fld.getValue(), dFrmtEnu, msgInfo,
                            gui.frame.frm);
                    if (valStr == null) {
                        break;
                    }
                    fld.setValue(valStr);
                    txtFldScaleHorizVertEq(valStr, ctrl);
                } else {
                    valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                            msgInfo, gui.frame.frm);
                    if (valDbl == null) {
                        break;
                    }
                    fld.setValue(valDbl);
                    txtFldScaleHorizVertEq(valDbl, ctrl);
                }
                break;
            case YYYYMMDD:
                if (xCoord) {
                    msgInfo += (" "
                            + LngTAxes.text(EnUsTAxes.MSG_CARTES_X_DATE.name()));
                    valStr = (String) new MsgBox().getDateStrTh((String) fld.getValue(),
                            dFrmtEnu, msgInfo, gui.frame.frm);
                    if (valStr == null) {
                        break;
                    }
                    fld.setValue(valStr);
                    txtFldScaleHorizVertEq(valStr, ctrl);
                } else {
                    valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                            msgInfo, gui.frame.frm);
                    if (valDbl == null) {
                        break;
                    }
                    fld.setValue(valDbl);
                    txtFldScaleHorizVertEq(valDbl, ctrl);
                }
                break;
            default:
                valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                        msgInfo, gui.frame.frm);
                if (valDbl == null) {
                    break;
                }
                fld.setValue(valDbl);
                txtFldScaleHorizVertEq(valDbl, ctrl);
                break;
        }
    }
    /*
     * Test
     */

    public static void main(String args[]) {
    }
}
