 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.image;

import java.util.Arrays;
import name.mgplldz.guimy.ctrls.MousePointerMy;
import name.mgplldz.guimy.ctrls.MsgBox;
import com.sun.pdfview.PDFFile;
import com.sun.pdfview.PDFPage;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import name.mgplldz.guimy.ctrls.G2DQual;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.gui.tab.image.SeriesBox.PointOOoD;
import name.mgplldz.oood.gui.tab.image.SeriesBox.Series;
import name.mgplldz.oood.gui.menu.interp.lang.EnUsMInterp;
import name.mgplldz.oood.gui.menu.interp.lang.LngMInterp;
import name.mgplldz.oood.gui.tab.image.lang.EnUsTImage;
import name.mgplldz.oood.gui.tab.image.lang.LngTImage;
import name.mgplldz.oood.scale.Scale;
import name.mgplldz.spline.Spline;
import name.mgplldz.spline.Spline.SplineExceptionDuplicates;
import name.mgplldz.spline.Spline.SplineExceptionMorePoints;
import name.mgplldz.tracemystack.TraceMyStack;

public class ImgPanel extends JPanel {

    public enum ImgPanFileTypeEnum {

        IMAGE,
        PDF
    }

    public enum ImgPanScalePointsEnum {

        X_AXIS_P1,
        X_AXIS_P2,
        Y_AXIS_P1,
        Y_AXIS_P2,
        POLAR_0_0,
        POLAR_X_Y,
    }

    public enum ImgPanCropPointsEnum {

        CROP_IMG_P1,
        CROP_IMG_P2
    }

    public enum ImgPanTabsEnum {

        IMAGE,
        SCALE,
        DIGITIZE
    }
    private File imgFile;
    private BufferedImage buffImgOrig;
    private BufferedImage buffImg;
    private final SeriesBox seriesBox = new SeriesBox(this);
    private final Series interpSeries =
            new SeriesBox(this).new Series("Interpolation data", "Y", "Y");
    private final Scale scale;
    private int selectedSeriesIdx = -1;
    private int ptSize = 5;
    private int ptShift = setPtShift(ptSize);
    private int morePts = 5;
    private int pdfPageIdx = 0;
    private int pdfDocPagesNb = 0;
    private int selPointInASeriesIdx = -1;
    private final BasicStroke ptStroke = new BasicStroke(0);
    private BasicStroke lineStroke =
            new BasicStroke(3, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
    private BasicStroke lineCropStroke =
            new BasicStroke(3, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
    private Color ptCol = Color.green;
    private Color ptColSel = Color.red;
    private Color ptColAnotherSeries = Color.lightGray;
    private Color lineCol = Color.orange;
    private Color lineScaleCol = Color.cyan.darker();
    private Color lineColAnotherSeries = Color.lightGray;
    private Color lineCropCol = Color.green;
    private PDFPage pdfPage = null;
    private PDFFile pdfFile = null;
    private double imageScale = 1.0;
    private double imageRotateDegrees = 0.0;
    private Point cropPointP1 = new Point(100, 100);
    private Point cropPointP2 = new Point(200, 200);
    private boolean pdfFl = false;
    private boolean pdfCropFl = false;
    private boolean interpFl = false;
    private Point2D imgFitSize = null;
    private MousePointerMy pointers;
    private int pointerIdx = 0;
    private ImgPanTabsEnum tabEnu = ImgPanTabsEnum.IMAGE;
    private boolean drawScaleFl = false;
    private boolean drawPointsFl = true;
    private boolean drawLinesFl = false;
    private boolean drawAllSeriesFl = false;
    private boolean drawImageFl = true;
    private ImgPanScalePointsEnum scalePointEnum = ImgPanScalePointsEnum.X_AXIS_P1;
    private ImgPanCropPointsEnum cropPointEnum = ImgPanCropPointsEnum.CROP_IMG_P1;
    private final JScrollPane scrl = new JScrollPane(this);
    private int ptsNbBeforeInterp;
    private final int DBG_LEV = 3;

    public ImgPanel(int x, int y, int w, int h, double xSc, double ySc) {
        setPreferredSize(new Dimension(1, 1));
        pointers = MousePointerMy.getInstance();
        setPointerAndIcoByIdx(0);
        scrl.setHorizontalScrollBarPolicy(
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        scrl.setVerticalScrollBarPolicy(
                ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scrl.setBounds((int) (x * xSc), (int) (y * ySc), (int) (w * xSc), (int) (h * ySc));
        scale = new Scale();
        scale.scaleCartes(new Point(1, 10), new Point(20, 10),
                new Point(10, 1), new Point(10, 20));
    }

    @Override
    public void paintComponent(Graphics g) {
        grabFocus();
        Graphics2D g2d = (Graphics2D) g;
        G2DQual.q(g2d, 1);
        super.paintComponent(g2d);
        g2d.setColor(new Color(240, 240, 240));
        g2d.fillRect(0, 0, this.getWidth(), this.getHeight());
        if (drawImageFl) {
            drawImg(buffImg, g2d);
        }
        switch (tabEnu) {
            case IMAGE:
                drawCropRect(g2d);
                break;
            case SCALE:
                drawScale(g2d);
                break;
            case DIGITIZE:
                if (drawScaleFl) {
                    drawScale(g2d);
                }
                if (!interpFl) {
                    if (drawLinesFl) {
                        drawLines(g2d);
                    }
                    if (drawPointsFl) {
                        drawPoints(g2d);
                    }
                } else {
                    drawInterpolation(g2d);
                }
                break;
            default:
                break;
        }
    }

    /*
     * Public
     */
    public File getImgFile() {
        return imgFile;
    }

    public Scale getScale() {
        return scale;
    }

    public void setSelPointIdx(int idx) {
        selPointInASeriesIdx = idx;
        repaint();
    }

    public int getSelPointIdx() {
        return selPointInASeriesIdx;
    }

    public void setSelectedSeriesIdx(int idx) {
        selectedSeriesIdx = idx;
        String msg = "Selected series [" + selectedSeriesIdx + "]";
        TraceMyStack.print(Thread.currentThread().getStackTrace(), msg, DBG_LEV);
    }

    public int getSelectedSerieIdx() {
        return selectedSeriesIdx;
    }

    public Point2D[] getSelectedSeriesGrph()
            throws
            ImgPanelNoSeriesException {
        if (selectedSeriesIdx < 0) {
            throw new ImgPanelNoSeriesException();
        }
        int rows = seriesBox.getPointsCount(selectedSeriesIdx);
        Point2D out[] = new Point2D[rows];
        for (int i = 0; i < rows; ++i) {
            out[i] = new Point.Double(
                    seriesBox.getGraph(selectedSeriesIdx, i).getX(),
                    seriesBox.getGraph(selectedSeriesIdx, i).getY());
        }
        return out;
    }

    public JScrollPane getScrollPane() {
        return scrl;
    }

    public void setFitSize(Point2D size) throws OOoDException {
        if (size == null) {
            imgFitSize = null;
        } else {
            imgFitSize = new Point.Double(size.getX(), size.getY());
        }
        imgScaleRotateAndShow();
    }

    public void changePdfPageIdx(int delta) throws OOoDException {
        pdfCropFl = false;
        if (pdfDocPagesNb == 0) {
            pdfPageIdx = 0;
            return;
        } else if (delta < 0 && pdfPageIdx <= 1) {
            pdfPageIdx = 1;
        } else if (delta > 0 && pdfPageIdx >= pdfDocPagesNb) {
            pdfPageIdx = pdfDocPagesNb;
        } else {
            pdfPageIdx += delta;
        }
        pdfPage = pdfFile.getPage(pdfPageIdx);
        imgScaleRotateAndShow();
    }

    public int getPDFPageIdx() {
        return pdfPageIdx;
    }

    public int getPDFDocPagesNb() {
        return pdfDocPagesNb;
    }

    public void cropImage()
            throws
            OOoDException {
        if (buffImg == null) {
            return;
        }
        int xMin = cropPointP1.x < cropPointP2.x ? cropPointP1.x : cropPointP2.x;
        int yMin = cropPointP1.y < cropPointP2.y ? cropPointP1.y : cropPointP2.y;
        int width = Math.abs(cropPointP1.x - cropPointP2.x);
        int height = Math.abs(cropPointP1.y - cropPointP2.y);
        BufferedImage buffImgNew =
                new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        //System.out.println(xMin + ":" + yMin + " " + width + ":" + height);
        if (xMin < 0
                || xMin + width < 0
                || yMin < 0
                || yMin + height < 0
                || xMin > buffImg.getWidth()
                || xMin + width > buffImg.getWidth()
                || yMin > buffImg.getHeight()
                || (yMin + height > buffImg.getHeight())) {
            String msg = LngTImage.text(EnUsTImage.MSG_SELECTION_INSIDE_IMAGE.name());
            throw new OOoDException(msg);
        }
        for (int i = 0; i < width; ++i) {
            for (int k = 0; k < height; ++k) {
                buffImgNew.setRGB(i, k, buffImg.getRGB(xMin + i, yMin + k));
            }
        }
        buffImgOrig = buffImg = buffImgNew;
        pdfCropFl = pdfFl;
        imgScaleRotateAndShow();
    }

    public Point getCropPointP1() {
        return cropPointP1;
    }

    public Point getCropPointP2() {
        return cropPointP2;
    }

    public void movePoint(int serieIdx, int ptIdx,
            int dx, int dy,
            int picWidth, int picHeight) {
        int x = 0, y = 0;
        Point ptScrThisOld = seriesBox.getScreen(serieIdx, ptIdx);
        Point ptScrThisNew =
                new Point(ptScrThisOld.x + dx, ptScrThisOld.y + dy);
        Point2D ptGrphThisNew = getScale().scr2Grph(ptScrThisNew);
        Point2D ptGrphPrev = null;
        Point2D ptGrphNext = null;
        try {
            ptGrphPrev = seriesBox.getGraph(serieIdx, ptIdx - 1);
        } catch (java.lang.IndexOutOfBoundsException ignore) {
        }
        try {
            ptGrphNext = seriesBox.getGraph(serieIdx, ptIdx + 1);
        } catch (java.lang.IndexOutOfBoundsException ignore) {
        }
        int ptsNb = seriesBox.getPointsCount(serieIdx);
        if (ptScrThisNew.x >= 0
                && ptScrThisNew.x <= (picWidth - 1)
                && ptScrThisNew.y >= 0
                && ptScrThisNew.y <= (picHeight - 1)) {
            /*
             *  Cartesian
             */
            if (getScale().getCartesianModeFl()) {
                if (ptsNb == 1) {
                    x = ptScrThisNew.x;
                    y = ptScrThisNew.y;
                } else if (ptsNb >= 2) {
                    if (ptIdx == 0) {
                        if (ptGrphThisNew.getX() <= ptGrphNext.getX()) {
                            x = ptScrThisNew.x;
                            y = ptScrThisNew.y;
                        } else {
                            x = ptScrThisOld.x;
                            y = ptScrThisOld.y;
                        }
                    } else if (ptIdx == ptsNb - 1) {
                        if (ptGrphThisNew.getX() >= ptGrphPrev.getX()) {
                            x = ptScrThisNew.x;
                            y = ptScrThisNew.y;
                        } else {
                            x = ptScrThisOld.x;
                            y = ptScrThisOld.y;
                        }
                    } else if (ptIdx > 0
                            && ptIdx < ptsNb - 1) {
                        if (ptGrphThisNew.getX() >= ptGrphPrev.getX()
                                && ptGrphThisNew.getX() <= ptGrphNext.getX()) {
                            x = ptScrThisNew.x;
                            y = ptScrThisNew.y;
                        } else {
                            x = ptScrThisOld.x;
                            y = ptScrThisOld.y;
                        }
                    }
                }
            } else {
                /*
                 *  Polar
                 */
                if (ptsNb == 1) {
                    x = ptScrThisNew.x;
                    y = ptScrThisNew.y;
                } else if (ptsNb >= 2) {
                    if (ptIdx == 0) {
                        if (ptGrphThisNew.getY() <= ptGrphNext.getY()) {
                            x = ptScrThisNew.x;
                            y = ptScrThisNew.y;
                        } else {
                            x = ptScrThisOld.x;
                            y = ptScrThisOld.y;
                        }
                    } else if (ptIdx == ptsNb - 1) {
                        if (ptGrphThisNew.getY() >= ptGrphPrev.getY()) {
                            x = ptScrThisNew.x;
                            y = ptScrThisNew.y;
                        } else {
                            x = ptScrThisOld.x;
                            y = ptScrThisOld.y;
                        }
                    } else if (ptIdx > 0
                            && ptIdx < ptsNb - 1) {
                        if (ptGrphThisNew.getY() >= ptGrphPrev.getY()
                                && ptGrphThisNew.getY() < ptGrphNext.getY()) {
                            x = ptScrThisNew.x;
                            y = ptScrThisNew.y;
                        } else {
                            x = ptScrThisOld.x;
                            y = ptScrThisOld.y;
                        }
                    }
                }
            }
        } else {
            x = ptScrThisOld.x;
            y = ptScrThisOld.y;
        }
        seriesBox.setScreen(serieIdx, ptIdx, x, y);
        ptGrphThisNew = getScale().scr2Grph(new Point(x, y));
        seriesBox.setGraph(serieIdx, ptIdx,
                ptGrphThisNew.getX(), ptGrphThisNew.getY());
        TraceMyStack.print(Thread.currentThread().getStackTrace(),
                "new " + x + ":" + y
                + " [from maks " + (picWidth - 1) + ":" + (picHeight - 1) + "]",
                DBG_LEV);
    }

    public void setImageScale(double scale) throws OOoDException {
        imgFitSize = null;
        imageScale = scale;
        imgScaleRotateAndShow();
    }

    public boolean getInterpFl() {
        return interpFl;
    }

    public void setPDFPageIdx(int idx) throws OOoDException {
        if (pdfFile == null) {
            return;
        }
        pdfCropFl = false;
        if (pdfDocPagesNb == 0) {
            pdfPageIdx = 0;
        } else if (idx < 1) {
            pdfPageIdx = 1;
        } else if (idx > pdfDocPagesNb) {
            pdfPageIdx = pdfDocPagesNb;
        } else {
            pdfPageIdx = idx;
        }
        pdfPage = pdfFile.getPage(pdfPageIdx);
        imgScaleRotateAndShow();
    }

    public void setImageRotateDegrees(double degs) throws OOoDException {
        imageRotateDegrees = degs;
        imgScaleRotateAndShow();
    }

    public void setCropPointEnum(ImgPanCropPointsEnum ptEnu) {
        cropPointEnum = ptEnu;
        repaint();
    }

    public void setScalePointEnum(ImgPanScalePointsEnum ptEnu) {
        scalePointEnum = ptEnu;
        repaint();
    }

    public void setDrawAllSeriesFl(boolean fl) {
        drawAllSeriesFl = fl;
        repaint();
    }

    public void setDrawPointsFl(boolean fl) {
        drawPointsFl = fl;
        repaint();
    }

    public void setDrawLinesFl(boolean fl) {
        drawLinesFl = fl;
        repaint();
    }

    public void setDrawScaleFl(boolean fl) {
        drawScaleFl = fl;
        repaint();
    }

    public void setDrawImageFl(boolean fl) {
        drawImageFl = fl;
        repaint();
    }

    public Integer getPtSize() {
        return new Integer(ptSize);
    }

    public void setPtSize(int s) {
        ptShift = setPtShift(s);
        ptSize = s;
    }

    public Integer getLineWidth() {
        return new Integer((int) lineStroke.getLineWidth());
    }

    public void setLineWidth(int w) {
        lineStroke = new BasicStroke(
                w, lineStroke.getEndCap(), lineStroke.getLineJoin());
    }

    public ImageIcon[] getPtrsIcos() {
        return pointers.getPtrsIcos();
    }

    public int getChoosenPointerIdx() {
        return pointerIdx;
    }

    public final void setPointerAndIcoByIdx(int idx) {
        idx = idx > pointers.getPtrsNb() - 1 ? pointers.getPtrsNb() - 1 : idx;
        setCursor(pointers.getPtrByIdx(idx));
        pointerIdx = idx;
    }

    public int getPointerIdxByIco(ImageIcon ico) {
        return pointers.getIcoIdx(ico);
    }

    public void setPointerColor() {
        pointers.setPtrCol();
    }

    public void loadImageFromClipboard() throws OOoDException {
        resetState();
        Image img = null;
        Transferable transferable = Toolkit.getDefaultToolkit().
                getSystemClipboard().getContents(null);
        if (transferable != null
                && transferable.isDataFlavorSupported(DataFlavor.imageFlavor)) {
            try {
                img = (Image) transferable.getTransferData(DataFlavor.imageFlavor);
            } catch (UnsupportedFlavorException e) {
                TraceMyStack.print(e.getStackTrace(), e.getMessage(), DBG_LEV);
            } catch (IOException e) {
                TraceMyStack.print(e.getStackTrace(), e.getMessage(), DBG_LEV);
            }
        }
        if (img == null) {
            MsgBox.msgPlainTh("Can't get an image from the clipboard.",
                    SwingUtilities.getRoot(this),
                    JOptionPane.INFORMATION_MESSAGE);
        } else {
            new ImageIcon(img).getImage();
            int w = img.getWidth(null);
            int h = img.getHeight(null);
            BufferedImage bI = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
            bI.getGraphics().drawImage(img, 0, 0, null);
            buffImg = buffImgOrig = bI;
            imgScaleRotateAndShow();
        }
    }

    public void loadFile(File file, ImgPanFileTypeEnum fTypeEnu) throws OOoDException {
        if (file == null) {
            return;
        }
        resetState();
        imgFile = file;
        switch (fTypeEnu) {
            case IMAGE:
                pdfFl = false;
                try {
                    buffImgOrig = ImageIO.read(imgFile);
                } catch (IOException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                }
                break;
            case PDF:
                pdfFl = true;
                try {
                    RandomAccessFile raf = new RandomAccessFile(file, "r");
                    FileChannel channel = raf.getChannel();
                    ByteBuffer buf = channel.map(
                            FileChannel.MapMode.READ_ONLY, 0, channel.size());
                    pdfFile = new PDFFile(buf);
                    pdfDocPagesNb = pdfFile.getNumPages();
                    pdfPageIdx = 1;
                } catch (Exception ex) {
                    pdfPageIdx = 0;
                    pdfDocPagesNb = 0;
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                }
                break;
            default:
                break;
        }
        imgScaleRotateAndShow();
    }

    public void reloadImage() throws OOoDException {
        if (imgFile == null) {
            return;
        }
        if (pdfFl) {
            pdfCropFl = false;
        } else {
            try {
                buffImgOrig = ImageIO.read(imgFile);
            } catch (IOException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            }
        }
        buffImg = buffImgOrig;
        imgScaleRotateAndShow();
    }

    public void setInterpFl(boolean fl) {
        interpFl = fl;
    }

    public Integer getMorePts() {
        return new Integer(morePts);
    }

    public void setMorePts(int n) {
        morePts = n;
    }

    public void setPtsNbBeforeInterp(int n) {
        ptsNbBeforeInterp = n;
    }

    public void interpolate2(int serieIdx)
            throws
            OOoDException {
        double xOld[] = null;
        double yOld[] = null;
        double xNew[] = null;
        double yNew[] = null;
        Point2D ptArr[] = getScreenXYs(serieIdx);
        Point2D ptArrRotat[] = new Point2D[ptArr.length];
        AffineTransform affTrans = new AffineTransform();
        Double rad = scale.getYAxisAngleRad();
        if (Double.isInfinite(rad) || Double.isNaN(rad)) {
            rad = 0.0;
        }
        affTrans.rotate(-rad, getWidth() / 2.0, getHeight() / 2.0);
        affTrans.transform(ptArr, 0, ptArrRotat, 0, ptArr.length);
        Set<Point2D> ptSet = new TreeSet<Point2D>(new Point2DXComparator());
        for (Point2D pt : ptArrRotat) {
            if (!ptSet.add(pt)) {
                throw new OOoDException(LngMInterp.text(
                        EnUsMInterp.MSG_DELETE_DUPLICATES.name()));
            }
        }
        xOld = new double[ptSet.size()];
        yOld = new double[ptSet.size()];
        prepXYArrays(ptSet, xOld, yOld);
        try {
            xNew = Spline.xNew(xOld, morePts);
            yNew = Spline.yNew(xOld, yOld, morePts, xNew);
            ptArr = prepP2DArr(xNew, yNew);
            ptArrRotat = new Point2D[ptArr.length];
            affTrans = new AffineTransform();
            affTrans.rotate(rad, getWidth() / 2.0, getHeight() / 2.0);
            affTrans.transform(ptArr, 0, ptArrRotat, 0, ptArr.length);
            xNew = new double[ptArrRotat.length];
            yNew = new double[ptArrRotat.length];
            prepXYArrays(ptArrRotat, xNew, yNew);
            loadInterp(xNew, yNew);
        } catch (SplineExceptionMorePoints ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            throw new OOoDException(LngMInterp.text(
                    EnUsMInterp.MSG_INTERP_AT_LEAST_3_PTS.name()));
        } catch (SplineExceptionDuplicates ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            throw new OOoDException(LngMInterp.text(
                    EnUsMInterp.MSG_DELETE_DUPLICATES.name()));
        } finally {
            seriesBox.sortSeriesByGrph(serieIdx, true);
        }
    }

    public void mergeInterp(int seriesIdx, boolean mergeFl) {
        if (mergeFl) {
            seriesBox.delAllPointsFromSeries(seriesIdx);
            int rows = interpSeries.size();
            for (int i = 0; i < rows; ++i) {
                Point ptScr = interpSeries.getScreen(i);
                Point2D ptGrph = scale.scr2Grph(ptScr);
                seriesBox.insertPointUnique(seriesIdx,
                        ptScr.x, ptScr.y, ptGrph.getX(), ptGrph.getY());
            }
        }
        interpSeries.delAllPoints();
    }

    public int getPtsNbBeforeInterp() {
        return ptsNbBeforeInterp;
    }

    public void resetScaleCartes(double xSc, double ySc) {
        scale.scaleCartes(
                new Point((int) (50 * xSc), (int) (200 * ySc)),
                new Point((int) (350 * xSc), (int) (200 * ySc)),
                new Point((int) (200 * xSc), (int) (350 * ySc)),
                new Point((int) (200 * xSc), (int) (50 * ySc)));
        repaint();
    }

    public void resetScalePolar(double xSc, double ySc) {
        scale.scalePolar(
                new Point((int) (200 * xSc), (int) (200 * ySc)),
                new Point((int) (350 * xSc), (int) (200 * ySc)));
        repaint();
    }

    public void removeDuplSelSeries() {
        if (selectedSeriesIdx < 0) {
            return;
        }
        removeDuplicates(selectedSeriesIdx);
    }

    public void setTabEnum(ImgPanTabsEnum tE) {
        tabEnu = tE;
    }

    public BufferedImage getBuffImg() {
        return buffImg;
    }

    public SeriesBox getSeriesBox() {
        return seriesBox;
    }

    public void scaleCropPoints(double sx, double sy) {
        cropPointP1.setLocation((int) (cropPointP1.x * sx), (int) (cropPointP1.y * sy));
        cropPointP2.setLocation((int) (cropPointP2.x * sx), (int) (cropPointP2.y * sy));
        repaint();
    }


    /*
     * Package
     */
    int getPointerAndIcoByIdx() {
        return pointerIdx;
    }

    void origImg() {
        buffImg = buffImgOrig;
    }

    void setImg(BufferedImage i) {
        buffImg = i;
    }

    void imgScaleRotateAndShow()
            throws OOoDException {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DBG_LEV);
        if (buffImgOrig == null && pdfFile == null) {
            return;
        }
        if (pdfFl) {
            if ((pdfPage = pdfFile.getPage(pdfPageIdx)) == null) {
                return;
            }
        }
        if (imgFitSize != null) {
            Point2D dimTmp1, dimTmp2;
            if (pdfFl && !this.pdfCropFl) {
                dimTmp1 = calcDimAfterRotImg(imageRotateDegrees,
                        new Point.Double(
                        pdfPage.getWidth(),
                        pdfPage.getHeight()));
            } else {
                dimTmp1 = calcDimAfterRotImg(imageRotateDegrees,
                        new Point.Double(
                        buffImgOrig.getWidth(),
                        buffImgOrig.getHeight()));
            }
            dimTmp2 = calcDimToFitImg(dimTmp1, imgFitSize);
            double imSc = 1.0;
            if (dimTmp2.getX() > 0) {
                imSc = dimTmp2.getX() / dimTmp1.getX();
            } else if (dimTmp2.getY() > 0) {
                imSc = dimTmp2.getY() / dimTmp1.getY();
            }
            imageScale = imSc;
        }
        if (pdfFl && !pdfCropFl) {
            loadPDFPageIntoBuffImage();
        } else {
            scaleImage(imageScale, buffImgOrig);
        }
        rotateImg(imageRotateDegrees, buffImg);
        setPreferredSize(new Dimension(buffImg.getWidth(), buffImg.getHeight()));
        revalidate();
        repaint();
    }

    Cursor getPointerByIco(ImageIcon ico) {
        return pointers.getPointerByIco(ico);
    }

    void removeDuplicates(int serieIdx) {
        Point2D ptArr[] = getScreenXYs(serieIdx);
        Point2D ptArrRotat[] = new Point2D[ptArr.length];
        AffineTransform affTrans = new AffineTransform();
        Double rad = scale.getYAxisAngleRad();
        if (Double.isInfinite(rad) || Double.isNaN(rad)) {
            rad = 0.0;
        }
        affTrans.rotate(-rad, getWidth() / 2.0, getHeight() / 2.0);
        affTrans.transform(ptArr, 0, ptArrRotat, 0, ptArr.length);
        Set<Point2D> ptSet = new TreeSet<Point2D>(new Point2DXComparator());
        ptSet.addAll(Arrays.asList(ptArrRotat));
        ptArr = new Point2D[ptSet.size()];
        int i = 0;
        for (Point2D pt : ptSet) {
            ptArr[i] = new Point.Double(pt.getX(), pt.getY());
            ++i;
        }
        ptArrRotat = new Point2D[ptArr.length];
        affTrans = new AffineTransform();
        affTrans.rotate(rad, getWidth() / 2.0, getHeight() / 2.0);
        affTrans.transform(ptArr, 0, ptArrRotat, 0, ptArr.length);
        seriesBox.clearSeriesByIdx(serieIdx);
        Point ptScr = new Point(0, 0);
        Point2D ptGrph = new Point.Double(0, 0);
        for (Point2D pt : ptArrRotat) {
            ptScr.setLocation(Math.round(pt.getX()), Math.round(pt.getY()));
            ptGrph.setLocation(scale.scr2Grph(ptScr));
            seriesBox.insertPointUnique(serieIdx, ptScr.x, ptScr.y,
                    ptGrph.getX(), ptGrph.getY());
        }
    }

    void loadPDFPageIntoBuffImage() throws OOoDException {
        if (pdfPage == null) {
            return;
        }
        try {
            int pgWidth = (int) pdfPage.getBBox().getWidth();
            int pgHeight = (int) pdfPage.getBBox().getHeight();
            Rectangle rect = new Rectangle(0, 0, pgWidth, pgHeight);
            int rotation = pdfPage.getRotation();
            Rectangle rect1 = rect;
            if (rotation == 90 || rotation == 270) {
                rect1 = new Rectangle(0, 0, rect.height, rect.width);
            }
            buffImgOrig = (BufferedImage) pdfPage.getImage(
                    (int) (rect.width * imageScale),
                    (int) (rect.height * imageScale),
                    rect1, null, true, true);
        } catch (OutOfMemoryError ex) {
            String msg = LngTImage.text(
                    EnUsTImage.MSG_CHOOSE_SMALLER_SCALE.name());
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            throw new OOoDException(msg);
        }
        buffImg = buffImgOrig;
    }

    double getImageScale() {
        return imageScale;
    }

    double getImageRotateDegrees() {
        return imageRotateDegrees;
    }

    /*
     * private
     */
    private Point2D calcDimToFitImg(
            Point2D dimImg,
            Point2D dimImgWindow) {
        double imgProport = dimImg.getX() / dimImg.getY();
        double imgWindowProport =
                dimImgWindow.getX() / dimImgWindow.getY();
        if (imgWindowProport < imgProport) {
            return new Point.Double(dimImgWindow.getX(), -1);
        } else {
            return new Point.Double(-1, dimImgWindow.getY());
        }
    }

    private void scaleImage(int w, int h)
            throws OOoDException {
        if (buffImgOrig == null) {
            return;
        }
        try {
            Image img = buffImgOrig.getScaledInstance(w, h, Image.SCALE_SMOOTH);
            buffImg = imgToBuffImg(img);
        } catch (OutOfMemoryError ex) {
            String msg = LngTImage.text(
                    EnUsTImage.MSG_CHOOSE_SMALLER_SCALE.name());
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            throw new OOoDException(msg);
        }
    }

    private void scaleImage(double scale, BufferedImage bI)
            throws OOoDException {
        if (bI == null) {
            return;
        }
        try {
            Image img = bI.getScaledInstance(
                    (int) (bI.getWidth() * scale),
                    (int) (bI.getHeight() * scale),
                    Image.SCALE_SMOOTH);
            buffImg = imgToBuffImg(img);
        } catch (OutOfMemoryError ex) {
            String msg = LngTImage.text(
                    EnUsTImage.MSG_CHOOSE_SMALLER_SCALE.name());
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            throw new OOoDException(msg);
        }
    }

    private Point2D[] prepP2DArr(double x[], double y[]) {
        int rows = x.length;
        Point2D ptArr[] = new Point2D[rows];
        for (int i = 0; i < rows; ++i) {
            ptArr[i] = new Point.Double(x[i], y[i]);
        }
        return ptArr;
    }

    private void prepXYArrays(Point2D ptArr[],
            double x[], double y[]) {
        for (int i = 0; i < ptArr.length; ++i) {
            x[i] = ptArr[i].getX();
            y[i] = ptArr[i].getY();
        }
    }

    private void prepXYArrays(Set<Point2D> ptSet,
            double x[], double y[]) {
        int i = 0;
        for (Point2D pt : ptSet) {
            x[i] = pt.getX();
            y[i] = pt.getY();
            ++i;
        }
    }

    private Point2D[] getScreenXYs(int serieIdx) {
        int rows = seriesBox.getPointsCount(serieIdx);
        Point2D ptArr[] = new Point2D[rows];
        for (int i = 0; i < rows; ++i) {
            ptArr[i] = new Point.Double(seriesBox.getScreen(serieIdx, i).x,
                    seriesBox.getScreen(serieIdx, i).y);
        }
        return ptArr;
    }

    private double[] getScreenXs(int serieIdx) {
        int rows = seriesBox.getPointsCount(serieIdx);
        double x[] = new double[rows];
        for (int i = 0; i < rows; ++i) {
            x[i] = seriesBox.getScreen(serieIdx, i).x;
        }
        return x;
    }

    private void loadInterp(double x[], double y[]) {
        if (x.length != y.length) {
            String msg = "Can't load interpolation data.\n"
                    + "x[] and y[] must have the same size.";
            TraceMyStack.print(Thread.currentThread().getStackTrace(), msg,
                    DBG_LEV);
            return;
        }
        int rows = x.length;
        for (int i = 0; i < rows; ++i) {
            interpSeries.addPoint(new PointOOoD(
                    (int) Math.round(x[i]), (int) Math.round(y[i]), 0, 0));
        }
    }

    private void drawLines(Graphics g) {
        Graphics2D g2d = ((Graphics2D) g);
        if (drawAllSeriesFl) {
            int seriesNb = seriesBox.getSeriesCount();
            for (int sI = 0; sI < seriesNb; ++sI) {
                Color col = (sI == selectedSeriesIdx)
                        ? lineCol
                        : lineColAnotherSeries;
                g.setColor(col);
                int ptsNb = seriesBox.getPointsCount(sI);
                if (ptsNb == 1) {
                    g2d.setStroke(ptStroke);
                    g2d.setColor(ptCol);
                    int size = ptSize;
                    g2d.fillOval(seriesBox.getScreen(sI, 0).x - ptShift,
                            seriesBox.getScreen(sI, 0).y - ptShift,
                            size, size);
                }
                g2d.setStroke(lineStroke);
                for (int pI = 0; pI < (ptsNb - 1); ++pI) {
                    col = (sI == selectedSeriesIdx)
                            ? lineCol
                            : lineColAnotherSeries;
                    g.setColor(col);
                    g2d.drawLine(seriesBox.getScreen(sI, pI).x,
                            seriesBox.getScreen(sI, pI).y,
                            seriesBox.getScreen(sI, pI + 1).x,
                            seriesBox.getScreen(sI, pI + 1).y);
                }
            }
        } else {
            if (selectedSeriesIdx < 0) {
                return;
            }
            int ptsNb = seriesBox.getPointsCount(selectedSeriesIdx);
            if (ptsNb == 1) {
                g2d.setStroke(ptStroke);
                g2d.setColor(ptCol);
                int size = ptSize;
                g2d.fillOval(seriesBox.getScreen(selectedSeriesIdx, 0).x - ptShift,
                        seriesBox.getScreen(selectedSeriesIdx, 0).y - ptShift,
                        size, size);
            }
            g2d.setStroke(lineStroke);
            g.setColor(lineCol);
            for (int pI = 0; pI < (ptsNb - 1); ++pI) {
                g2d.drawLine(seriesBox.getScreen(selectedSeriesIdx, pI).x,
                        seriesBox.getScreen(selectedSeriesIdx, pI).y,
                        seriesBox.getScreen(selectedSeriesIdx, pI + 1).x,
                        seriesBox.getScreen(selectedSeriesIdx, pI + 1).y);
            }
        }
        if (selectedSeriesIdx >= 0 && selPointInASeriesIdx >= 0) {
            g.setColor(ptColSel);
            g2d.fillOval(
                    seriesBox.getScreen(selectedSeriesIdx, selPointInASeriesIdx).x
                    - ptShift,
                    seriesBox.getScreen(selectedSeriesIdx, selPointInASeriesIdx).y
                    - ptShift,
                    ptSize, ptSize);
        }
    }

    private void drawPoints(Graphics g) {
        Graphics2D g2d = ((Graphics2D) g);
        int size = ptSize;
        if (drawAllSeriesFl) {
            int seriesNb = seriesBox.getSeriesCount();
            g2d.setStroke(ptStroke);
            for (int sI = 0; sI < seriesNb; ++sI) {
                Color col = (sI == selectedSeriesIdx)
                        ? ptCol
                        : ptColAnotherSeries;
                g.setColor(col);
                int ptsNb = seriesBox.getPointsCount(sI);
                for (int ptIdx = 0; ptIdx < ptsNb; ++ptIdx) {
                    g2d.fillOval(seriesBox.getScreen(sI, ptIdx).x - ptShift,
                            seriesBox.getScreen(sI, ptIdx).y - ptShift,
                            size, size);
                }
            }
        } else {
            if (selectedSeriesIdx < 0) {
                return;
            }
            int ptsNb = seriesBox.getPointsCount(selectedSeriesIdx);
            g2d.setStroke(ptStroke);
            g2d.setColor(ptCol);
            for (int pI = 0; pI < ptsNb; ++pI) {
                g2d.fillOval(seriesBox.getScreen(selectedSeriesIdx, pI).x - ptShift,
                        seriesBox.getScreen(selectedSeriesIdx, pI).y - ptShift,
                        size, size);
            }
        }
        if (selectedSeriesIdx >= 0 && selPointInASeriesIdx >= 0) {
            g.setColor(ptColSel);
            g2d.fillOval(
                    seriesBox.getScreen(selectedSeriesIdx, selPointInASeriesIdx).x
                    - ptShift,
                    seriesBox.getScreen(selectedSeriesIdx, selPointInASeriesIdx).y
                    - ptShift,
                    size, size);
        }
    }

    private void drawInterpolation(Graphics g) {
        Graphics2D g2d = ((Graphics2D) g);
        int ptsNb = interpSeries.size();
        if (drawLinesFl) {
            g2d.setStroke(lineStroke);
            g2d.setColor(lineCol);
            for (int pI = 0; pI < (ptsNb - 1); ++pI) {
                g2d.drawLine(interpSeries.getScreen(pI).x,
                        interpSeries.getScreen(pI).y,
                        interpSeries.getScreen(pI + 1).x,
                        interpSeries.getScreen(pI + 1).y);
            }
        }
        if (drawPointsFl) {
            g2d.setStroke(ptStroke);
            g2d.setColor(ptCol);
            int size = ptSize;
            for (int pI = 0; pI < ptsNb; ++pI) {
                g2d.fillOval(interpSeries.getScreen(pI).x - ptShift,
                        interpSeries.getScreen(pI).y - ptShift,
                        size, size);
            }
        }
    }

    private void drawScale(Graphics g) {
        int size = ptSize;
        Graphics2D g2d = (Graphics2D) g;
        /*
         * Cartesian
         */
        if (getScale().getCartesianModeFl()) {
            g2d.setStroke(lineStroke);
            g2d.setColor(lineScaleCol);
            g2d.drawLine(scale.getXAxisP1Scr().x, scale.getXAxisP1Scr().y,
                    scale.getXAxisP2Scr().x, scale.getXAxisP2Scr().y);
            g2d.drawLine(scale.getYAxisP1Scr().x, scale.getYAxisP1Scr().y,
                    scale.getYAxisP2Scr().x, scale.getYAxisP2Scr().y);
            g2d.setStroke(ptStroke);
            g.fillOval(scale.getXAxisP1Scr().x - ptShift,
                    scale.getXAxisP1Scr().y - ptShift,
                    size, size);
            g.fillOval(scale.getXAxisP2Scr().x - ptShift,
                    scale.getXAxisP2Scr().y - ptShift,
                    size, size);
            g.fillOval(scale.getYAxisP1Scr().x - ptShift,
                    scale.getYAxisP1Scr().y - ptShift,
                    size, size);
            g.fillOval(scale.getYAxisP2Scr().x - ptShift,
                    scale.getYAxisP2Scr().y - ptShift,
                    size, size);
            g2d.setStroke(ptStroke);
            g2d.setColor(ptColSel);
            switch (tabEnu) {
                case SCALE:
                    switch (scalePointEnum) {
                        case X_AXIS_P1:
                            g.fillOval(scale.getXAxisP1Scr().x - ptShift,
                                    scale.getXAxisP1Scr().y - ptShift,
                                    size, size);
                            break;
                        case X_AXIS_P2:
                            g.fillOval(scale.getXAxisP2Scr().x - ptShift,
                                    scale.getXAxisP2Scr().y - ptShift,
                                    size, size);
                            break;
                        case Y_AXIS_P1:
                            g.fillOval(scale.getYAxisP1Scr().x - ptShift,
                                    scale.getYAxisP1Scr().y - ptShift,
                                    size, size);
                            break;
                        case Y_AXIS_P2:
                            g.fillOval(scale.getYAxisP2Scr().x - ptShift,
                                    scale.getYAxisP2Scr().y - ptShift,
                                    size, size);
                            break;
                        default:
                            break;
                    }
                default:
                    break;
            }
        }
        /*
         * Polar
         */
        if (!scale.getCartesianModeFl()) {
            g2d.setStroke(lineStroke);
            g2d.setColor(lineScaleCol);
            g2d.drawLine(scale.getXAxisP1Scr().x, scale.getXAxisP1Scr().y,
                    scale.getXAxisP2Scr().x, scale.getXAxisP2Scr().y);
            g2d.setStroke(ptStroke);
            g.fillOval(scale.getXAxisP1Scr().x - ptShift,
                    scale.getXAxisP1Scr().y - ptShift,
                    size, size);
            g.fillOval(scale.getXAxisP2Scr().x - ptShift,
                    scale.getXAxisP2Scr().y - ptShift,
                    size, size);
            g2d.setStroke(ptStroke);
            g.setColor(ptColSel);
            switch (tabEnu) {
                case SCALE:
                    switch (scalePointEnum) {
                        case POLAR_0_0:
                            g.fillOval(scale.getXAxisP1Scr().x - ptShift,
                                    scale.getXAxisP1Scr().y - ptShift,
                                    size, size);
                            break;
                        case POLAR_X_Y:
                            g.fillOval(scale.getXAxisP2Scr().x - ptShift,
                                    scale.getXAxisP2Scr().y - ptShift,
                                    size, size);
                            break;
                        default:
                            break;
                    }
                    break;
                default:
                    break;
            }
        }
    }

    private void drawCropRect(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setStroke(lineStroke);
        g2d.setColor(lineCropCol);
        int xMin = cropPointP1.x < cropPointP2.x ? cropPointP1.x : cropPointP2.x;
        int yMin = cropPointP1.y < cropPointP2.y ? cropPointP1.y : cropPointP2.y;
        int width = Math.abs(cropPointP1.x - cropPointP2.x);
        int height = Math.abs(cropPointP1.y - cropPointP2.y);
        g2d.drawRect(xMin, yMin, width, height);
        g2d.setStroke(ptStroke);
        g2d.setColor(ptCol);
        g2d.fillOval(cropPointP1.x - ptShift, cropPointP1.y - ptShift,
                ptSize, ptSize);
        g2d.fillOval(cropPointP2.x - ptShift, cropPointP2.y - ptShift,
                ptSize, ptSize);
        g2d.setColor(ptColSel);
        switch (cropPointEnum) {
            case CROP_IMG_P1:
                g2d.fillOval(cropPointP1.x - ptShift, cropPointP1.y - ptShift,
                        ptSize, ptSize);
                break;
            case CROP_IMG_P2:
                g2d.fillOval(cropPointP2.x - ptShift, cropPointP2.y - ptShift,
                        ptSize, ptSize);
                break;
            default:
                break;
        }
    }

    private BufferedImage imgToBuffImg(Image img) {
        BufferedImage bImg =
                new BufferedImage(img.getWidth(null),
                img.getHeight(null),
                BufferedImage.TYPE_INT_RGB);
        Graphics grph = bImg.getGraphics();
        drawImg(img, grph);
        grph.dispose();
        return bImg;
    }

    private void drawImg(Image img, Graphics g) {
        if (img != null) {
            //Color col = gui.frame.getBackground();
            Color col = getBackground();
            g.setColor(col);
            g.fillRect(0, 0, img.getWidth(this), img.getHeight(this));
            g.drawImage(img, 0, 0, null);
        }
    }

    private int setPtShift(int ptSize) {
        return ptShift = (ptSize - 1) / 2;
    }

    private void rotateImg(double degs, BufferedImage bI) {
        if (bI == null) {
            return;
        }
        degs %= 360;
        degs = Math.toRadians(degs);
        int wOld = bI.getWidth();
        int hOld = bI.getHeight();
        double xOld1 = wOld / 2.0;
        double yOld1 = hOld / 2.0;
        double xOld2 = wOld / 2.0;
        double yOld2 = -hOld / 2.0;
        AffineTransform affTrans = new AffineTransform();
        affTrans.rotate(degs);
        Point2D ptRotat1 = new Point.Double(0, 0);
        Point2D ptRotat2 = new Point.Double(0, 0);
        affTrans.transform(new Point.Double(xOld1, yOld1), ptRotat1);
        affTrans.transform(new Point.Double(xOld2, yOld2), ptRotat2);
        double xNew1 = Math.abs(ptRotat1.getX());
        double yNew1 = Math.abs(ptRotat1.getY());
        double xNew2 = Math.abs(ptRotat2.getX());
        double yNew2 = Math.abs(ptRotat2.getY());
        double wNew = (xNew1 >= xNew2) ? xNew1 * 2.0 : xNew2 * 2.0;
        double hNew = (yNew1 >= yNew2) ? yNew1 * 2.0 : yNew2 * 2.0;
        int xShift = (int) Math.round((wNew - wOld) / 2.0);
        int yShift = (int) Math.round((hNew - hOld) / 2.0);
        affTrans = new AffineTransform();
        affTrans.rotate(degs, wNew / 2.0, hNew / 2.0);
        affTrans.translate(xShift, yShift);
        AffineTransformOp affTransOp = new AffineTransformOp(affTrans,
                AffineTransformOp.TYPE_BICUBIC);
        BufferedImage buffImgOut = affTransOp.filter(bI, null);
        buffImg = buffImgOut;
    }

    private Point2D calcDimAfterRotImg(double degs, Point2D dimImg) {
        BufferedImage bI = new BufferedImage(
                (int) Math.round(dimImg.getX()),
                (int) Math.round(dimImg.getY()),
                BufferedImage.TYPE_INT_RGB);
        degs %= 360;
        degs = Math.toRadians(degs);
        int wOld = bI.getWidth();
        int hOld = bI.getHeight();
        double xOld1 = wOld / 2.0;
        double yOld1 = hOld / 2.0;
        double xOld2 = wOld / 2.0;
        double yOld2 = -hOld / 2.0;
        AffineTransform affTrans = new AffineTransform();
        affTrans.rotate(degs);
        Point2D ptRotat1 = new Point.Double(0, 0);
        Point2D ptRotat2 = new Point.Double(0, 0);
        affTrans.transform(new Point.Double(xOld1, yOld1), ptRotat1);
        affTrans.transform(new Point.Double(xOld2, yOld2), ptRotat2);
        double xNew1 = Math.abs(ptRotat1.getX());
        double yNew1 = Math.abs(ptRotat1.getY());
        double xNew2 = Math.abs(ptRotat2.getX());
        double yNew2 = Math.abs(ptRotat2.getY());
        double wNew = (xNew1 >= xNew2) ? xNew1 * 2.0 : xNew2 * 2.0;
        double hNew = (yNew1 >= yNew2) ? yNew1 * 2.0 : yNew2 * 2.0;
        double xShift = (wNew - wOld) / 2.0;
        double yShift = (hNew - hOld) / 2.0;
        affTrans = new AffineTransform();
        affTrans.rotate(degs, wNew / 2.0, hNew / 2.0);
        affTrans.translate(xShift, yShift);
        AffineTransformOp affTransOp = new AffineTransformOp(affTrans,
                AffineTransformOp.TYPE_BICUBIC);
        BufferedImage buffImgOut =
                affTransOp.createCompatibleDestImage(bI, bI.getColorModel());
        return new Point.Double(buffImgOut.getWidth(), buffImgOut.getHeight());
    }

    private void resetState() {
        //setPreferredSize(new Dimension(1, 1));
        buffImg = null;
        buffImgOrig = null;
        imgFile = null;
        pdfFl = false;
        pdfCropFl = false;
        pdfPage = null;
        pdfFile = null;
        pdfDocPagesNb = 0;
        pdfPageIdx = 0;
    }

    static class Point2DXComparator implements Comparator<Point2D> {

        public int compare(Point2D p1, Point2D p2) {
            int cmp = 0;
            if (p1.getX() > p2.getX()) {
                cmp = 1;
            }
            if (p1.getX() < p2.getX()) {
                cmp = -1;
            }
            if (p1.getX() == p2.getX()) {
                cmp = 0;
            }
            return cmp;
        }
    }

    /*
     * Exceptions
     */
    public class ImgPanelNoSeriesException extends Exception {

        public ImgPanelNoSeriesException() {
            super("No series.");
        }
    }

    public static void main(String args[]) {
    }
}
