 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.image;

import name.mgplldz.guimy.ctrls.MsgBox;
import java.awt.Point;
import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;
import name.mgplldz.guimy.listeners.ButAction;
import name.mgplldz.guimy.ctrls.ButMy;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.scale.Scale.ScaleLogException;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListButActionTImage extends ButAction {

    private GUI gui;
    private boolean doClickFl = false;
    private final int DBG_LEV = 3;

    public ListButActionTImage(boolean click, GUI g) {
        gui = g;
        doClickFl = click;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        ButMy but = (ButMy) e.getSource();
        if (doClickFl) {
            but.doClick();
            return;
        }
        String butName = but.getName();
        TraceMyStack.print(Thread.currentThread().getStackTrace(), butName, DBG_LEV);
        CtrlTImage ctrl = CtrlTImage.valueOf(butName);
        switch (ctrl) {
            case BUT_FIT_IMG:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                if (gui.frame.tabPane.tImage.chkScaleImg.isSelected()) {
                    gui.frame.tabPane.tImage.chkScaleImg.setIgnoreItemListIdx(1);
                    gui.frame.tabPane.tImage.chkScaleImg.setSelected(false);
                    gui.frame.tabPane.tImage.txtScaleImg.onListAndON(false);
                }
                try {
                    gui.frame.tabPane.tImage.imgPan.setFitSize(
                            new Point.Double(
                            gui.frame.tabPane.tImage.imgPan.getScrollPane().getViewport().getWidth(),
                            gui.frame.tabPane.tImage.imgPan.getScrollPane().getViewport().getHeight()));
                } catch (OOoDException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                    MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                            JOptionPane.INFORMATION_MESSAGE);
                }
                break;
            case BUT_ORIG_SIZE_IMG:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                if (gui.frame.tabPane.tImage.chkScaleImg.isSelected()) {
                    gui.frame.tabPane.tImage.chkScaleImg.setIgnoreItemListIdx(1);
                    gui.frame.tabPane.tImage.chkScaleImg.setSelected(false);
                    gui.frame.tabPane.tImage.txtScaleImg.onListAndON(false);
                }
                imgScale(1.0);
                break;
            case BUT_PDF_PG_NEXT:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                try {
                    gui.frame.tabPane.tImage.imgPan.changePdfPageIdx(+1);
                } catch (OOoDException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                    MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                            JOptionPane.INFORMATION_MESSAGE);
                }
                gui.frame.tabPane.tImage.txtPDFPageNumber.setValue(
                        gui.frame.tabPane.tImage.imgPan.getPDFPageIdx()
                        + "/"
                        + gui.frame.tabPane.tImage.imgPan.getPDFDocPagesNb());
                break;
            case BUT_PDF_PG_PREV:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                try {
                    gui.frame.tabPane.tImage.imgPan.changePdfPageIdx(-1);
                } catch (OOoDException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                    MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                            JOptionPane.INFORMATION_MESSAGE);
                }
                gui.frame.tabPane.tImage.txtPDFPageNumber.setValue(
                        gui.frame.tabPane.tImage.imgPan.getPDFPageIdx()
                        + "/"
                        + gui.frame.tabPane.tImage.imgPan.getPDFDocPagesNb());
                break;
            case BUT_CROP_IMAGE:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                try {
                    gui.frame.tabPane.tImage.imgPan.cropImage();
                    gui.frame.tabPane.tImage.butOrigSize.doClick();
                    gui.frame.tabPane.tImage.chkRotateImg.setSelected(false);
                } catch (OOoDException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                    MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                            JOptionPane.INFORMATION_MESSAGE);
                }
                break;
            case BUT_TUNE_UP:
                tune(0, -1);
                break;
            case BUT_TUNE_DOWN:
                tune(0, 1);
                break;
            case BUT_TUNE_LEFT:
                tune(-1, 0);
                break;
            case BUT_TUNE_RIGHT:
                tune(1, 0);
                break;
            case BUT_DEL:
                delPointFromSerie();
                break;
            default:
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        "Wrong button.", DBG_LEV);
                break;
        }
    }

    private void tune(int dx, int dy) {
        int tabIdx = gui.frame.tabPane.pane.getSelectedIndex();
        switch (tabIdx) {
            case 0:
                if (gui.frame.tabPane.tImage.radButCropImageP1.isSelected()) {
                    gui.frame.tabPane.tImage.imgPan.getCropPointP1().x += dx;
                    gui.frame.tabPane.tImage.imgPan.getCropPointP1().y += dy;
                }
                if (gui.frame.tabPane.tImage.radButCropImageP2.isSelected()) {
                    gui.frame.tabPane.tImage.imgPan.getCropPointP2().x += dx;
                    gui.frame.tabPane.tImage.imgPan.getCropPointP2().y += dy;
                }
                break;
            case 1:
                /*
                 *  Cartesian
                 */
                if (gui.frame.menuBar.menuScale.rbCartesian.isSelected()) {
                    if (gui.frame.tabPane.tScale.radButXAxisP1.isSelected()) {
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x += dx;
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y += dy;
                        if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                        }
                        if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x;
                            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                            if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                                gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x =
                                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
                            }
                        }
                        break;
                    }
                    if (gui.frame.tabPane.tScale.radButXAxisP2.isSelected()) {
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x += dx;
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y += dy;
                        if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y;
                        }
                        if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x;
                            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                            if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                                gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x =
                                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
                            }
                        }
                        break;
                    }
                    if (gui.frame.tabPane.tScale.radButYAxisP1.isSelected()) {
                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x += dx;
                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().y += dy;
                        if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
                        }
                        if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().y;
                            if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                                gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                            }
                        }
                        break;
                    }
                    if (gui.frame.tabPane.tScale.radButYAxisP2.isSelected()) {
                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x += dx;
                        gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().y += dy;
                        if (gui.frame.tabPane.tScale.chkYVertical.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP2Scr().x;
                        }
                        if (gui.frame.tabPane.tScale.chkP1EqualsP1.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().x;
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getYAxisP1Scr().y;
                            if (gui.frame.tabPane.tScale.chkXHorizontal.isSelected()) {
                                gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                            }
                        }
                        break;
                    }
                    break;
                }
                /*
                 *  Polar
                 */
                if (gui.frame.menuBar.menuScale.rbPolar.isSelected()) {
                    if (gui.frame.tabPane.tScale.radButPolar00.isSelected()) {
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x += dx;
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y += dy;
                        if (gui.frame.tabPane.tScale.chkRadiusHorizontal.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y;
                        }
                        if (gui.frame.tabPane.tScale.chkRadiusVertical.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x;
                        }
                        break;
                    }
                    if (gui.frame.tabPane.tScale.radButPolarXY.isSelected()) {
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x += dx;
                        gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y += dy;
                        if (gui.frame.tabPane.tScale.chkRadiusHorizontal.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().y =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().y;
                        }
                        if (gui.frame.tabPane.tScale.chkRadiusVertical.isSelected()) {
                            gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP1Scr().x =
                                    gui.frame.tabPane.tImage.imgPan.getScale().getXAxisP2Scr().x;
                        }
                        break;
                    }
                    break;
                }
                break;
            case 2:
                int serieIdx = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx();
                if (serieIdx < 0) {
                    return;
                }
                int ptIdx = gui.frame.tabPane.tDigit.table.getSelectedRow();
                if (ptIdx < 0) {
                    return;
                }
                gui.frame.tabPane.tImage.imgPan.movePoint(serieIdx, ptIdx, dx, dy,
                        gui.frame.tabPane.tImage.imgPan.getWidth(), gui.frame.tabPane.tImage.imgPan.getHeight());
                gui.frame.tabPane.tDigit.table.updateRow(ptIdx,
                        gui.frame.tabPane.tImage.imgPan.getSeriesBox().getGraph(
                        serieIdx, ptIdx));
                break;
            default:
                break;
        }
        if (gui.frame.menuBar.menuScale.rbCartesian.isSelected()) {
            try {
                gui.setScaleGrph();
            } catch (ScaleLogException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            }
        }
        gui.frame.tabPane.tImage.imgPan.repaint();
    }

    private void delPointFromSerie() {
        int serieIdx = gui.frame.tabPane.tImage.imgPan.getSelectedSerieIdx();
        int ptIdx = gui.frame.tabPane.tDigit.table.getSelectedRow();
        if (serieIdx >= 0 && ptIdx >= 0) {
            gui.frame.tabPane.tImage.imgPan.getSeriesBox().delPointFromSeries(
                    serieIdx, ptIdx);
            int nextSelected = (ptIdx == 0) ? 1 : -1;
            gui.frame.tabPane.tDigit.table.selectNextPrevRow(nextSelected);
            ((DefaultTableModel) gui.frame.tabPane.tDigit.table.getModel()).removeRow(ptIdx);
            gui.frame.tabPane.tImage.imgPan.repaint();
        }
    }

    private void imgScale(Double scale) {
        try {
            gui.frame.tabPane.tImage.imgPan.setImageScale(scale);
        } catch (OOoDException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }
}
