/* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Godard
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.image;

import java.awt.event.MouseEvent;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.ctrls.*;
import name.mgplldz.guimy.listeners.TxtFldMouseList;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.GUI.OOoDException;
import name.mgplldz.oood.gui.tab.image.lang.EnUsTImage;
import name.mgplldz.oood.gui.tab.image.lang.LngTImage;
import name.mgplldz.tracemystack.TraceMyStack;

public class ListTxtFldMouseTImage extends TxtFldMouseList {

    private final GUI gui;
    private final int DBG_LEV = 3;

    public ListTxtFldMouseTImage(GUI g) {
        gui = g;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        TxtFldMy fld = (TxtFldMy) e.getSource();
        TraceMyStack.print(Thread.currentThread().getStackTrace(),
                fld.getName(), DBG_LEV);
        if (fld.getIgnoreMouseListFl()) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    fld.getName() + ": ignores mouse listener.", DBG_LEV);
            return;
        }
        CtrlTImage ctrl = CtrlTImage.valueOf(fld.getName());
        Double valDbl;
        Integer valInt;
        switch (ctrl) {
            case TXT_SCALE_IMG:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                        LngTImage.text(EnUsTImage.MSG_IMG_SCALE.name()),
                        gui.frame.frm);
                if (valDbl == null) {
                    break;
                }
                fld.setValue(valDbl.doubleValue());
                imgScale(valDbl);
                break;
            case TXT_ROTATE_IMG:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                valDbl = (Double) new MsgBox().getDblTh((Double) fld.getValue(),
                        LngTImage.text(EnUsTImage.MSG_IMG_DEGREES.name()),
                        gui.frame.frm);
                if (valDbl == null) {
                    break;
                }
                valDbl %= 360;
                fld.setValue(valDbl);
                imgRotateDegrees(valDbl);
                break;
            case TXT_PDF_PAGE_NUMBER:
                gui.frame.menuBar.menuDraw.chkDrawImage.setSelected(true);
                valInt = (Integer) new MsgBox().getIntTh(
                        gui.frame.tabPane.tImage.imgPan.getPDFPageIdx(),
                        LngTImage.text(EnUsTImage.MSG_PDF_PAGE_NUMBER.name()),
                        gui.frame.frm);
                if (valInt == null) {
                    break;
                }
                try {
                    gui.frame.tabPane.tImage.imgPan.setPDFPageIdx(valInt);
                } catch (OOoDException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                    MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                            JOptionPane.INFORMATION_MESSAGE);
                }
                gui.frame.tabPane.tImage.txtPDFPageNumber.setValue(
                        gui.frame.tabPane.tImage.imgPan.getPDFPageIdx()
                        + "/" + gui.frame.tabPane.tImage.imgPan.getPDFDocPagesNb());
                break;
            default:
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        "Wrong text field.", DBG_LEV);
                break;
        }
    }

    @Override
    public void mousePressed(MouseEvent e) {
        return;
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        return;
    }

    @Override
    public void mouseEntered(MouseEvent e) {
        return;
    }

    @Override
    public void mouseExited(MouseEvent e) {
        return;
    }

    /*
     *  Private
     */
    private void imgScale(Double scale) {
        try {
            gui.frame.tabPane.tImage.imgPan.setImageScale(scale);
        } catch (OOoDException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }

    private void imgRotateDegrees(Double degs) {
        try {
            gui.frame.tabPane.tImage.imgPan.setImageRotateDegrees(degs);
        } catch (OOoDException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            MsgBox.msgPlainTh(ex.getMessage(), gui.frame.frm,
                    JOptionPane.INFORMATION_MESSAGE);
        }
    }

    /*
     * Test
     */
    public static void main(String args[]) {
    }
}
