 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.image;

import java.awt.Point;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import name.mgplldz.oood.gui.tab.image.SeriesBox.PointOOoD.PointOOoDComparator;
import name.mgplldz.tracemystack.TraceMyStack;

public class SeriesBox {

    private final ArrayList<Series> series = new ArrayList<Series>();
    private ImgPanel imgPan;
    private final ArrayList<PointOOoD> tmpArrLst = new ArrayList<PointOOoD>();
    private final int DGB_LEV = 3;
    /*
     * Public
     */

    public SeriesBox(ImgPanel ip) {
        imgPan = ip;
    }

    public String getXName(int seriesIdx) {
        return series.get(seriesIdx).getXName();
    }

    public String getYName(int seriesIdx) {
        return series.get(seriesIdx).getYName();
    }

    public int getSeriesCount() {
        return series.size();
    }

    public String getSeriesName(int seriesIdx) {
        return series.get(seriesIdx).getName();
    }

    public Point2D getGraph(int seriesIdx, int pointIdx) {
        return series.get(seriesIdx).getPoint(pointIdx).getGraph();
    }

    public int getPointsCount(int seriesIdx) {
        if (seriesIdx < 0) {
            return 0;
        }
        return series.get(seriesIdx).size();
    }

    public void recalcSeries() {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "",
                DGB_LEV);
        if (imgPan.getScale().getCartesianModeFl()) {
            for (Series s : series) {
                for (PointOOoD p : s.points) {
                    p.setGraph(imgPan.getScale().scr2Grph(p.getScreen()));
                }
            }
        } else {
            int rows = 0;
            for (Series s : series) {
                rows = s.size();
                for (int i = 0; i < rows; ++i) {
                    s.points.get(i).setGraph(imgPan.getScale().scr2Grph(
                            s.points.get(i).getScreen()));
                    tmpArrLst.add(s.points.get(i));
                }
                s.points.clear();
                for (int i = 0; i < rows; ++i) {
                    s.insertPoint(tmpArrLst.get(i));
                }
                tmpArrLst.clear();
            }
        }
    }

    public void setSeriesName(int seriesIdx, String name) {
        series.get(seriesIdx).setName(name);
    }

    public void addSeries(String name, String xName, String yName) {
        series.add(new Series(name, xName, yName));
    }

    public void setYNameAll(String name) {
        for (Series s : series) {
            s.setYName(name);
        }
    }

    public void setXNameAll(String name) {
        for (Series s : series) {
            s.setXName(name);
        }
    }

    public void setXName(int seriesIdx, String name) {
        series.get(seriesIdx).setXName(name);
    }

    public void setYName(int seriesIdx, String name) {
        series.get(seriesIdx).setYName(name);
    }

    public void delByIdx(int idx) {
        try {
            series.get(idx);
        } catch (Exception e) {
            String msg2 = "Can't delete series with "
                    + "index " + "[" + idx + "].";
            TraceMyStack.print(Thread.currentThread().getStackTrace(), msg2,
                    DGB_LEV);
            return;
        }
        series.get(idx).delAllPoints();
        series.get(idx).setName(null);
        series.remove(idx);
    }

    public void delAllSeries() {
        for (Series s : series) {
            s.delAllPoints();
            s.setName(null);
        }
        series.clear();
    }

    public int insertPoint(int seriesIdx, int xs, int ys,
            double xg, double yg) {
        return series.get(seriesIdx).insertPoint(xs, ys, xg, yg);
    }

    public void delPointFromSeries(int seriesIdx, int ptIdx) {
        series.get(seriesIdx).delPoint(ptIdx);
    }

    /*
     * Package
     */
    void clearSeriesByIdx(int seriesIdx) {
        series.get(seriesIdx).points.clear();
    }

    void sortSeriesByGrph(int idx, boolean grphFl) {
        Collections.sort(series.get(idx).points,
                new PointOOoDComparator(imgPan.getScale().getCartesianModeFl(),
                grphFl));
    }

    void setGraph(int seriesIdx, int ptIdx, double x, double y) {
        series.get(seriesIdx).getPoint(ptIdx).setGraph(x, y);
    }

    void setScreen(int seriesIdx, int ptIdx, int x, int y) {
        series.get(seriesIdx).getPoint(ptIdx).setScreen(x, y);
    }

    Point getScreen(int seriesIdx, int pointIdx) {
        return series.get(seriesIdx).getPoint(pointIdx).getScreen();
    }

    void delAllPointsFromSeries(int seriesIdx) {
        series.get(seriesIdx).delAllPoints();
    }

    int insertPointUnique(int seriesIdx, int xs, int ys,
            double xg, double yg) {
        return series.get(seriesIdx).insertPointUnique(xs, ys, xg, yg);
    }

    void swapPoints(int seriesIdx, int pointIdx, int swapIdx) {
        try {
            Collections.swap(series.get(seriesIdx).points, pointIdx,
                    pointIdx + swapIdx);
        } catch (IndexOutOfBoundsException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DGB_LEV);
        }
    }

    /*
     *  Class
     */
    class Series {

        private String name;
        private String xName;
        private String yName;
        private final ArrayList<PointOOoD> points = new ArrayList<PointOOoD>();

        public Series(String n, String xn, String yn) {
            name = n;
            xName = xn;
            yName = yn;
        }

        /*
         *  Public
         */
        public void setScreen(double xData[], double yData[]) {
            if (xData.length != yData.length) {
                String msg = "Can't create series.\n"
                        + "xData[] and yData must have the same size.";
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        msg, DGB_LEV);
                return;
            }
            int rows = xData.length;
            for (int i = 0; i < rows; ++i) {
                points.add(new PointOOoD((int) xData[i], (int) yData[i], 0, 0));
            }
        }

        public void addPoint(PointOOoD pt) {
            points.add(pt);
        }

        /*
         *  Private
         */
        private int insertPointUnique(int xs, int ys, double xg, double yg) {
            PointOOoD ptToAdd = new PointOOoD(xs, ys, xg, yg);
            return insertPointUnique(ptToAdd);
        }

        private int insertPointUnique(PointOOoD ptToAdd) {
            int idx = Collections.binarySearch(points, ptToAdd,
                    new PointOOoDComparator(
                    imgPan.getScale().getCartesianModeFl(), true));
            if (idx < 0) {
                if (points.isEmpty()) {
                    points.add(ptToAdd);
                } else {
                    points.add(-idx - 1, ptToAdd);
                }
            }
            return idx;
        }

        private int insertPoint(int xs, int ys, double xg, double yg) {
            PointOOoD ptToAdd = new PointOOoD(xs, ys, xg, yg);
            return insertPoint(ptToAdd);
        }

        private int insertPoint(PointOOoD ptToAdd) {
            int idx = Collections.binarySearch(points, ptToAdd,
                    new PointOOoDComparator(
                    imgPan.getScale().getCartesianModeFl(), true));
            if (idx < 0) {
                if (points.isEmpty()) {
                    points.add(ptToAdd);
                } else {
                    points.add(-idx - 1, ptToAdd);
                }
            } else {
                points.add(idx, ptToAdd);
            }
            return idx;
        }

        public void delPoint(int idx) {
            points.remove(idx);
        }

        public void delAllPoints() {
            points.clear();
        }

        public PointOOoD getPoint(int idx) {
            return points.get(idx);
        }

        public void setName(String n) {
            name = n;
        }

        public String getName() {
            return name;
        }

        public void setXName(String n) {
            xName = n;
        }

        public String getXName() {
            return xName;
        }

        public void setYName(String n) {
            yName = n;
        }

        public String getYName() {
            return yName;
        }

        public int size() {
            return points.size();
        }

        public Point getScreen(int idx) {
            return points.get(idx).getScreen();
        }

        public Point2D getGraph(int idx) {
            return points.get(idx).getGraph();
        }
    }

    /*
     *  Class
     */
    static class PointOOoD {

        private Point scr;
        private Point2D grph;

        public PointOOoD() {
        }

        public PointOOoD(Point s, Point2D g) {
            scr = new Point(s.x, s.y);
            grph = new Point.Double(g.getX(), g.getY());
        }

        public PointOOoD(int xs, int ys, double xg, double yg) {
            scr = new Point(xs, ys);
            grph = new Point.Double(xg, yg);
        }

        public void setScreen(int x, int y) {
            scr.x = x;
            scr.y = y;
        }

        public void setGraph(double x, double y) {
            grph.setLocation(x, y);
        }

        public void setGraph(Point2D pG) {
            grph.setLocation(pG);
        }

        public Point getScreen() {
            return scr;
        }

        public Point2D getGraph() {
            return grph;
        }

        static class PointOOoDComparator implements Comparator<PointOOoD> {

            private boolean graphFl;
            private boolean cartesModeFl;

            PointOOoDComparator(boolean cartFl, boolean grphFl) {
                graphFl = grphFl;
                cartesModeFl = cartFl;
            }

            public int compare(PointOOoD pt1, PointOOoD pt2) {
                int compar = 0;
                double xGrph1 = pt1.getGraph().getX();
                double xGrph2 = pt2.getGraph().getX();
                double xScr1 = pt1.getScreen().x;
                double xScr2 = pt2.getScreen().x;
                double angleGrph1 = pt1.getGraph().getY();
                double angleGrph2 = pt2.getGraph().getY();
                //if (imgPan.getScale().getCartesianModeFl()) {
                if (cartesModeFl) {
                    if (graphFl) {
                        if (xGrph1 > xGrph2) {
                            compar = 1;
                            return compar;
                        }
                        if (xGrph1 < xGrph2) {
                            compar = -1;
                            return compar;
                        }
                        if (xGrph1 == xGrph2) {
                            compar = 0;
                            return compar;
                        }
                    } else {
                        if (xScr1 > xScr2) {
                            compar = 1;
                            return compar;
                        }
                        if (xScr1 < xScr2) {
                            compar = -1;
                            return compar;
                        }
                        if (xScr1 == xScr2) {
                            compar = 0;
                            return compar;
                        }
                    }
                } else {
                    if (angleGrph1 > angleGrph2) {
                        compar = 1;
                        return compar;
                    }
                    if (angleGrph1 < angleGrph2) {
                        compar = -1;
                        return compar;
                    }
                    if (angleGrph1 == angleGrph2) {
                        compar = 0;
                        return compar;
                    }
                }
                return compar;
            }
        }
    }
}
