 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.gui.tab.image;

import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.ToolTipManager;
import name.mgplldz.guimy.ctrls.*;
import name.mgplldz.oood.gui.GUI;
import name.mgplldz.oood.gui.tab.image.CtrlTImage.*;
import name.mgplldz.tracemystack.TraceMyStack;
import name.mgplldz.oood.gui.tab.image.lang.*;

public class TImage {

    private final GUI gui;
    private int imgPanW = 400;
    private int imgPanH = 400;
    /*
     * Border Image
     */
    public final BrdMy bordImage;
    public final TabMy tab;
    public final ImgPanel imgPan;
    public final TxtFldMy txtImgNamePath;
    public final ButMy butFitSize;
    public final ButMy butOrigSize;
    public final TxtFldMy txtScaleImg;
    public final TxtFldMy txtRotateImg;
    public final ChkBoxMy chkScaleImg;
    public final ChkBoxMy chkRotateImg;
    public final ButtonGroup radButGrpCropImage;
    public final ButMy butCropImage;
    public final RadButMy radButCropImageP1;
    public final RadButMy radButCropImageP2;
    /*
     *  Border PDF
     */
    public final BrdMy bordPDF;
    public final ButMy butPDFPgPrev;
    public final ButMy butPDFPgNext;
    public final TxtFldMy txtPDFPageNumber;
    /*
     *  Border tune point
     */
    public final BrdMy bordTunePoint;
    public final ButMy butLeft;
    public final ButMy butUp;
    public final ButMy butDown;
    public final ButMy butRight;
    public final ButMy butDel;
    /*
     * Listeners
     */
    private final ListButActionTImage butActDontClick;
    private final ListButActionTImage butActClick;
    private final ListImgPanMouseTImage imgPanMouseList;
    private final ListTxtFldMouseTImage txtFldMouseList;
    private final ListRadButActTImage radButActList;
    private final ListChkBoxItemTImage chkBoxItemList;
    private final int DGB_LEV = 3;

    public TImage(GUI g) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DGB_LEV);
        gui = g;
        tab = new TabMy(LngTImage.text(EnUsTImage.T_IMAGE.name()),
                gui.getFnt());
        bordImage = new BrdMy(10, 10, 280, 225,
                LngTImage.text(EnUsTImage.BRD_IMG.name()),
                CtrlTImage.BRD_IMG.name(),
                gui.getFnt(), gui.getXSc(), gui.getYSc());
        bordTunePoint = new BrdMy(10, 360, 280, 65,
                LngTImage.text(EnUsTImage.BRD_TUNE_POINT.name()),
                CtrlTImage.BRD_TUNE_PT.name(),
                gui.getFnt(), gui.getXSc(), gui.getYSc());
        /////
        imgPan = new ImgPanel(295, 18, imgPanW, imgPanH, gui.getXSc(), gui.getYSc());
        imgPanW *= gui.getXSc();
        imgPanH *= gui.getYSc();
        tab.add(imgPan.getScrollPane());
        /////
        txtImgNamePath = new TxtFldMy(10, 25, 210, 30,
                LngTImage.text(EnUsTImage.TXT_IMG_NAME.name()),
                gui.getFnt(), JTextField.LEFT,
                bordImage, gui.COL_GRAY, CtrlTImage.TXT_NAME_PATH.name(),
                !TxtFldMy.EDITABLE, !TxtFldMy.FOCUSABLE, gui.getXSc(), gui.getYSc());
        /////
        butFitSize = new ButMy(10, 65, 100, 30,
                LngTImage.text(EnUsTImage.BUT_FIT_IMG.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.BUT_FIT_IMG.name(),
                null, gui.getXSc(), gui.getYSc());
        butOrigSize = new ButMy(120, 65, 100, 30,
                LngTImage.text(EnUsTImage.BUT_ORIG_SIZE_IMG.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.BUT_ORIG_SIZE_IMG.toString(),
                null, gui.getXSc(), gui.getYSc());
        txtScaleImg = new TxtFldMy(10, 105, 100, 30,
                new Double(1.0), gui.getFnt(), JTextField.LEFT,
                bordImage, gui.COL_WHITE,
                CtrlTImage.TXT_SCALE_IMG.name(),
                TxtFldMy.EDITABLE, !TxtFldMy.FOCUSABLE, gui.getXSc(), gui.getYSc());
        txtRotateImg = new TxtFldMy(10, 145, 100, 30,
                new Double(0), gui.getFnt(), JTextField.LEFT,
                bordImage, gui.COL_WHITE,
                CtrlTImage.TXT_ROTATE_IMG.name(),
                TxtFldMy.EDITABLE, !TxtFldMy.FOCUSABLE, gui.getXSc(), gui.getYSc());
        chkScaleImg = new ChkBoxMy(115, 105, 80, 30,
                LngTImage.text(EnUsTImage.CHK_SCALE_IMG.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.CHK_SCALE_IMG.name(),
                gui.getXSc(), gui.getYSc());
        chkRotateImg = new ChkBoxMy(115, 145, 80, 30,
                LngTImage.text(EnUsTImage.CHK_ROTATE_IMG.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.CHK_ROTATE_IMG.name(),
                gui.getXSc(), gui.getYSc());
        /////
        radButGrpCropImage = new ButtonGroup();
        butCropImage = new ButMy(10, 185, 100, 30,
                LngTImage.text(EnUsTImage.BUT_CROP_IMAGE.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.BUT_CROP_IMAGE.name(),
                null, gui.getXSc(), gui.getYSc());
        radButCropImageP1 = new RadButMy(125, 185, 50, 30,
                LngTImage.text(EnUsTImage.RB_CROP_IMAGE_P1.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.RB_CROP_IMAGE_P1.toString(),
                radButGrpCropImage, gui.getXSc(), gui.getYSc());
        radButCropImageP2 = new RadButMy(175, 185, 50, 30,
                LngTImage.text(EnUsTImage.RB_CROP_IMAGE_P2.name()),
                gui.getFnt(), bordImage,
                CtrlTImage.RB_CROP_IMAGE_P2.toString(), radButGrpCropImage,
                gui.getXSc(), gui.getYSc());
        tab.add(bordImage);
        /*
         *  Border PDF
         */
        bordPDF = new BrdMy(10, 240, 280, 65,
                LngTImage.text(EnUsTImage.BRD_PDF.name()),
                CtrlTImage.BRD_PDF.name(),
                gui.getFnt(), gui.getXSc(), gui.getYSc());
        butPDFPgPrev = new ButMy(10, 25, 60, 30,
                LngTImage.text(EnUsTImage.BUT_PDF_PG_PREV.name()),
                gui.getFnt(), bordPDF,
                CtrlTImage.BUT_PDF_PG_PREV.name(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        butPDFPgNext = new ButMy(165, 25, 60, 30,
                LngTImage.text(EnUsTImage.BUT_PDF_PG_NEXT.name()),
                gui.getFnt(), bordPDF,
                CtrlTImage.BUT_PDF_PG_NEXT.name(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        txtPDFPageNumber = new TxtFldMy(75, 25, 85, 30,
                "0/0", gui.getFnt(), JTextField.CENTER,
                bordPDF, gui.COL_WHITE,
                CtrlTImage.TXT_PDF_PAGE_NUMBER.name(),
                TxtFldMy.EDITABLE, !TxtFldMy.FOCUSABLE,
                gui.getXSc(), gui.getYSc());
        tab.add(bordPDF);
        /*
         *  Border tune point
         */
        butLeft = new ButMy(10, 25, 30, 30, "←", gui.getFnt17Bld(),
                bordTunePoint, CtrlTImage.BUT_TUNE_LEFT.toString(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        butUp = new ButMy(45, 25, 30, 30, "↑", gui.getFnt17Bld(),
                bordTunePoint, CtrlTImage.BUT_TUNE_UP.toString(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        butDown = new ButMy(80, 25, 30, 30, "↓", gui.getFnt17Bld(),
                bordTunePoint, CtrlTImage.BUT_TUNE_DOWN.toString(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        butRight = new ButMy(115, 25, 30, 30, "→", gui.getFnt17Bld(),
                bordTunePoint, CtrlTImage.BUT_TUNE_RIGHT.toString(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        butDel = new ButMy(165, 25, 60, 30,
                LngTImage.text(EnUsTImage.BUT_DEL.name()),
                gui.getFnt(), bordTunePoint, CtrlTImage.BUT_DEL.toString(),
                gui.margZero, gui.getXSc(), gui.getYSc());
        tab.add(bordTunePoint);
        /*
         * Listeners
         */
        butActDontClick = new ListButActionTImage(false, gui);
        butActClick = new ListButActionTImage(true, gui);
        imgPanMouseList = new ListImgPanMouseTImage(gui);
        txtFldMouseList = new ListTxtFldMouseTImage(gui);
        radButActList = new ListRadButActTImage(gui);
        chkBoxItemList = new ListChkBoxItemTImage(gui);
    }

    public final void addListeners() {
        imgPan.addMouseListener(imgPanMouseList);
        butFitSize.addActionListener(butActDontClick);
        butOrigSize.addActionListener(butActDontClick);
        butPDFPgNext.addActionListener(butActDontClick);
        butPDFPgPrev.addActionListener(butActDontClick);
        butCropImage.addActionListener(butActDontClick);
        butRight.addActionListener(butActDontClick);
        butLeft.addActionListener(butActDontClick);
        butUp.addActionListener(butActDontClick);
        butDown.addActionListener(butActDontClick);
        butDel.addActionListener(butActDontClick);
        butUp.setAction("UP", butUp.getName(),
                JComponent.WHEN_IN_FOCUSED_WINDOW, butActClick);
        butDown.setAction("DOWN", butUp.getName(),
                JComponent.WHEN_IN_FOCUSED_WINDOW, butActClick);
        butLeft.setAction("LEFT", butUp.getName(),
                JComponent.WHEN_IN_FOCUSED_WINDOW, butActClick);
        butRight.setAction("RIGHT", butUp.getName(),
                JComponent.WHEN_IN_FOCUSED_WINDOW, butActClick);
        butDel.setAction("DELETE", butUp.getName(),
                JComponent.WHEN_IN_FOCUSED_WINDOW, butActClick);
        txtPDFPageNumber.addMouseListener(txtFldMouseList);
        txtRotateImg.addMouseListener(txtFldMouseList);
        txtScaleImg.addMouseListener(txtFldMouseList);
        radButCropImageP1.addActionListener(radButActList);
        radButCropImageP2.addActionListener(radButActList);
        chkScaleImg.addItemListener(chkBoxItemList);
        chkRotateImg.addItemListener(chkBoxItemList);
    }

    public final void addToolTips() {
        txtImgNamePath.setToolTipText(
                LngTImage.text(EnUsTImage.TIP_T_IMG_FULL_PATH.name()));
        butLeft.setToolTipText(LngTImage.text(EnUsTImage.TIP_TUNE_BUTTONS.name()));
        butRight.setToolTipText(LngTImage.text(EnUsTImage.TIP_TUNE_BUTTONS.name()));
        butUp.setToolTipText(LngTImage.text(EnUsTImage.TIP_TUNE_BUTTONS.name()));
        butDown.setToolTipText(LngTImage.text(EnUsTImage.TIP_TUNE_BUTTONS.name()));
        butDel.setToolTipText(LngTImage.text(EnUsTImage.TIP_TUNE_BUTTONS.name()));
    }

    public void toolTipsShow(boolean showFl) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", DGB_LEV);
        if (showFl) {
            ToolTipManager.sharedInstance().registerComponent(butLeft);
            ToolTipManager.sharedInstance().registerComponent(butRight);
            ToolTipManager.sharedInstance().registerComponent(butUp);
            ToolTipManager.sharedInstance().registerComponent(butDown);
            ToolTipManager.sharedInstance().registerComponent(butDel);
        } else {
            ToolTipManager.sharedInstance().unregisterComponent(butLeft);
            ToolTipManager.sharedInstance().unregisterComponent(butRight);
            ToolTipManager.sharedInstance().unregisterComponent(butUp);
            ToolTipManager.sharedInstance().unregisterComponent(butDown);
            ToolTipManager.sharedInstance().unregisterComponent(butDel);
        }
    }

    public int getImgPanW() {
        return imgPanW;
    }

    public int getImgPanH() {
        return imgPanH;
    }
}
