 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.save;

import java.awt.Component;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import name.mgplldz.guimy.ctrls.*;
import name.mgplldz.oood.gui.filechoose.FileFilterMy;
import name.mgplldz.oood.gui.tab.image.ImgPanel;
import name.mgplldz.oood.gui.menu.series.ItmsMSeries;
import name.mgplldz.oood.lang.EnUs;
import name.mgplldz.oood.lang.Lng;
import name.mgplldz.oood.save.lang.*;
import name.mgplldz.tracemystack.TraceMyStack;

public class CSV {

    private ImgPanel imgPan;
    private Component parentForMsgBox;
    private static ItmsMSeries mode;
    private boolean cartesianFl;
    private boolean clockwiseFl;
    private Integer csvColWidth = 15;
    private final String strFrmtSci = "%- " + csvColWidth.toString() + ".6e";
    private final String strFrmtStr = "%-" + csvColWidth.toString() + "s";
    private final String columnSeparator = ";";
    private static final String nL = System.getProperty("line.separator");
    private final GregorianCalendar calend =
            new GregorianCalendar(TimeZone.getDefault());
    private final int DBG_LEV = 3;

    public CSV(ImgPanel iP, Component parentFr) {
        imgPan = iP;
        parentForMsgBox = parentFr;
        cartesianFl = imgPan.getScale().getCartesianModeFl();
        clockwiseFl = !imgPan.getScale().getChangeDirectionFl();
    }


    /*
     * Public
     */
    public void save() throws SaveSeriesHelpNoDataException,
            SaveSeriesHelpFileException,
            SaveSeriesHelpDirCreateException,
            SaveSeriesHelpDirExistsException {
        if (imgPan.getSeriesBox().getSeriesCount() <= 0) {
            throw new SaveSeriesHelpNoDataException("No series to save.");
        }
        switch (mode) {
            case RB_SAVE_SERIES_MULTI_COL_CSV:
            case RB_SAVE_SERIES_ONE_COL_CSV:
                fileChooseFile();
                break;
            case RB_SAVE_SERIES_ONE_FILE_PER_SERIES_CSV:
                fileChooseDir();
                break;
            default:
                break;
        }
    }

    /*
     *  private
     */
    private void fileChooseDir() throws
            SaveSeriesHelpFileException,
            SaveSeriesHelpDirCreateException,
            SaveSeriesHelpDirExistsException {
        JFileChooser fileChoos = new JFileChooser();
        FileFilterMy fileFilt = new FileFilterMy(FileFilterMy.filtersEnum.DIRS_SAVE);
        Writer output = null;
        int returnVal = -1;
        String fileNameBase = LngSave.text(EnUsSave.SERIES_.name());
        String fileNameExt = ".csv";
        String dirNameBase = "OOoD_";
        DecimalFormat decForm = new DecimalFormat("000");
        int seriesNb = imgPan.getSeriesBox().getSeriesCount();
        fileChoos.addChoosableFileFilter(fileFilt);
        fileChoos.setAcceptAllFileFilterUsed(false);
        fileChoos.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        JFrame frm = (JFrame) parentForMsgBox;
        boolean onTopFl = frm.isAlwaysOnTop();
        if (onTopFl) {
            frm.setAlwaysOnTop(false);
        }
        returnVal = fileChoos.showSaveDialog(parentForMsgBox);
        if (onTopFl) {
            frm.setAlwaysOnTop(true);
        }
        DateFormat dateForm = DateFormat.getDateTimeInstance(
                DateFormat.MEDIUM, DateFormat.MEDIUM);
        calend.setTimeInMillis(System.currentTimeMillis());
        String dateStr = dateForm.format(calend.getTime());
        String dirOutName = dirNameBase + dateStr;
        dirOutName = dirOutName.replace(' ', '_');
        dirOutName = dirOutName.replace(':', '-');
        StringBuilder dirPath = new StringBuilder();
        String dirChoosenName;
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            dirChoosenName = fileChoos.getSelectedFile().getPath();
            if (!(new File(dirChoosenName).exists())) {
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        "Dir doesn't exists: " + dirChoosenName, DBG_LEV);
                throw new SaveSeriesHelpDirExistsException(dirChoosenName);
            }
            dirPath.append(dirChoosenName);
            dirPath.append(File.separator).append(dirOutName);
            File dirFile = new File(dirPath.toString());
            boolean createDirFl;
            if (dirFile.exists()) {
                if (new MsgBox().msgAskTh(
                        LngSave.text(EnUs.DIR_EXISTS.name()),
                        parentForMsgBox,
                        JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION) {
                    return;
                }
                createDirFl = false;
            } else {
                createDirFl = true;
            }
            if (createDirFl) {
                if (!dirFile.mkdir()) {
                    String msg = "Can't create dir: " + dirPath.toString();
                    TraceMyStack.print(Thread.currentThread().getStackTrace(),
                            msg, DBG_LEV);
                    throw new SaveSeriesHelpDirCreateException(dirPath.toString());
                }
            }
            String filePathName = null;
            for (int i = 0; i < seriesNb; ++i) {
                File file = new File(dirFile + File.separator
                        + fileNameBase + decForm.format(i + 1) + fileNameExt);
                try {
                    filePathName = file.getCanonicalPath();
                } catch (IOException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                }
                try {
                    output = new OutputStreamWriter(
                            new FileOutputStream(file), "UTF-8");
                    calend.setTimeInMillis(System.currentTimeMillis());
                    header(output, calend, i);
                    writeSerie(output, i);
                } catch (IOException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                    String msg = "Can't write file: " + filePathName;
                    throw new SaveSeriesHelpFileException(msg);
                } catch (ParseException ex) {
                    TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                            DBG_LEV);
                    String msg = "Can't write file: " + filePathName;
                    throw new SaveSeriesHelpFileException(msg);
                } finally {
                    try {
                        output.close();
                    } catch (Exception ex1) {
                        TraceMyStack.print(ex1.getStackTrace(), ex1.getMessage(),
                                DBG_LEV);
                    }
                }
            }
            MsgBox.msgPlainTh(
                    Lng.text(EnUs.DATA_SAVED_IN_DIR.name())
                    + nL + dirPath,
                    parentForMsgBox, JOptionPane.INFORMATION_MESSAGE);
        }
    }

    private void fileChooseFile() throws SaveSeriesHelpFileException {
        JFileChooser fileChoos = new JFileChooser();
        FileFilterMy fileFilt = new FileFilterMy(FileFilterMy.filtersEnum.CSV_SAVE);
        int returnVal = -1;
        boolean fileExtOKFl = false;
        String filePathName = null;
        Writer output = null;
        fileChoos.addChoosableFileFilter(fileFilt);
        fileChoos.setAcceptAllFileFilterUsed(false);
        fileChoos.setFileSelectionMode(JFileChooser.FILES_ONLY);
        JFrame frm = (JFrame) parentForMsgBox;
        boolean onTopFl = frm.isAlwaysOnTop();
        if (onTopFl) {
            frm.setAlwaysOnTop(false);
        }
        returnVal = fileChoos.showSaveDialog(parentForMsgBox);
        if (onTopFl) {
            frm.setAlwaysOnTop(true);
        }
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fileChoos.getSelectedFile();
            fileExtOKFl = file.getPath().toLowerCase().endsWith(".csv");
            if (!fileExtOKFl) {
                file = new File(file.getPath() + ".csv");
            }
            try {
                filePathName = file.getCanonicalPath();
            } catch (IOException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            }
            if (file.exists()) {
                if (new MsgBox().msgAskTh(
                        Lng.text(EnUs.MSG_FILE_EXISTS.name())
                        + nL + filePathName,
                        parentForMsgBox,
                        JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            try {
                output = new OutputStreamWriter(
                        new FileOutputStream(file), "UTF-8");
                calend.setTimeInMillis(System.currentTimeMillis());
                header(output, calend, 0);
                switch (mode) {
                    case RB_SAVE_SERIES_MULTI_COL_CSV:
                        multiColumn(output);
                        break;
                    case RB_SAVE_SERIES_ONE_COL_CSV:
                        oneColumn(output);
                        break;
                    case RB_SAVE_SERIES_ONE_FILE_PER_SERIES_CSV:
                        break;
                    default:
                        TraceMyStack.print(Thread.currentThread().getStackTrace(),
                                "Save mode doesn't exist.", DBG_LEV);
                        break;
                }
            } catch (IOException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                String msg = "Can't write file: " + filePathName;
                throw new SaveSeriesHelpFileException(msg);
            } catch (ParseException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
                String msg = "Can't write file: " + filePathName;
                throw new SaveSeriesHelpFileException(msg);
            } finally {
                try {
                    output.close();
                } catch (Exception ex1) {
                    TraceMyStack.print(ex1.getStackTrace(), ex1.getMessage(), DBG_LEV);
                }
            }
            MsgBox.msgPlainTh(
                    Lng.text(EnUs.DATA_SAVED_IN_FILE.name())
                    + nL + filePathName,
                    parentForMsgBox, JOptionPane.INFORMATION_MESSAGE);
        }
    }

    private void writeSerie(Writer output, int serieIdx) throws IOException,
            ParseException {
        int pointsNb = imgPan.getSeriesBox().getPointsCount(serieIdx);
        Point2D ptGrph = null;
        output.write("\"" + LngSave.text(EnUsSave.SERIES_NAME.name())
                + "\"" + nL);
        output.write("\""
                + imgPan.getSeriesBox().getSeriesName(serieIdx)
                + "\"" + nL);
        output.write("\"" + LngSave.text(EnUsSave.NUMBER_OF_POINTS.name())
                + "\"" + nL);
        output.write(pointsNb + nL);
        output.write(String.format(
                strFrmtStr, "\""
                + imgPan.getSeriesBox().getXName(serieIdx)
                + "\"") + columnSeparator);
        output.write(String.format(strFrmtStr, "\""
                + imgPan.getSeriesBox().getYName(serieIdx)
                + "\"") + columnSeparator);
        output.write(nL);
        String outStr = "";
        DateFormatEnum dFrmtEnu = imgPan.getScale().getDateFrmtEnu();
        for (int k = 0; k < pointsNb; ++k) {
            ptGrph = imgPan.getSeriesBox().getGraph(serieIdx, k);
            switch (imgPan.getScale().getDateFrmtEnu()) {
                case HHMMSS:
                case YYYYMMDD:
                    outStr = String.format(strFrmtStr,
                            DateFormatEnum.formatDateTime(
                            Math.round(ptGrph.getX()), dFrmtEnu));
                    break;
                default:
                    outStr = String.format(strFrmtSci, ptGrph.getX());
                    break;
            }
            output.write(outStr
                    + columnSeparator
                    + String.format(strFrmtSci, ptGrph.getY())
                    + columnSeparator + nL);
        }
    }

    private void multiColumn(Writer output) throws IOException, ParseException {
        int seriesNb = imgPan.getSeriesBox().getSeriesCount();
        Point2D ptGrph = null;
        int ptsNbMax = imgPan.getSeriesBox().getPointsCount(0);
        int ptsNb = 0;
        for (int i = 0; i < seriesNb; ++i) {
            ptsNb = imgPan.getSeriesBox().getPointsCount(i);
            ptsNbMax = ptsNb > ptsNbMax ? ptsNb : ptsNbMax;
        }
        String outStr = "";
        DateFormatEnum dFrmtEnu = imgPan.getScale().getDateFrmtEnu();
        for (int k = 0; k < ptsNbMax; ++k) {
            for (int i = 0; i < seriesNb; ++i) {
                try {
                    ptGrph = imgPan.getSeriesBox().getGraph(i, k);
                    switch (imgPan.getScale().getDateFrmtEnu()) {
                        case HHMMSS:
                        case YYYYMMDD:
                            outStr = String.format(strFrmtStr,
                                    DateFormatEnum.formatDateTime(
                                    Math.round(ptGrph.getX()), dFrmtEnu));
                            break;
                        default:
                            outStr = String.format(strFrmtSci, ptGrph.getX());
                            break;
                    }
                    output.write(outStr
                            + columnSeparator
                            + String.format(strFrmtSci, ptGrph.getY())
                            + columnSeparator);
                } catch (IndexOutOfBoundsException ex) {
                    output.write(String.format(strFrmtStr, "")
                            + columnSeparator
                            + String.format(strFrmtStr, "")
                            + columnSeparator);
                }
            }
            output.write(nL);
        }
    }

    private void oneColumn(Writer output)
            throws IOException, ParseException {
        int seriesNb = imgPan.getSeriesBox().getSeriesCount();
        for (int i = 0; i < seriesNb; ++i) {
            writeSerie(output, i);
        }
    }

    private void header(Writer output, GregorianCalendar cal,
            int seriesIdx) throws IOException {
        int seriesNb = imgPan.getSeriesBox().getSeriesCount();
        SimpleDateFormat dateFrmt = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat timeFrmt = new SimpleDateFormat("HH:mm:ss");
        dateFrmt.setTimeZone(cal.getTimeZone());
        timeFrmt.setTimeZone(cal.getTimeZone());
        String dateStr = dateFrmt.format(cal.getTime());
        String timeStr = timeFrmt.format(cal.getTime());
        String xScaleType;
        String yScaleType;
        switch (imgPan.getScale().getXScaleTypeEnum()) {
            case PLAIN:
                xScaleType = LngSave.text(EnUsSave.PLAIN.name());
                break;
            case LOG10:
                xScaleType = LngSave.text(EnUsSave.LOG10.name());
                break;
            case TIME:
                xScaleType = LngSave.text(EnUsSave.TIME.name()) + " ["
                        + imgPan.getScale().getDateFrmtEnu().
                        getDateFormat().toLocalizedPattern().toUpperCase()
                        + "]";
                break;
            case DATE:
                xScaleType = LngSave.text(EnUsSave.DATE.name()) + " ["
                        + imgPan.getScale().getDateFrmtEnu().
                        getDateFormat().toLocalizedPattern().toUpperCase()
                        + "] "
                        + imgPan.getScale().getDateFrmtEnu().
                        getDateFormat().getTimeZone().
                        getDisplayName(false, TimeZone.SHORT);
                break;
            default:
                xScaleType = "???";
                break;
        }
        switch (imgPan.getScale().getYScaleTypeEnum()) {
            case PLAIN:
                yScaleType = LngSave.text(EnUsSave.PLAIN.name());
                break;
            case LOG10:
                yScaleType = LngSave.text(EnUsSave.LOG10.name());
                break;
            default:
                yScaleType = "???";
                break;
        }
        String imgPath =
                imgPan.getImgFile() == null
                ? LngSave.text(EnUsSave.NO_IMAGE.name())
                : imgPan.getImgFile().getCanonicalPath();
        output.write(dateStr + nL);
        output.write(timeStr + nL);
        output.write("\""
                + LngSave.text(EnUsSave.IMAGE_FILE.name())
                + "\"" + nL);
        output.write("\"" + imgPath + "\"" + nL);
        output.write("\""
                + LngSave.text(EnUsSave.COORDINATE_SYSTEM.name())
                + "\"" + nL);
        if (cartesianFl) {
            output.write("\""
                    + LngSave.text(EnUsSave.CARTESIAN.name())
                    + "\"" + nL);
            output.write("\""
                    + LngSave.text(EnUsSave.X_AXIS_TYPE.name())
                    + "\"" + nL);
            output.write("\"" + xScaleType + "\"" + nL);
            output.write("\""
                    + LngSave.text(EnUsSave.Y_AXIS_TYPE.name())
                    + "\"" + nL);
            output.write("\"" + yScaleType + "\"" + nL);
        } else {
            output.write("\""
                    + LngSave.text(EnUsSave.POLAR.name())
                    + "\"" + nL);
            output.write("\""
                    + LngSave.text(EnUsSave.CLOCKWISE.name())
                    + "\"" + nL);
            output.write("\"" + clockwiseFl + "\"" + nL);
        }
        output.write("\""
                + LngSave.text(EnUsSave.NUMBER_OF_SERIES.name())
                + "\"" + nL);
        switch (mode) {
            case RB_SAVE_SERIES_ONE_FILE_PER_SERIES_CSV:
                output.write("\"" + (seriesIdx + 1)
                        + "/" + seriesNb + "\"" + nL);
                break;
            case RB_SAVE_SERIES_ONE_COL_CSV:
                output.write(seriesNb + nL);
                break;
            case RB_SAVE_SERIES_MULTI_COL_CSV:
                output.write(seriesNb + nL);
                for (int i = 0; i < seriesNb; ++i) {
                    output.write(String.format(strFrmtStr,
                            "\""
                            + LngSave.text(EnUsSave.SERIES_NAME.name())
                            + "\"")
                            + columnSeparator);
                    output.write(String.format(strFrmtStr, "")
                            + columnSeparator);
                }
                output.write(nL);
                for (int i = 0; i < seriesNb; ++i) {
                    output.write(String.format(
                            strFrmtStr,
                            "\"" + imgPan.getSeriesBox().getSeriesName(i)
                            + "\"")
                            + columnSeparator);
                    output.write(String.format(strFrmtStr, "")
                            + columnSeparator);
                }
                output.write(nL);
                for (int i = 0; i < seriesNb; ++i) {
                    output.write(String.format(
                            strFrmtStr, "\""
                            + LngSave.text(EnUsSave.NUMBER_OF_POINTS.name())
                            + "\"") + columnSeparator);
                    output.write(String.format(strFrmtStr, "")
                            + columnSeparator);
                }
                output.write(nL);
                for (int i = 0; i < seriesNb; ++i) {
                    output.write(String.format(strFrmtStr,
                            imgPan.getSeriesBox().getPointsCount(i))
                            + columnSeparator);
                    output.write(String.format(strFrmtStr, "")
                            + columnSeparator);
                }
                output.write(nL);
                for (int i = 0; i < seriesNb; ++i) {
                    output.write(String.format(strFrmtStr,
                            "\"" + imgPan.getSeriesBox().getXName(i) + "\"")
                            + columnSeparator);
                    output.write(String.format(strFrmtStr,
                            "\"" + imgPan.getSeriesBox().getYName(i) + "\"")
                            + columnSeparator);
                }
                output.write(nL);
                break;
            default:
                TraceMyStack.print(Thread.currentThread().getStackTrace(),
                        "Save mode doesn't exist.", DBG_LEV);
                break;
        }
    }

    public static void setMode(ItmsMSeries m) {
        mode = m;
    }

    public static ItmsMSeries getMode() {
        return mode;
    }

    public class SaveSeriesHelpDirCreateException extends Exception {

        public SaveSeriesHelpDirCreateException(String msg) {
            super(msg);
        }
    }

    public class SaveSeriesHelpDirExistsException extends Exception {

        public SaveSeriesHelpDirExistsException(String msg) {
            super(msg);
        }
    }

    public class SaveSeriesHelpFileException extends Exception {

        public SaveSeriesHelpFileException(String msg) {
            super(msg);
        }
    }

    public class SaveSeriesHelpNoDataException extends Exception {

        public SaveSeriesHelpNoDataException(String msg) {
            super(msg);
        }
    }

    public static void main(String args[]) {
    }
}
