 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.save;

import com.sun.star.lang.XComponent;
import com.sun.star.sheet.XSpreadsheet;
import com.sun.star.uno.XComponentContext;
import java.awt.Component;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import name.mgplldz.guimy.ctrls.DateFormatEnum;
import name.mgplldz.oood.gui.tab.image.ImgPanel;
import name.mgplldz.oood.gui.menu.series.ItmsMSeries;
import name.mgplldz.oood.save.lang.EnUsSave;
import name.mgplldz.oood.save.lang.LngSave;
import name.mgplldz.ooohelp.DocHelp;
import name.mgplldz.ooohelp.RangeHelp;
import name.mgplldz.ooohelp.RangeHelp.RangeHelpSetRngDataFormulaException;
import name.mgplldz.ooohelp.SheetHelp;
import name.mgplldz.ooohelp.SheetHelp.SheetHelpSheetExistsException;
import name.mgplldz.tracemystack.TraceMyStack;

public class Office {

    private XComponentContext context;
    private static ItmsMSeries mode;
    private ImgPanel imgPan;
    private boolean cartesianFl;
    private boolean clockwiseFl;
    private final int DBG_LEV = 3;

    public Office(XComponentContext c, ImgPanel ip, Component parentFr) {
        context = c;
        imgPan = ip;
        cartesianFl = imgPan.getScale().getCartesianModeFl();
        clockwiseFl = !imgPan.getScale().getChangeDirectionFl();
    }

    public void save() {
        if (context == null) {
            return;
        }
        DocHelp docH = new DocHelp(context);
        SheetHelp sheetH = new SheetHelp(context);
        RangeHelp rangeH = new RangeHelp(context);
        XComponent sprCompon = docH.newSpreadshCompon();
        XSpreadsheet sheet = null;
        GregorianCalendar cal;
        String[][] head0;
        Object[][] head1;
        Object[][] head2;
        Object dtaArr[][];
        String formulaArr[][];
        try {
            switch (mode) {
                case RB_SAVE_SERIES_ONE_SHEET:
                    String name = "oood";
                    sheet = sheetH.addNewSheetInDocComponent(name, (short) 0, sprCompon);
                    sheetH.getComponentView(sprCompon).setActiveSheet(sheet);
                    cal = new GregorianCalendar(TimeZone.getDefault());
                    head0 = header0(cal);
                    head1 = header1(true);
                    rangeH.setRngFormulaArrInSheet(0, 0, 0, head0.length - 1,
                            sheet, head0);
                    rangeH.setRngDataArrInSheet(0, head0.length, 0,
                            head0.length + head1.length - 1,
                            sheet, head1);
                    for (int serieIdx = 0; serieIdx
                            < imgPan.getSeriesBox().getSeriesCount();
                            ++serieIdx) {
                        int ptsNb = imgPan.getSeriesBox().getPointsCount(
                                serieIdx);
                        head2 = header2(serieIdx, false);
                        rangeH.setRngDataArrInSheet(serieIdx * 2,
                                head0.length + head1.length,
                                serieIdx * 2 + head2[0].length - 1,
                                head0.length + head1.length + head2.length - 1,
                                sheet, head2);
                        switch (imgPan.getScale().getDateFrmtEnu()) {
                            case HHMMSS:
                            case YYYYMMDD:
                                formulaArr = prepXFormulaCol(serieIdx);
                                rangeH.setRngFormulaArrInSheet(serieIdx * 2,
                                        head0.length + head1.length
                                        + head2.length,
                                        serieIdx * 2,
                                        head0.length + head1.length
                                        + head2.length + ptsNb - 1,
                                        sheet, formulaArr);
                                break;
                            default:
                                dtaArr = prepXDtaCol(serieIdx);
                                rangeH.setRngDataArrInSheet(serieIdx * 2,
                                        head0.length + head1.length
                                        + head2.length,
                                        serieIdx * 2,
                                        head0.length + head1.length
                                        + head2.length + ptsNb - 1,
                                        sheet, dtaArr);
                                break;
                        }
                        dtaArr = prepYDtaCol(serieIdx);
                        rangeH.setRngDataArrInSheet(serieIdx * 2 + 1,
                                head0.length + head1.length
                                + head2.length,
                                serieIdx * 2 + 1,
                                head0.length + head1.length
                                + head2.length + ptsNb - 1,
                                sheet, dtaArr);
                    }
                    break;
                case RB_SAVE_SERIES_ONE_SHEET_PER_SERIES:
                    String nameBase = "oood_s_";
                    XSpreadsheet sheetFrst = null;
                    cal =
                            new GregorianCalendar(TimeZone.getDefault());
                    for (int serieIdx = 0; serieIdx
                            < imgPan.getSeriesBox().getSeriesCount(); ++serieIdx) {
                        sheet = sheetH.addNewSheetInDocComponent(
                                nameBase + (serieIdx + 1), (short) (serieIdx), sprCompon);
                        if (serieIdx == 0) {
                            sheetFrst = sheet;
                        }
                        int ptsNb = imgPan.getSeriesBox().getPointsCount(
                                serieIdx);
                        head0 = header0(cal);
                        head1 = header1(false);
                        rangeH.setRngFormulaArrInSheet(0, 0, 0, head0.length - 1,
                                sheet, head0);
                        rangeH.setRngDataArrInSheet(0, head0.length, 0,
                                head0.length + head1.length - 1,
                                sheet, head1);
                        head2 = header2(serieIdx, true);
                        rangeH.setRngDataArrInSheet(0,
                                head0.length + head1.length,
                                head2[0].length - 1,
                                head0.length + head1.length + head2.length - 1,
                                sheet, head2);
                        switch (imgPan.getScale().getDateFrmtEnu()) {
                            case HHMMSS:
                            case YYYYMMDD:
                                formulaArr = prepXFormulaCol(serieIdx);
                                rangeH.setRngFormulaArrInSheet(0,
                                        head0.length + head1.length
                                        + head2.length,
                                        0,
                                        head0.length + head1.length
                                        + head2.length + ptsNb - 1,
                                        sheet, formulaArr);
                                break;
                            default:
                                dtaArr = prepXDtaCol(serieIdx);
                                rangeH.setRngDataArrInSheet(0,
                                        head0.length + head1.length
                                        + head2.length,
                                        0,
                                        head0.length + head1.length
                                        + head2.length + ptsNb - 1,
                                        sheet, dtaArr);
                                break;
                        }
                        dtaArr = prepYDtaCol(serieIdx);
                        rangeH.setRngDataArrInSheet(1,
                                head0.length + head1.length
                                + head2.length,
                                1,
                                head0.length + head1.length
                                + head2.length + ptsNb - 1,
                                sheet, dtaArr);
                    }
                    sheetH.getComponentView(sprCompon).setActiveSheet(sheetFrst);
                    break;
            }
        } catch (com.sun.star.lang.IndexOutOfBoundsException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        } catch (SheetHelpSheetExistsException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        } catch (RangeHelpSetRngDataFormulaException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        }
    }

    public static void setMode(ItmsMSeries m) {
        mode = m;
    }

    public static ItmsMSeries getMode() {
        return mode;
    }

    private Object[][] header2(int seriesIdx, boolean seriesNbFl) {
        int seriesNb = imgPan.getSeriesBox().getSeriesCount();
        ArrayList<Object> outLstC1 = new ArrayList<Object>();
        ArrayList<Object> outLstC2 = new ArrayList<Object>();
        if (seriesNbFl) {
            outLstC1.add(LngSave.text(EnUsSave.NUMBER_OF_SERIES.name()));
            outLstC2.add("");
            outLstC1.add((seriesIdx + 1) + "/" + seriesNb);
            outLstC2.add("");
        }
        outLstC1.add(LngSave.text(EnUsSave.SERIES_NAME.name()));
        outLstC2.add("");
        outLstC1.add(imgPan.getSeriesBox().getSeriesName(seriesIdx));
        outLstC2.add("");
        outLstC1.add(LngSave.text(EnUsSave.NUMBER_OF_POINTS.name()));
        outLstC2.add("");
        outLstC1.add(imgPan.getSeriesBox().getPointsCount(seriesIdx));
        outLstC2.add("");
        outLstC1.add(imgPan.getSeriesBox().getXName(seriesIdx));
        outLstC2.add(imgPan.getSeriesBox().getYName(seriesIdx));
        Object out[][] = new Object[outLstC1.size()][2];
        for (int i = 0; i < out.length; ++i) {
            out[i][0] = (Object) outLstC1.get(i);
            out[i][1] = (Object) outLstC2.get(i);
        }
        return out;
    }

    private Object[][] header1(boolean seriesNbFl) {
        int seriesNb = imgPan.getSeriesBox().getSeriesCount();
        String xScaleType;
        String yScaleType;
        switch (imgPan.getScale().getXScaleTypeEnum()) {
            case PLAIN:
                xScaleType = LngSave.text(EnUsSave.PLAIN.name());
                break;
            case LOG10:
                xScaleType = LngSave.text(EnUsSave.LOG10.name());
                break;
            case TIME:
                xScaleType = LngSave.text(EnUsSave.TIME.name()) + " ["
                        + imgPan.getScale().getDateFrmtEnu().
                        getDateFormat().toLocalizedPattern().toUpperCase()
                        + "]";
                break;
            case DATE:
                xScaleType = LngSave.text(EnUsSave.DATE.name()) + " ["
                        + imgPan.getScale().getDateFrmtEnu().
                        getDateFormat().toLocalizedPattern().toUpperCase()
                        + "] "
                        + imgPan.getScale().getDateFrmtEnu().
                        getDateFormat().getTimeZone().
                        getDisplayName(false, TimeZone.SHORT);
                break;
            default:
                xScaleType = "?";
                break;
        }
        switch (imgPan.getScale().getYScaleTypeEnum()) {
            case PLAIN:
                yScaleType = LngSave.text(EnUsSave.PLAIN.name());
                break;
            case LOG10:
                yScaleType = LngSave.text(EnUsSave.LOG10.name());
                break;
            default:
                yScaleType = "?";
                break;
        }
        String imgPath = "?";
        try {
            imgPath =
                    imgPan.getImgFile() == null
                    ? LngSave.text(EnUsSave.NO_IMAGE.name())
                    : imgPan.getImgFile().getCanonicalPath();
        } catch (java.io.IOException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
        }
        ArrayList<Object> outLst = new ArrayList<Object>();
        outLst.add(LngSave.text(EnUsSave.IMAGE_FILE.name()));
        outLst.add(imgPath);
        outLst.add(LngSave.text(EnUsSave.COORDINATE_SYSTEM.name()));
        if (cartesianFl) {
            outLst.add(LngSave.text(EnUsSave.CARTESIAN.name()));
            outLst.add(LngSave.text(EnUsSave.X_AXIS_TYPE.name()));
            outLst.add(xScaleType);
            outLst.add(LngSave.text(EnUsSave.Y_AXIS_TYPE.name()));
            outLst.add(yScaleType);
        } else {
            outLst.add(LngSave.text(EnUsSave.POLAR.name()));
            outLst.add(LngSave.text(EnUsSave.CLOCKWISE.name()));
            outLst.add(clockwiseFl);
        }
        if (seriesNbFl) {
            outLst.add(LngSave.text(EnUsSave.NUMBER_OF_SERIES.name()));
            outLst.add(seriesNb);
        }
        Object outArr[][] = new Object[outLst.size()][1];
        for (int i = 0; i < outArr.length; ++i) {
            outArr[i][0] = (Object) outLst.get(i);
        }
        return outArr;
    }

    private String[][] header0(GregorianCalendar cal) {
        ArrayList<String> outLst = new ArrayList<String>();
        outLst.add("=DATE("
                + cal.get(GregorianCalendar.YEAR)
                + ";"
                + DateFormatEnum.monthNb(cal.get(GregorianCalendar.MONTH))
                + ";"
                + cal.get(GregorianCalendar.DAY_OF_MONTH)
                + ")");
        outLst.add("=TIME("
                + cal.get(GregorianCalendar.HOUR_OF_DAY)
                + ";"
                + cal.get(GregorianCalendar.MINUTE)
                + ";"
                + cal.get(GregorianCalendar.SECOND)
                + ")");
        String outArr[][] = new String[outLst.size()][1];
        for (int i = 0; i < outArr.length; ++i) {
            outArr[i][0] = outLst.get(i);
        }
        return outArr;
    }

    public Object[][] prepXDtaCol(int seriesIdx) {
        int ptsNb = imgPan.getSeriesBox().getPointsCount(seriesIdx);
        Object dtaArrCol[][] = new Object[ptsNb][1];
        for (int k = 0; k < ptsNb; ++k) {
            dtaArrCol[k][0] = imgPan.getSeriesBox().getGraph(seriesIdx, k).getX();
        }
        return dtaArrCol;
    }

    public Object[][] prepYDtaCol(int seriesIdx) {
        int ptsNb = imgPan.getSeriesBox().getPointsCount(seriesIdx);
        Object dtaArrCol[][] = new Object[ptsNb][1];
        for (int k = 0; k < ptsNb; ++k) {
            dtaArrCol[k][0] = imgPan.getSeriesBox().getGraph(seriesIdx, k).getY();
        }
        return dtaArrCol;
    }

    public String[][] prepXFormulaCol(int seriesIdx) {
        int ptsNb = imgPan.getSeriesBox().getPointsCount(seriesIdx);
        String formualArrCol[][] = new String[ptsNb][1];
        GregorianCalendar cal =
                new GregorianCalendar(TimeZone.getTimeZone("GMT"));
        long millis;
        long hours;
        long minutes;
        long seconds;
        switch (imgPan.getScale().getDateFrmtEnu()) {
            case HHMMSS:
                for (int k = 0; k < ptsNb; ++k) {
                    millis = Math.round(
                            imgPan.getSeriesBox().getGraph(seriesIdx, k).getX());
                    hours = millis / DateFormatEnum.MLS_HOUR;
                    millis -= hours * DateFormatEnum.MLS_HOUR;
                    minutes = millis / DateFormatEnum.MLS_MIN;
                    millis -= minutes * DateFormatEnum.MLS_MIN;
                    seconds = millis / DateFormatEnum.MLS_SEC;
                    formualArrCol[k][0] = "=TIME("
                            + hours
                            + ";"
                            + minutes
                            + ";"
                            + seconds;
                }
                break;
            case YYYYMMDD:
                for (int k = 0; k < ptsNb; ++k) {
                    cal.setTimeInMillis(Math.round(
                            imgPan.getSeriesBox().getGraph(seriesIdx, k).getX()));
                    formualArrCol[k][0] = "=DATE("
                            + cal.get(GregorianCalendar.YEAR)
                            + ";"
                            + DateFormatEnum.monthNb(cal.get(
                            GregorianCalendar.MONTH))
                            + ";"
                            + cal.get(GregorianCalendar.DAY_OF_MONTH);
                }
                break;
            default:
                break;
        }
        return formualArrCol;
    }
}
