 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman, Laurent Balland-Poirier
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *    Laurent Balland-Poirier: jumbo4444@yahoo.fr
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.oood.scale;

import java.awt.Point;
import java.awt.geom.Point2D;
import java.lang.reflect.Field;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import name.mgplldz.guimy.ctrls.DateFormatEnum;
import name.mgplldz.tracemystack.TraceMyStack;

public final class Scale {

    public enum ScaleTypeEnum {

        PLAIN,
        LOG10,
        DATE,
        TIME;
    }
    private Point xAxisP1Scr;
    private Point xAxisP2Scr;
    private Point yAxisP1Scr;
    private Point yAxisP2Scr;
    private Point2D xAxisP1Grph;
    private Point2D xAxisP2Grph;
    private Point2D yAxisP1Grph;
    private Point2D yAxisP2Grph;
    private double xSlopeScr;
    private double ySlopeScr;
    private double xShiftScr;
    private double yShiftScr;
    private double rLengthScr;
    private double rAngleScr;
    private double rLengthGrph;
    private double rAngleGrph;
    private boolean cartesianModeFl;
    private boolean changeDirectionFl = true;
    private DateFormatEnum dateFrmtEnu = DateFormatEnum.NO_DATE;
    private ScaleTypeEnum xScaleTypeEnu = ScaleTypeEnum.PLAIN;
    private ScaleTypeEnum yScaleTypeEnu = ScaleTypeEnum.PLAIN;
    private final int DBG_LEV = 3;

    enum coordSys {

        CARTESIAN,
        POLAR
    }

    public Scale() {
    }

    public Scale(Point xAxis1Scr, Point xAxis2Scr,
            Point yAxis1Scr, Point yAxis2Scr) {
        scaleCartes(xAxis1Scr, xAxis2Scr, yAxis1Scr, yAxis2Scr);
    }

    public Scale(Point xAxis1Scr, Point xAxis2Scr) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), toString(),
                DBG_LEV);
        scalePolar(xAxis1Scr, xAxis2Scr);
    }

    /*
     * Public
     */
    public static Point2D crossPoint(double xAxisSlope, double xAxisShift,
            double yAxisSlope, double yAxisShift) {
        double x, y;
        if (Double.isInfinite(yAxisSlope)) {
            x = yAxisShift;
            y = xAxisShift - xAxisSlope * x;
        } else {
            x = (yAxisSlope - yAxisShift) / (xAxisSlope - yAxisSlope);
            y = yAxisSlope - xAxisSlope * x;
        }
        return new Point.Double(x, y);
    }

    public static double slope(Point a, Point b) {
        int dxInt = b.x - a.x;
        int dyInt = a.y - b.y;
        return (double) dyInt / (double) dxInt;
    }

    public static double shift(Point2D a, double slope) {
        if (Double.isInfinite(slope)) {
            return a.getX();
        } else {
            return a.getY() + slope * a.getX();
        }
    }

    public double getYAxisAngleRad() {
        double degs = Math.toDegrees(Math.atan(-ySlopeScr));
        if (degs >= 0.0) {
            degs -= 90.0;
        } else {
            degs += 90.0;
        }
        return Math.toRadians(degs);
    }

    public void setXScaleTypeEnum(ScaleTypeEnum type) {
        xScaleTypeEnu = type;
        switch (xScaleTypeEnu) {
            case PLAIN:
            case LOG10:
                dateFrmtEnu = DateFormatEnum.NO_DATE;
                break;
            case TIME:
                dateFrmtEnu = DateFormatEnum.HHMMSS;
                break;
            case DATE:
                dateFrmtEnu = DateFormatEnum.YYYYMMDD;
                break;
            default:
                dateFrmtEnu = DateFormatEnum.NO_DATE;
                break;
        }
    }

    public ScaleTypeEnum getXScaleTypeEnum() {
        return xScaleTypeEnu;
    }

    public void setYScaleTypeEnum(ScaleTypeEnum type) {
        yScaleTypeEnu = type;
    }

    public ScaleTypeEnum getYScaleTypeEnum() {
        return yScaleTypeEnu;
    }

    public void scaleCartes(Point xAxis1Scr, Point xAxis2Scr,
            Point yAxis1Scr, Point yAxis2Scr) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), toString(),
                DBG_LEV);
        cartesianModeFl = true;
        xAxisP1Scr = new Point(xAxis1Scr.x, xAxis1Scr.y);
        xAxisP2Scr = new Point(xAxis2Scr.x, xAxis2Scr.y);
        yAxisP1Scr = new Point(yAxis1Scr.x, yAxis1Scr.y);
        yAxisP2Scr = new Point(yAxis2Scr.x, yAxis2Scr.y);
        xSlopeScr = slope(xAxisP1Scr, xAxisP2Scr);
        ySlopeScr = slope(yAxisP1Scr, yAxisP2Scr);
        System.out.println("xSlopeScr: " + xSlopeScr);
        System.out.println("ySlopeScr: " + ySlopeScr);
        xShiftScr = shift(new Point.Double(xAxisP1Scr.x, xAxisP1Scr.y), xSlopeScr);
        yShiftScr = shift(new Point.Double(yAxisP1Scr.x, yAxisP1Scr.y), ySlopeScr);
        System.out.println("xShiftScr: " + xShiftScr);
        System.out.println("yShiftScr: " + yShiftScr);
    }

    public final void scalePolar(Point xAxis1Scr, Point xAxis2Scr) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), toString(),
                DBG_LEV);
        cartesianModeFl = false;
        xAxisP1Scr = xAxis1Scr;
        xAxisP2Scr = xAxis2Scr;
        rLengthScr = distance(xAxisP1Scr, xAxisP2Scr);
        rAngleScr = angle(
                new Point.Double(xAxisP1Scr.x, xAxisP1Scr.y),
                new Point.Double(xAxisP2Scr.x, xAxisP2Scr.y));
    }

    public DateFormatEnum getDateFrmtEnu() {
        return dateFrmtEnu;
    }

    public void setGrph(
            Object xAxisP1XOrLenghtObj,
            Double xAxisP1YOrDegrees,
            Object xAxisP2XObj,
            Double xAxisP2Y,
            Object yAxisP1XObj,
            Double yAxisP1Y,
            Object yAxisP2XObj,
            Double yAxisP2Y)
            throws
            ScaleLogException {
        /*
         * Cartesian
         */
        if (cartesianModeFl) {
            if (xAxisP1XOrLenghtObj == null
                    || xAxisP1YOrDegrees == null
                    || xAxisP2XObj == null
                    || xAxisP2Y == null
                    || yAxisP1XObj == null
                    || yAxisP1Y == null
                    || yAxisP2XObj == null
                    || yAxisP2Y == null) {
                throw new IllegalArgumentException("Null value in scale.");
            }
            Double xAxisP1XOrLenght = null;
            Double xAxisP2X = null;
            Double yAxisP1X = null;
            Double yAxisP2X = null;
            SimpleDateFormat dateFrmt = null;
            try {
                dateFrmt = dateFrmtEnu.getDateFormat();
            } catch (NullPointerException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                        DBG_LEV);
            }
            switch (xScaleTypeEnu) {
                case PLAIN:
                    xAxisP1XOrLenght = (Double) xAxisP1XOrLenghtObj;
                    xAxisP2X = (Double) xAxisP2XObj;
                    yAxisP1X = (Double) yAxisP1XObj;
                    yAxisP2X = (Double) yAxisP2XObj;
                    break;
                case LOG10:
                    xAxisP1XOrLenght = (Double) xAxisP1XOrLenghtObj;
                    xAxisP2X = (Double) xAxisP2XObj;
                    yAxisP1X = (Double) yAxisP1XObj;
                    yAxisP2X = (Double) yAxisP2XObj;
                    if (xAxisP1XOrLenght <= 0
                            || xAxisP2X <= 0
                            || yAxisP1X <= 0
                            || yAxisP2X <= 0) {
                        throw new ScaleLogException(
                                "X Log10, X must be > 0");
                    }
                    xAxisP1XOrLenght = Math.log10(xAxisP1XOrLenght);
                    xAxisP2X = Math.log10(xAxisP2X);
                    yAxisP1X = Math.log10(yAxisP1X);
                    yAxisP2X = Math.log10(yAxisP2X);
                    break;
                case TIME:
                    try {
                        xAxisP1XOrLenght = new Double(dateFrmt.parse(
                                (String) xAxisP1XOrLenghtObj).getTime());
                        xAxisP2X = new Double(dateFrmt.parse(
                                (String) xAxisP2XObj).getTime());
                        yAxisP1X = new Double(dateFrmt.parse(
                                (String) yAxisP1XObj).getTime());
                        yAxisP2X = new Double(dateFrmt.parse(
                                (String) yAxisP2XObj).getTime());
                    } catch (ParseException ex) {
                        TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                                DBG_LEV);
                    }
                    break;
                case DATE:
                    try {
                        xAxisP1XOrLenght = new Double(dateFrmt.parse(
                                (String) xAxisP1XOrLenghtObj).getTime());
                        xAxisP2X = new Double(dateFrmt.parse(
                                (String) xAxisP2XObj).getTime());
                        yAxisP1X = new Double(dateFrmt.parse(
                                (String) yAxisP1XObj).getTime());
                        yAxisP2X = new Double(dateFrmt.parse(
                                (String) yAxisP2XObj).getTime());
                    } catch (ParseException ex) {
                        TraceMyStack.print(ex.getStackTrace(), ex.getMessage(),
                                DBG_LEV);
                    }
                    break;
                default:
                    break;
            }
            switch (yScaleTypeEnu) {
                case PLAIN:
                    break;
                case LOG10:
                    if (xAxisP1YOrDegrees <= 0
                            || xAxisP2Y <= 0
                            || yAxisP1Y <= 0
                            || yAxisP2Y <= 0) {
                        throw new ScaleLogException(
                                "Y Log10, Y must be > 0");
                    }
                    xAxisP1YOrDegrees = Math.log10(xAxisP1YOrDegrees);
                    xAxisP2Y = Math.log10(xAxisP2Y);
                    yAxisP1Y = Math.log10(yAxisP1Y);
                    yAxisP2Y = Math.log10(yAxisP2Y);
                    break;
                default:
                    break;

            }
            xAxisP1Grph = new Point.Double(xAxisP1XOrLenght,
                    xAxisP1YOrDegrees);
            xAxisP2Grph = new Point.Double(xAxisP2X, xAxisP2Y);
            yAxisP1Grph = new Point.Double(yAxisP1X, yAxisP1Y);
            yAxisP2Grph = new Point.Double(yAxisP2X, yAxisP2Y);
            /*
             * Polar
             */
        } else {
            Double xAxisP1XOrLenght = (Double) xAxisP1XOrLenghtObj;
            rLengthGrph = xAxisP1XOrLenght;
            rAngleGrph = xAxisP1YOrDegrees;
        }
    }

    public void recalcScr() {
        if (cartesianModeFl) {
            xSlopeScr = slope(xAxisP1Scr, xAxisP2Scr);
            ySlopeScr = slope(yAxisP1Scr, yAxisP2Scr);
            xShiftScr = shift(new Point.Double(xAxisP1Scr.x, xAxisP1Scr.y), xSlopeScr);
            yShiftScr = shift(new Point.Double(yAxisP1Scr.x, yAxisP1Scr.y), ySlopeScr);
        } else {
            rLengthScr = distance(xAxisP1Scr, xAxisP2Scr);
            rAngleScr = angle(
                    new Point.Double(xAxisP1Scr.x, xAxisP1Scr.y),
                    new Point.Double(xAxisP2Scr.x, xAxisP2Scr.y));
        }
    }

    public void setXAxisP1Scr(Point p) {
        xAxisP1Scr.x = p.x;
        xAxisP1Scr.y = p.y;
    }

    public Point getXAxisP1Scr() {
        return xAxisP1Scr;
    }

    public void setXAxisP2Scr(Point p) {
        xAxisP2Scr.x = p.x;
        xAxisP2Scr.y = p.y;
    }

    public Point getXAxisP2Scr() {
        return xAxisP2Scr;
    }

    public void setYAxisP1Scr(Point p) {
        yAxisP1Scr.x = p.x;
        yAxisP1Scr.y = p.y;
    }

    public Point getYAxisP1Scr() {
        return yAxisP1Scr;
    }

    public void setYAxisP2Scr(Point p) {
        yAxisP2Scr.x = p.x;
        yAxisP2Scr.y = p.y;
    }

    public Point getYAxisP2Scr() {
        return yAxisP2Scr;
    }

    public Point2D getXAxisP1Grph() {
        return xAxisP1Grph;
    }

    public Point2D getXAxisP2Grph() {
        return xAxisP2Grph;
    }

    public Point2D getYAxisP1Grph() {
        return yAxisP1Grph;
    }

    public Point2D getYAxisP2Grph() {
        return yAxisP2Grph;
    }

    public Point2D scr2Grph(Point ptScr) {
        if (cartesianModeFl) {
            Point2D ptGrph = cartesian(ptScr);
            switch (xScaleTypeEnu) {
                case LOG10:
                    ptGrph.setLocation(Math.pow(10, ptGrph.getX()),
                            ptGrph.getY());
                    break;
                default:
                    break;
            }
            switch (yScaleTypeEnu) {
                case LOG10:
                    ptGrph.setLocation(ptGrph.getX(),
                            Math.pow(10, ptGrph.getY()));
                    break;
                default:
                    break;
            }
            return ptGrph;
        } else {
            return polar(ptScr);
        }
    }

    public void setXYLog10(boolean xLog, boolean yLog) {
        if (xLog) {
            xScaleTypeEnu = ScaleTypeEnum.LOG10;
        }
        if (yLog) {
            yScaleTypeEnu = ScaleTypeEnum.LOG10;
        }
    }

    public boolean getCartesianModeFl() {
        return cartesianModeFl;
    }

    public void setChangeDirectionFl(boolean fl) {
        changeDirectionFl = fl;
    }

    public boolean getChangeDirectionFl() {
        return changeDirectionFl;
    }

    /*
     *  private
     */
    private double distance(Point a, Point b) {
        if (a.equals(b)) {
            return 0;
        }
        return Math.hypot(b.x - a.x, b.y - a.y);
    }

    private Point2D crossPointXAxis(double axisSlope, double axisShift,
            double slope2, double shift2) {
        double x, y;
        if (Double.isInfinite(slope2)) {
            x = Double.POSITIVE_INFINITY;
            y = axisShift;
        } else {
            x = (axisShift - shift2) / (axisSlope - slope2);
            y = axisShift - axisSlope * x;
        }
        return new Point.Double(x, y);
    }

    private Point2D crossPointYAxis(double axisSlope, double axisShift,
            double slope2, double shift2) {
        double x, y;
        if (Double.isInfinite(axisSlope)) {
            x = yShiftScr;
            y = shift2 - slope2 * x;
        } else {
            x = (axisShift - shift2) / (axisSlope - slope2);
            y = axisShift - axisSlope * x;
        }
        return new Point.Double(x, y);
    }

    private double angle(Point2D a, Point2D b) {
        double dx = b.getX() - a.getX();
        double dy = changeDirectionFl
                ? a.getY() - b.getY()
                : b.getY() - a.getY();
        double length = Math.hypot(dx, dy);
        double sin = dy / length;
        if (dx >= 0 && dy >= 0) {
            return Math.toDegrees(Math.asin(sin));
        }
        if (dx >= 0 && dy < 0) {
            return 360 + Math.toDegrees(Math.asin(sin));
        }
        if (dx < 0) {
            return 180 - Math.toDegrees(Math.asin(sin));
        }
        return Double.NaN;
    }

    private Point2D calcCrossAxisPointInScreenCoord(Point2D p) {
        double x, bx, by;
        bx = p.getY() + xSlopeScr * p.getX();
        by = p.getY() + ySlopeScr * p.getX();
        Point2D crossXAxis = new Point.Double();
        Point2D crossYAxis = new Point.Double();
        crossXAxis.setLocation(
                crossPointXAxis(xSlopeScr, xShiftScr, ySlopeScr, by));
        crossYAxis.setLocation(
                crossPointYAxis(ySlopeScr, yShiftScr, xSlopeScr, bx));
        if (Double.isInfinite(crossXAxis.getX())) {
            crossXAxis.setLocation(p.getX(), crossXAxis.getY());
        }
        System.out.println("bx:by: " + bx + ":" + by);
        System.out.println("X-axis cross: " + crossXAxis.getX() + ":"
                + crossXAxis.getY());
        System.out.println("Y-axis cross: " + crossYAxis.getX() + ":"
                + crossYAxis.getY());
        return new Point.Double(crossXAxis.getX(), crossYAxis.getY());
    }

    private Point2D calcPointInGraphCoord(Point2D xyCrossAxis) {
        double x = (xAxisP2Grph.getX() - xAxisP1Grph.getX())
                * (xyCrossAxis.getX() - xAxisP1Scr.x)
                / (xAxisP2Scr.getX() - xAxisP1Scr.getX()) + xAxisP1Grph.getX();
        double y = (yAxisP2Grph.getY() - yAxisP1Grph.getY())
                * (xyCrossAxis.getY() - yAxisP1Scr.getY())
                / (yAxisP2Scr.getY() - yAxisP1Scr.getY()) + yAxisP1Grph.getY();
        return new Point.Double(x, y);
    }

    private Point2D cartesian(Point pt) {
        Point2D xyCross = calcCrossAxisPointInScreenCoord(
                new Point.Double(pt.x, pt.y));
        System.out.println("xyCrossAxis: " + xyCross.getX() + ":" + xyCross.getY());
        return calcPointInGraphCoord(xyCross);
    }

    private Point2D polar(Point pt) {
        if (pt.equals(xAxisP1Scr)) {
            return new Point.Double(0, 0);
        }
        double ptAngle = (angle(
                new Point.Double(xAxisP1Scr.x, xAxisP1Scr.y),
                new Point.Double(pt.x, pt.y))
                + (rAngleGrph - rAngleScr + 360)) % 360;
        double ptLengthScr = distance(xAxisP1Scr, pt);
        double ptLengthGrph = rLengthGrph / rLengthScr * ptLengthScr;
        return new Point.Double(ptLengthGrph, ptAngle);
    }

    /*
     * clasess
     */
    public class ScaleLogException extends Exception {

        public ScaleLogException(String s) {
            super(s);
        }
    }

    @Override
    public String toString() {
        StringBuilder result = new StringBuilder();
        String NEW_LINE = System.getProperty("line.separator");
        Field fldsArr[] = this.getClass().getDeclaredFields();
        result.append(this.getClass().getName()).append("{").append(NEW_LINE);
        for (int i = 0; i < fldsArr.length; ++i) {
            try {
                result.append("\t").append(fldsArr[i].getName()).append(" : ").
                        append(fldsArr[i].get(this)).append(NEW_LINE);
            } catch (IllegalArgumentException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            } catch (IllegalAccessException ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), DBG_LEV);
            }
        }
        result.append("}");
        return result.toString();
    }

    /*
     * test
     */
    public static void main(String args[]) {
        try {
            cartesianTest();
            // polarTest();
        } catch (ScaleLogException ex) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    ex.getMessage(), 3);
        }
    }

    private static void cartesianTest() throws ScaleLogException {
        Point x1Scr = new Point(20, 180);
        Point x2Scr = new Point(180, 180);
        Point y1Scr = new Point(20, 180);
        Point y2Scr = new Point(20, 20);
        GregorianCalendar calXAxisP1X =
                new GregorianCalendar(TimeZone.getTimeZone("GMT"));
        GregorianCalendar calXAxisP2X =
                new GregorianCalendar(TimeZone.getTimeZone("GMT"));
//        Point2D xP1Grph = new Point.Double(
//                calXAxisP1X.getTime().getTime(), 0);
//        Point2D xP2Grph = new Point.Double(
//                calXAxisP2X.getTime().getTime(), 100);
//        Point2D yP1Grph = new Point.Double(
//                calXAxisP1X.getTime().getTime(), 10);
//        Point2D yP2Grph = new Point.Double(
//                calXAxisP1X.getTime().getTime(), 10);
        Point2D xP1Grph = new Point.Double(0, 0);
        Point2D xP2Grph = new Point.Double(1000, 0);
        Point2D yP1Grph = new Point.Double(0, 0);
        Point2D yP2Grph = new Point.Double(1000, 1000);
        Point pt = new Point(30, 20);
        Scale sclCart = new Scale(x1Scr, x2Scr, y1Scr, y2Scr);
        sclCart.setXScaleTypeEnum(ScaleTypeEnum.PLAIN);
//        SimpleDateFormat timeFrmt = sclCart.getDateFrmtEnu().getDateFormat();
//        sclCart.setXScaleTypeEnum(ScaleTypeEnum.TIME);
//        try {
//            calXAxisP1X.setTime(timeFrmt.parse("10:00:00"));
//            calXAxisP2X.setTime(timeFrmt.parse("20:00:00"));
//        } catch (ParseException ex) {
//            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
//        }
        sclCart.setXYLog10(false, false);
        sclCart.setGrph(xP1Grph.getX(), xP1Grph.getY(),
                xP2Grph.getX(), xP2Grph.getY(),
                yP1Grph.getX(), yP1Grph.getY(),
                yP2Grph.getX(), yP2Grph.getY());
        Point2D ptOut = sclCart.scr2Grph(pt);
        System.out.println("Final: " + ptOut.getX() + ":" + ptOut.getY());
//        System.out.println(sclCart.toString());
//        GregorianCalendar calOut =
//                new GregorianCalendar(TimeZone.getTimeZone("GMT"));
//        calOut.setTime(new Date((long) ptOut.getX()));
//        System.out.println("pt graph: "
//                + timeFrmt.format(calOut.getTime().getTime())
//                + ":" + ptOut.getY());
    }

    private static void polarTest() throws ScaleLogException {
        Point pt = new Point(100, 200);
        Point polP1 = new Point(100, 100);
        Point polP2 = new Point(200, 200);
        Point2D lengthDegrees = new Point.Double(100, 0);
        Scale sclPol = new Scale(polP1, polP2);
        sclPol.setChangeDirectionFl(true);
        sclPol.setGrph(lengthDegrees.getX(), lengthDegrees.getY(),
                null, null,
                null, null,
                null, null);
        Point2D ptOut = sclPol.scr2Grph(pt);
//        System.out.println(sclPol.toString());
//        System.out.println("pt graph: " + ptOut.getX() + ":" + ptOut.getY());
    }
}
