 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.ooohelp;

import com.sun.star.awt.Size;
import com.sun.star.beans.XPropertySet;
import com.sun.star.chart.XChartDocument;
import com.sun.star.chart.XDiagram;
import com.sun.star.chart2.XChartType;
import com.sun.star.container.XNameAccess;
import com.sun.star.document.XEmbeddedObjectSupplier;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.sheet.XCellRangeAddressable;
import com.sun.star.sheet.XSpreadsheet;
import com.sun.star.table.CellRangeAddress;
import com.sun.star.table.XCellRange;
import com.sun.star.table.XTableChart;
import com.sun.star.table.XTableChartsSupplier;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import java.awt.Color;
import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.ooohelp.SheetHelp.SheetHelpSheetExistsException;
import name.mgplldz.tracemystack.TraceMyStack;

public class ChartHelp {

    private XComponentContext m_xContext;

    public ChartHelp(XComponentContext m_xContext) {
        this.m_xContext = m_xContext;
    }

    public void chartExample() {
        Object chart = null;
        XSpreadsheet spreadsh = null;
        SheetHelp sht = new SheetHelp(m_xContext);
        spreadsh = sht.getActiveSheet();
        chart = chartEmptyCreateInSheet("chart1", spreadsh);
        chartAddSerie(chart, "c1:c3", "d1:d3");
        formatSerieLastInChart(chart, true, true,
                200, 100,
                new Color(255, 0, 255).getRGB(),
                com.sun.star.chart.ChartSymbolType.AUTO);
        chartAddSerie(chart, "a1:a3", "b1:b3");
        formatSerieLastInChart(chart, true, true,
                200, 100,
                new Color(0, 0, 255).getRGB(),
                com.sun.star.chart.ChartSymbolType.AUTO);
    }

    public Object chartEmptyCreateInSheet(String chartName, XSpreadsheet spreadsh) {
        Object chartFromCollection = null;
        String dtaRngName = "a1:b1";

        chartFromCollection = addChartToCollectionInSheet(dtaRngName, chartName, spreadsh);
        addDiagramToChart(chartFromCollection, "XYDiagram");
        removeFirstChartType(chartFromCollection, 0);
        return chartFromCollection;
    }

    public void chartAddSerie(Object chartFromCollect,
            String xRange, String yRange) {
        com.sun.star.chart2.data.XLabeledDataSequence xLabSeq = null;
        com.sun.star.chart2.data.XLabeledDataSequence yLabSeq = null;
        com.sun.star.chart2.XDataSeriesContainer dataSeriesCont = null;

        xLabSeq = createSerieInChart(xRange, "values-x", chartFromCollect);
        yLabSeq = createSerieInChart(yRange, "values-y", chartFromCollect);
        dataSeriesCont = createChartType("ScatterChartType", chartFromCollect);
        addSerieToSerieCont(xLabSeq, yLabSeq, dataSeriesCont);
    }

    public void chartCreateInSheet_old(String dtaRngName, String chartName, XSpreadsheet spreadsh) {
        Object chartFromCollection = null;
        com.sun.star.chart2.data.XLabeledDataSequence xLabSeq = null;
        com.sun.star.chart2.data.XLabeledDataSequence yLabSeq = null;
        com.sun.star.chart2.XDataSeriesContainer dataSeriesCont = null;

        chartFromCollection = addChartToCollectionInSheet(dtaRngName, chartName, spreadsh);
        addDiagramToChart(chartFromCollection, "XYDiagram");
        removeFirstChartType(chartFromCollection, 0);
        xLabSeq = createSerieInChart("$Sheet1.$c$1:$c$3", "values-x", chartFromCollection);
        yLabSeq = createSerieInChart("$Sheet1.$d$1:$d$3", "values-y", chartFromCollection);
        dataSeriesCont = createChartType("ScatterChartType", chartFromCollection);
        addSerieToSerieCont(xLabSeq, yLabSeq, dataSeriesCont);
        formatSerieByIdxInChart(chartFromCollection, 1, true, true,
                200, 100,
                new Color(255, 0, 255).getRGB(),
                com.sun.star.chart.ChartSymbolType.AUTO);
        // 2'nd serie
        xLabSeq = createSerieInChart("$Sheet1.$e1:$e3", "values-x", chartFromCollection);
        yLabSeq = createSerieInChart("$Sheet1.$f1:$f3", "values-y", chartFromCollection);
        dataSeriesCont = createChartType("ScatterChartType", chartFromCollection);
        addSerieToSerieCont(xLabSeq, yLabSeq, dataSeriesCont);
        formatSerieByIdxInChart(chartFromCollection, 2, true, true,
                200, 100,
                new Color(0, 0, 255).getRGB(),
                com.sun.star.chart.ChartSymbolType.AUTO);
        CellRangeAddress[] cellRngAddresses = getDataRangesInChart(chartFromCollection);
    }

    public Object getChartByNameFromSheet(String chartName, XSpreadsheet spreadsh)
            throws ChartHelpException {
        XTableChartsSupplier chartTableSupply = null;
        Object chartFromCollection = null;

        chartTableSupply = (XTableChartsSupplier) UnoRuntime.queryInterface(
                XTableChartsSupplier.class, spreadsh);
        com.sun.star.table.XTableCharts aChartCollection =
                chartTableSupply.getCharts();
        XNameAccess aChartCollectionNA = (XNameAccess) UnoRuntime.queryInterface(
                XNameAccess.class, aChartCollection);
        try {
            chartFromCollection = aChartCollectionNA.getByName(chartName);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
            throw new ChartHelpException("Chart " + "["
                    + chartName + "]" + " doesn't exists");
        }
        return chartFromCollection;
    }

    public Object getChartByNameFromSheet(String chartName, String sheetName)
            throws Exception {
        SheetHelp sheetH = new SheetHelp(m_xContext);
        XTableChartsSupplier chartTableSupply = null;
        Object chartFromCollection = null;
        XSpreadsheet sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        chartTableSupply = (XTableChartsSupplier) UnoRuntime.queryInterface(
                XTableChartsSupplier.class, sheet);
        com.sun.star.table.XTableCharts aChartCollection =
                chartTableSupply.getCharts();
        XNameAccess aChartCollectionNA = (XNameAccess) UnoRuntime.queryInterface(
                XNameAccess.class, aChartCollection);

        try {
            chartFromCollection = aChartCollectionNA.getByName(chartName);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return chartFromCollection;
    }

    public Object addChartToCollectionInSheet(String dtaRngName,
            String chartName, XSpreadsheet spreadsh) {
        XTableChartsSupplier chartTableSupply = null;
        Object chartFromCollection = null;

        XCellRange cellRange = spreadsh.getCellRangeByName(dtaRngName);

        XCellRangeAddressable xCellRangeAddress =
                (XCellRangeAddressable) UnoRuntime.queryInterface(
                XCellRangeAddressable.class, cellRange);

        CellRangeAddress aCellRangeAddress = xCellRangeAddress.getRangeAddress();

        CellRangeAddress aAddresses[] = {aCellRangeAddress};

        chartTableSupply = (XTableChartsSupplier) UnoRuntime.queryInterface(
                XTableChartsSupplier.class, spreadsh);
        com.sun.star.table.XTableCharts aChartCollection =
                chartTableSupply.getCharts();
        XNameAccess aChartCollectionNA = (XNameAccess) UnoRuntime.queryInterface(
                XNameAccess.class, aChartCollection);
        com.sun.star.awt.Rectangle aRect =
                new com.sun.star.awt.Rectangle(1000, 1000, 15000, 9271);
        aChartCollection.addNewByName(chartName, aRect, aAddresses, false, false);
        try {
            chartFromCollection = aChartCollectionNA.getByName(chartName);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return chartFromCollection;
    }

    public Object addChartToCollectionInSheet(String dtaRngName,
            String chartName, String spreadshName)
            throws Exception, ChartHelpException {
        XTableChartsSupplier chartTableSupply = null;
        Object chartFromCollection = null;
        SheetHelp sheetH = new SheetHelp(m_xContext);
        XSpreadsheet spreadsh = sheetH.getSheetByNameInActiveDoc(spreadshName);
        XCellRange cellRange = spreadsh.getCellRangeByName(dtaRngName);
        XCellRangeAddressable xCellRangeAddress =
                (XCellRangeAddressable) UnoRuntime.queryInterface(
                XCellRangeAddressable.class, cellRange);
        CellRangeAddress aCellRangeAddress = xCellRangeAddress.getRangeAddress();
        CellRangeAddress aAddresses[] = {aCellRangeAddress};
        chartTableSupply = (XTableChartsSupplier) UnoRuntime.queryInterface(
                XTableChartsSupplier.class, spreadsh);
        com.sun.star.table.XTableCharts aChartCollection =
                chartTableSupply.getCharts();
        XNameAccess aChartCollectionNA = (XNameAccess) UnoRuntime.queryInterface(
                XNameAccess.class, aChartCollection);
        if (aChartCollectionNA != null && aChartCollectionNA.hasByName(chartName)) {
            String msg = this.getClass().getName() + "\n"
                    + "Chart exists: " + "chartName";
            throw new Exception(msg);
        }
        com.sun.star.awt.Rectangle aRect =
                new com.sun.star.awt.Rectangle(1000, 1000, 15000, 9271);
        aChartCollection.addNewByName(chartName, aRect, aAddresses, false, false);
        try {
            chartFromCollection = aChartCollectionNA.getByName(chartName);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return chartFromCollection;
    }

    public void addDiagramToChart(Object chartFromCollection,
            String diagramType) {
        XComponent embeddObj = null;
        XChartDocument aChartDocument = null;
        XDiagram aDiagram = null;

        XTableChart aTableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);

        XEmbeddedObjectSupplier embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, aTableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        aChartDocument = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);

        XMultiServiceFactory aFact = (XMultiServiceFactory) UnoRuntime.queryInterface(
                XMultiServiceFactory.class, aChartDocument);
        try {
            aDiagram = (com.sun.star.chart.XDiagram) UnoRuntime.queryInterface(
                    XDiagram.class,
                    aFact.createInstance("com.sun.star.chart." + diagramType));
        } catch (Exception ex) {
            Logger.getLogger(ChartHelp.class.getName()).log(Level.SEVERE, null, ex);
        }

        aChartDocument.setDiagram(aDiagram);
    }

    public void removeFirstChartType(Object chartFromCollection, int serieNb) {
        XComponent embeddObj = null;
        XChartDocument aChartDocument = null;

        XTableChart aTableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);

        XEmbeddedObjectSupplier embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, aTableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        aChartDocument = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);

        com.sun.star.chart2.XChartDocument aChartDoc2 =
                (com.sun.star.chart2.XChartDocument) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartDocument.class, aChartDocument);
        com.sun.star.chart2.XDiagram firstDiag = aChartDoc2.getFirstDiagram();
        com.sun.star.chart2.XCoordinateSystemContainer coordSysCont =
                (com.sun.star.chart2.XCoordinateSystemContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XCoordinateSystemContainer.class, firstDiag);
        com.sun.star.chart2.XCoordinateSystem coordSysArr[] =
                coordSysCont.getCoordinateSystems();
        com.sun.star.chart2.XChartTypeContainer chartTypeCont =
                (com.sun.star.chart2.XChartTypeContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartTypeContainer.class, coordSysArr[0]);
        try {
            chartTypeCont.removeChartType(chartTypeCont.getChartTypes()[serieNb]);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public com.sun.star.chart2.XDataSeriesContainer createChartType(
            String chartType,
            Object chartFromCollection) {
        Object xyChartTypeObj = null;
        com.sun.star.chart2.XChartType xyChartType = null;
        com.sun.star.chart2.XChartTypeContainer chartTypeCont = null;
        XChartDocument aChartDocument = null;
        XComponent embeddObj = null;

        XMultiComponentFactory xmcf = m_xContext.getServiceManager();

        XTableChart aTableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);
        XEmbeddedObjectSupplier embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, aTableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        aChartDocument = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);
        com.sun.star.chart2.XChartDocument aChartDoc2 =
                (com.sun.star.chart2.XChartDocument) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartDocument.class, aChartDocument);
        com.sun.star.chart2.XDiagram firstDiag = aChartDoc2.getFirstDiagram();
        com.sun.star.chart2.XCoordinateSystemContainer coordSysCont =
                (com.sun.star.chart2.XCoordinateSystemContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XCoordinateSystemContainer.class, firstDiag);
        com.sun.star.chart2.XCoordinateSystem coordSysArr[] =
                coordSysCont.getCoordinateSystems();
        chartTypeCont =
                (com.sun.star.chart2.XChartTypeContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartTypeContainer.class, coordSysArr[0]);
        try {
            xyChartTypeObj = xmcf.createInstanceWithContext(
                    "com.sun.star.chart2." + chartType, m_xContext);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        xyChartType = (com.sun.star.chart2.XChartType) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartType.class, xyChartTypeObj);
        try {
            chartTypeCont.addChartType(xyChartType);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        com.sun.star.chart2.XDataSeriesContainer dataSeriesCnt =
                (com.sun.star.chart2.XDataSeriesContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XDataSeriesContainer.class, xyChartType);
        return dataSeriesCnt;
    }

    public com.sun.star.chart2.data.XLabeledDataSequence createSerieInChart(
            String dataRangeAddress,
            String role,
            Object chartFromCollection) {
        Object labSeqObj = null;
        XComponent embeddObj = null;
        XChartDocument aChartDocument = null;
        com.sun.star.chart2.data.XDataSequence seq = null;
        com.sun.star.chart2.data.XLabeledDataSequence labSeq = null;
        com.sun.star.chart2.data.XDataProvider oDataProv = null;

        XMultiComponentFactory xmcf = m_xContext.getServiceManager();

        XTableChart aTableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);

        XEmbeddedObjectSupplier embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, aTableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        aChartDocument = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);

        com.sun.star.chart2.XChartDocument aChartDoc2 =
                (com.sun.star.chart2.XChartDocument) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartDocument.class, aChartDocument);
        oDataProv = aChartDoc2.getDataProvider();
        try {
            seq = oDataProv.createDataSequenceByRangeRepresentation(dataRangeAddress);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        XPropertySet xProp = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, seq);
        try {
            xProp.setPropertyValue("Role", role);
            labSeqObj = xmcf.createInstanceWithContext(
                    "com.sun.star.chart2.data.LabeledDataSequence", m_xContext);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        labSeq =
                (com.sun.star.chart2.data.XLabeledDataSequence) UnoRuntime.queryInterface(
                com.sun.star.chart2.data.XLabeledDataSequence.class, labSeqObj);
        labSeq.setValues(seq);
        return labSeq;
    }

    public void addSerieToSerieCont(
            com.sun.star.chart2.data.XLabeledDataSequence xLabSeq,
            com.sun.star.chart2.data.XLabeledDataSequence yLabSeq,
            com.sun.star.chart2.XDataSeriesContainer dataSeriesCont) {
        Object dataSerieObj = null;
        com.sun.star.chart2.XDataSeries dataSerie = null;
        com.sun.star.chart2.data.XDataSink dataSink = null;

        XMultiComponentFactory xmcf = m_xContext.getServiceManager();
        try {
            dataSerieObj = xmcf.createInstanceWithContext(
                    "com.sun.star.chart2.DataSeries", m_xContext);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        dataSerie = (com.sun.star.chart2.XDataSeries) UnoRuntime.queryInterface(
                com.sun.star.chart2.XDataSeries.class, dataSerieObj);
        dataSink = (com.sun.star.chart2.data.XDataSink) UnoRuntime.queryInterface(
                com.sun.star.chart2.data.XDataSink.class, dataSerie);
        com.sun.star.chart2.data.XLabeledDataSequence labSeq[] = {xLabSeq, yLabSeq};
        dataSink.setData(labSeq);
        try {
            dataSeriesCont.addDataSeries(dataSerie);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public void formatSerieByIdxInChart(Object chartFromCollection,
            int serieNb,
            boolean point,
            boolean line,
            int pointSize,
            int lineWidth,
            int color,
            int symbolInt) {
        XComponent embeddObj;
        XChartDocument aChartDocument;
        XDiagram diagr;
        XPropertySet prop;

        XTableChart aTableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);

        XEmbeddedObjectSupplier embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, aTableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        aChartDocument = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);
        diagr = aChartDocument.getDiagram();
        try {
            prop = diagr.getDataRowProperties(serieNb);
            /*Property[] props = prop.getPropertySetInfo().getProperties();
            for (int i = 0; i < props.length; ++i) {
            System.out.println(props[i].Name + ": " + prop.getPropertyValue(props[i].Name));
            }*/
            if (point) {
                prop.setPropertyValue("SymbolType",
                        new Integer(symbolInt));
                prop.setPropertyValue("SymbolSize",
                        new Size(pointSize, pointSize));
            } else {
                prop.setPropertyValue("SymbolType",
                        new Integer(com.sun.star.chart.ChartSymbolType.NONE));
            }
            if (line) {
                prop.setPropertyValue("LineWidth", lineWidth);

            } else {
                prop.setPropertyValue("LineStyle", com.sun.star.drawing.LineStyle.NONE);
            }
            prop.setPropertyValue("FillColor", color);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public void formatSerieLastInChart(Object chartFromCollection,
            boolean point,
            boolean line,
            int pointSize,
            int lineWidth,
            int color,
            int symbolInt) {
        XComponent embeddObj = null;
        XChartDocument aChartDocument = null;
        XDiagram diagr = null;
        XPropertySet prop = null;
        int i = 0;

        XTableChart aTableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);

        XEmbeddedObjectSupplier embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, aTableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        aChartDocument = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);
        diagr = aChartDocument.getDiagram();
        for (i = 1;; ++i) {
            try {
                prop = diagr.getDataRowProperties(i);
            } catch (IndexOutOfBoundsException ex) {
                String msg =
                        "This is not the IndexOutOfBoundException you're looking for...";
                TraceMyStack.print(ex.getStackTrace(), msg, 3);
                System.out.println(msg);
                --i;
                break;
            }
        }
        try {
            prop = diagr.getDataRowProperties(i);
            /*Property[] props = prop.getPropertySetInfo().getProperties();
            for (int i = 0; i < props.length; ++i) {
            System.out.println(props[i].Name + ": " + prop.getPropertyValue(props[i].Name));
            }*/
            if (point) {
                prop.setPropertyValue("SymbolType",
                        new Integer(symbolInt));
                prop.setPropertyValue("SymbolSize",
                        new Size(pointSize, pointSize));
            } else {
                prop.setPropertyValue("SymbolType",
                        new Integer(com.sun.star.chart.ChartSymbolType.NONE));
            }
            if (line) {
                prop.setPropertyValue("LineWidth", lineWidth);

            } else {
                prop.setPropertyValue("LineStyle", com.sun.star.drawing.LineStyle.NONE);
            }
            prop.setPropertyValue("FillColor", color);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public String[] getChartsNamesInSheet(XSpreadsheet spreadsh) {
        XTableChartsSupplier chartTableSupply = null;
        chartTableSupply = (XTableChartsSupplier) UnoRuntime.queryInterface(
                XTableChartsSupplier.class, spreadsh);
        return chartTableSupply.getCharts().getElementNames();
    }

    public String[] getChartsNamesInSheet(String sheetName) throws ChartHelpException {
        XTableChartsSupplier chartTableSupply = null;
        SheetHelp sheetH = new SheetHelp(m_xContext);
        XSpreadsheet sheet;
        try {
            sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        } catch (SheetHelpSheetExistsException ex) {
            throw new ChartHelpException(ex.getMessage());
        }
        chartTableSupply = (XTableChartsSupplier) UnoRuntime.queryInterface(
                XTableChartsSupplier.class, sheet);
        return chartTableSupply.getCharts().getElementNames();
    }

    public XChartType[] getChartTypesInChart(Object chartFromCollection,
            int coordSysIdx) {
        XComponent embeddObj = null;
        XChartDocument chartDoc = null;
        XTableChart tableChart = null;
        XEmbeddedObjectSupplier embedObjSuppl = null;
        com.sun.star.chart2.XChartDocument chartDoc2 = null;
        com.sun.star.chart2.XChartTypeContainer chartTypeCont = null;
        XChartType chartTypes[] = null;

        tableChart = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);
        embedObjSuppl =
                (XEmbeddedObjectSupplier) UnoRuntime.queryInterface(
                XEmbeddedObjectSupplier.class, tableChart);
        embeddObj = embedObjSuppl.getEmbeddedObject();
        chartDoc = (XChartDocument) UnoRuntime.queryInterface(
                XChartDocument.class, embeddObj);
        chartDoc2 =
                (com.sun.star.chart2.XChartDocument) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartDocument.class, chartDoc);

        com.sun.star.chart2.XDiagram firstDiag = chartDoc2.getFirstDiagram();
        com.sun.star.chart2.XCoordinateSystemContainer coordSysCont =
                (com.sun.star.chart2.XCoordinateSystemContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XCoordinateSystemContainer.class, firstDiag);
        com.sun.star.chart2.XCoordinateSystem coordSysArr[] =
                coordSysCont.getCoordinateSystems();
        chartTypeCont =
                (com.sun.star.chart2.XChartTypeContainer) UnoRuntime.queryInterface(
                com.sun.star.chart2.XChartTypeContainer.class, coordSysArr[coordSysIdx]);
        chartTypes = chartTypeCont.getChartTypes();
        return chartTypes;
    }

    public CellRangeAddress[] getDataRangesInChart(Object chartFromCollection) {
        XTableChart chartTable = (XTableChart) UnoRuntime.queryInterface(
                XTableChart.class, chartFromCollection);
        return chartTable.getRanges();
    }

    public class ChartHelpException extends Exception {

        public ChartHelpException(String msg) {
            super(msg);
        }
    }
}
