 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.ooohelp;

import com.sun.star.beans.Property;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XEnumeration;
import com.sun.star.container.XIndexAccess;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XModel;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.sheet.XSpreadsheetDocument;
import com.sun.star.ucb.XFileIdentifierConverter;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.uri.XUriReference;
import com.sun.star.uri.XUriReferenceFactory;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.tracemystack.TraceMyStack;

public class DocHelp {

    private XComponentContext m_xContext;

    public DocHelp(XComponentContext m_xContext) {
        this.m_xContext = m_xContext;
    }

    public String getLastSegmentFromURL(String url) {
        XUriReference uriRef = null;
        Object uriRefFactObj = null;
        XUriReferenceFactory uriRefFact = null;
        String urlName = null;

        try {
            XMultiComponentFactory xmcf = m_xContext.getServiceManager();
            uriRefFactObj =
                    xmcf.createInstanceWithContext(
                    "com.sun.star.uri.UriReferenceFactory", m_xContext);
            uriRefFact =
                    (XUriReferenceFactory) UnoRuntime.queryInterface(
                    XUriReferenceFactory.class, uriRefFactObj);
            uriRef = uriRefFact.parse(url);
            urlName = uriRef.getPathSegment(uriRef.getPathSegmentCount() - 1);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return urlName;
    }

    public String url2norm(String url) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        String norm = null;
        XMultiComponentFactory xmcf = m_xContext.getServiceManager();
        try {
            XFileIdentifierConverter xFileConverter =
                    (XFileIdentifierConverter) UnoRuntime.queryInterface(
                    XFileIdentifierConverter.class,
                    xmcf.createInstanceWithContext(
                    "com.sun.star.ucb.FileContentProvider", m_xContext));
            norm = xFileConverter.getSystemPathFromFileURL(url);
        } catch (com.sun.star.uno.Exception ex) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    ex.getMessage(), 3);
        }
        return norm;
    }

    public String[] getComponentsNames() {
        String names[] = null;
        List<String> namesLst = new ArrayList<String>();
        String tmpStr = null;
        Object viewDataObj = null;
        XIndexAccess viewData = null;

        XDesktop desk = getDesktop();
        XEnumeration compEnum = desk.getComponents().createEnumeration();
        com.sun.star.frame.XModel model = null;
        while (compEnum.hasMoreElements()) {
            try {
                model = (XModel) UnoRuntime.queryInterface(
                        XModel.class, compEnum.nextElement());
            } catch (Exception ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
            }
            if (model != null) {
                PropertyValue propArr[] = model.getArgs();
                if (model.getURL().isEmpty()) {
                    for (int i = 0; i < propArr.length; ++i) {
                        if (propArr[i].Name.equals("Title")) {
                            namesLst.add(propArr[i].Value.toString());
                        }
                    }
                } else {
                    namesLst.add(model.getURL());
                }
            }
        }
        if (!namesLst.isEmpty()) {
            names = namesLst.toArray(new String[namesLst.size()]);
        }
        return names;
    }

    public String[] getSpreadhsComponentsNames() {
        String names[] = null;
        List<String> namesLst = new ArrayList<String>();

        XDesktop desk = getDesktop();
        XEnumeration compEnum = desk.getComponents().createEnumeration();
        com.sun.star.frame.XModel model = null;
        Object componObj = null;
        while (compEnum.hasMoreElements()) {
            try {
                componObj = compEnum.nextElement();
            } catch (NoSuchElementException ex) {
                Logger.getLogger(DocHelp.class.getName()).log(Level.SEVERE, null, ex);
            } catch (WrappedTargetException ex) {
                Logger.getLogger(DocHelp.class.getName()).log(Level.SEVERE, null, ex);
            }
            model = (XModel) UnoRuntime.queryInterface(
                    XModel.class, componObj);
            if (model != null) {
                PropertyValue propValArr[] = model.getArgs();
                for (int i = 0; i < propValArr.length; ++i) {
                    if (propValArr[i].Name.equals("DocumentService")
                            && propValArr[i].Value.toString().equals(
                            "com.sun.star.sheet.SpreadsheetDocument")) {
                        if (model.getURL().isEmpty()) {
                            for (int ii = 0; ii < propValArr.length; ++ii) {
                                if (propValArr[ii].Name.equals("Title")) {
                                    namesLst.add(propValArr[ii].Value.toString());
                                }
                            }
                        } else {
                            namesLst.add(model.getURL());
                        }
                    }
                }
            }
        }
        if (!namesLst.isEmpty()) {
            names = namesLst.toArray(new String[namesLst.size()]);
        }
        return names;
    }

    public String getCurrentComponPath(boolean markUnsavedFl, String markStr) {
        XDesktop desk = getDesktop();
        XComponent compon = null;
        com.sun.star.frame.XModel model = null;
        compon = desk.getCurrentComponent();
        String path = null;
        model = (XModel) UnoRuntime.queryInterface(XModel.class, compon);
        if (model != null) {
            PropertyValue propArr[] = model.getArgs();
            if (model.getURL().isEmpty()) {
                for (int ii = 0; ii < propArr.length; ++ii) {
                    if (propArr[ii].Name.equals("Title")) {
                        path = propArr[ii].Value.toString();
                        if (markUnsavedFl) {
                            path += " " + markStr;
                        }
                    }
                }
            } else {
                path = model.getURL();
            }
        }
        return path;
    }

    public XDesktop getDesktop() {
        XDesktop desktop = null;
        try {
            XMultiComponentFactory xmcf = m_xContext.getServiceManager();
            Object desktopFrame = xmcf.createInstanceWithContext(
                    "com.sun.star.frame.Desktop", m_xContext);
            desktop = (XDesktop) UnoRuntime.queryInterface(XDesktop.class, desktopFrame);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return desktop;
    }

    public void printCurrentComponModelProp() {
        XDesktop desk = getDesktop();
        XComponent compon = null;
        com.sun.star.frame.XModel model = null;
        compon = desk.getCurrentComponent();

        model = (XModel) UnoRuntime.queryInterface(XModel.class, compon);
        if (model != null) {
            PropertyValue propArr[] = model.getArgs();
            for (int i = 0; i < propArr.length; ++i) {
                System.out.println("Name: " + propArr[i].Name);
                System.out.println("Value: " + propArr[i].Value.toString());
                System.out.println("State: " + propArr[i].State.toString());
            }
        }
    }

    public XComponent newSpreadshCompon() {
        String unoUrl = "private:factory/scalc";
        XComponent spreadComp = null;
        DocHelp shtDocH = new DocHelp(m_xContext);
        XDesktop desk = shtDocH.getDesktop();
        try {
            XComponentLoader xComponentLoader =
                    (XComponentLoader) UnoRuntime.queryInterface(
                    XComponentLoader.class, desk);
            PropertyValue[] loadProps = new PropertyValue[0];
            spreadComp = xComponentLoader.loadComponentFromURL(unoUrl, "_blank", 0,
                    loadProps);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return spreadComp;
    }

    public boolean checkSpreadshDocByName(String name) {
        DocHelp shtDoc = new DocHelp(m_xContext);
        XDesktop desk = shtDoc.getDesktop();
        boolean exists = false;
        XEnumeration compEnum = desk.getComponents().createEnumeration();
        com.sun.star.frame.XModel model = null;
        while (compEnum.hasMoreElements()) {
            try {
                model = (XModel) UnoRuntime.queryInterface(
                        XModel.class, compEnum.nextElement());
            } catch (Exception ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
            }
            if (model != null) {
                PropertyValue propArr[] = model.getArgs();
                for (int i = 0; i
                        < propArr.length;
                        ++i) {
                    if (propArr[i].Name.equals("Title") && propArr[i].Value.equals(name)) {
                        exists = true;
                        break;
                    }
                }
            }
        }
        return exists;
    }

    public XComponent checkSpreadshDocByName2(String name) {
        Object componObj = null;
        XComponent compon = null;
        DocHelp shtDoc = new DocHelp(m_xContext);
        XDesktop desk = shtDoc.getDesktop();
        XEnumeration compEnum = desk.getComponents().createEnumeration();
        com.sun.star.frame.XModel model = null;
        while (compEnum.hasMoreElements()) {
            try {
                componObj = compEnum.nextElement();
                model = (XModel) UnoRuntime.queryInterface(XModel.class, componObj);
            } catch (Exception ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
            }
            if (model != null) {
                PropertyValue propArr[] = model.getArgs();
                for (int i = 0; i < propArr.length; ++i) {
                    if (propArr[i].Name.equals("Title")
                            && propArr[i].Value.equals(name)) {
                        compon = (XComponent) UnoRuntime.queryInterface(
                                XComponent.class, componObj);
                        break;
                    }
                }
            }
        }
        return compon;
    }

    public void printCurrentComponProps() {
        XDesktop desk = getDesktop();
        XComponent component = desk.getCurrentComponent();
        XSpreadsheetDocument sheetDoc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, component);
        XPropertySet docProps =
                (XPropertySet) UnoRuntime.queryInterface(XPropertySet.class, component);
        Property[] propsArr = docProps.getPropertySetInfo().getProperties();
        for (int i = 0; i < propsArr.length; ++i) {
            try {
                System.out.println(propsArr[i].Name
                        + ": " + docProps.getPropertyValue(propsArr[i].Name));
            } catch (Exception ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
            }
        }
    }

    public void printComponentProps(XComponent component) {
        XDesktop desk = getDesktop();
        XPropertySet docProps =
                (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, component);
        Property[] propsArr = docProps.getPropertySetInfo().getProperties();
        for (int i = 0; i < propsArr.length; ++i) {
            try {
                System.out.println(propsArr[i].Name
                        + ": " + docProps.getPropertyValue(propsArr[i].Name));
            } catch (Exception ex) {
                TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
            }
        }
    }
}
