 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2010, 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.ooohelp;

import com.sun.star.beans.PropertyValue;
import com.sun.star.container.XNameAccess;
import com.sun.star.deployment.PackageInformationProvider;
import com.sun.star.deployment.XPackageInformationProvider;
import com.sun.star.lang.Locale;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.XURLTransformer;
import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.tracemystack.TraceMyStack;

public class OfficeHelp {

    private XComponentContext m_xContext;

    public OfficeHelp(XComponentContext m_xContext) {
        this.m_xContext = m_xContext;
    }

    public XNameAccess getRegKeyCont(String keyName)
            throws
            OOoNullContextException {
        Object configProvObj = null;
        XMultiServiceFactory xmsf = null;
        Object regKeyContObj = null;
        XNameAccess regKeyCont = null;
        PropertyValue[] propsArg = new PropertyValue[1];
        if (m_xContext == null) {
            throw new OOoNullContextException();
        }
        try {
            configProvObj = m_xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.configuration.ConfigurationProvider", m_xContext);
            propsArg[0] = new PropertyValue();
            propsArg[0].Name = "nodepath";
            propsArg[0].Value = keyName;
            xmsf = (XMultiServiceFactory) UnoRuntime.queryInterface(
                    XMultiServiceFactory.class, configProvObj);
            regKeyContObj = xmsf.createInstanceWithArguments(
                    "com.sun.star.configuration.ConfigurationAccess", propsArg);
        } catch (Exception ex) {
            Logger.getLogger(OfficeHelp.class.getName()).log(Level.SEVERE, null, ex);
        }
        regKeyCont = (XNameAccess) UnoRuntime.queryInterface(
                XNameAccess.class, regKeyContObj);
        return regKeyCont;
    }

    private Locale getLocale(XNameAccess regKeyCont) {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        Locale locale = new Locale();
        String localeStr = null;
        String localeStrArr[] = null;
        try {
            localeStr = regKeyCont.getByName("ooLocale").toString();
        } catch (com.sun.star.container.NoSuchElementException ex) {
            Logger.getLogger(OfficeHelp.class.getName()).log(Level.SEVERE, null, ex);
        } catch (WrappedTargetException ex) {
            Logger.getLogger(OfficeHelp.class.getName()).log(Level.SEVERE, null, ex);
        }
        localeStrArr = localeStr.split("-");
        locale.Language = localeStrArr[0];
        if (localeStrArr.length > 1) {
            locale.Country = localeStrArr[1];
        } else {
            locale.Country = localeStrArr[0].toUpperCase();
        }
        return locale;
    }

    public String getLocaleStr()
            throws
            OOoNullContextException {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        String localeStr = null;
        XNameAccess regKeyCont =
                getRegKeyCont("org.openoffice.Setup/L10N/");
        Locale locale = getLocale(regKeyCont);
        localeStr = locale.Language + "_" + locale.Country;
        return localeStr;
    }

    public String getOXTPath(String oxtIDString) {
        String oxtPath = null;
        XPackageInformationProvider packInfoProv =
                PackageInformationProvider.get(m_xContext);
        oxtPath = packInfoProv.getPackageLocation(oxtIDString);
        return oxtPath;
    }

    public com.sun.star.util.URL getUrlFromOxtID(String oxtIDString) {
        com.sun.star.util.URL oooURL = null;
        Object transfObj = null;
        XURLTransformer transf = null;
        com.sun.star.util.URL oooURLArr[] = null;
        String oooURLStr = getOXTPath(oxtIDString);
        try {
            transfObj = m_xContext.getServiceManager().
                    createInstanceWithContext(
                    "com.sun.star.util.URLTransformer", m_xContext);
        } catch (Exception ex) {
            Logger.getLogger(DocHelp.class.getName()).log(Level.SEVERE, null, ex);
        }
        transf = (XURLTransformer) UnoRuntime.queryInterface(
                XURLTransformer.class, transfObj);
        oooURLArr = new com.sun.star.util.URL[1];
        oooURLArr[0] = new com.sun.star.util.URL();
        oooURLArr[0].Complete = oooURLStr;
        transf.parseStrict(oooURLArr);
        oooURL = oooURLArr[0];
        return oooURL;
    }

    public static class OOoNullContextException extends Exception {

        public OOoNullContextException() {
            super("Office context == null.");
        }
    }
}
