 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.ooohelp;

import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XNameAccess;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XModel;
import com.sun.star.lang.IndexOutOfBoundsException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.sheet.XCellRangeAddressable;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.sheet.XCellRangeData;
import com.sun.star.sheet.XCellRangeFormula;
import com.sun.star.sheet.XSpreadsheet;
import com.sun.star.table.CellRangeAddress;
import com.sun.star.table.XCellRange;
import com.sun.star.table.XColumnRowRange;
import com.sun.star.table.XTableColumns;
import com.sun.star.uno.XInterface;
import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.ooohelp.SheetHelp.SheetHelpSheetExistsException;
import name.mgplldz.tracemystack.TraceMyStack;

public class RangeHelp {

    private XComponentContext m_xContext;
    //private static final int DATA_ARRAY = 0;
    //static final private int FORMULA_ARRAY = 1;

    public RangeHelp(XComponentContext m_xContext) {
        this.m_xContext = m_xContext;
    }

    public XCellRangeData getRngDataFromSheet(String rngName, XSpreadsheet spreadsh) {
        XCellRange rng = null;
        XCellRangeData rngDta = null;
        rng = spreadsh.getCellRangeByName(rngName);
        rngDta = (com.sun.star.sheet.XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        return rngDta;
    }

    public Object[][] getRngDataArrFromSheetName(String rngName,
            String sheetName) throws RangeHelpException, SheetHelpSheetExistsException {
        XCellRange rng = null;
        XCellRangeData rngDta = null;
        XSpreadsheet sheet = null;
        SheetHelp sheetH = new SheetHelp(m_xContext);
        sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        rng = sheet.getCellRangeByName(rngName);
        rngDta = (com.sun.star.sheet.XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        return rngDta.getDataArray();
    }

    public void setRngDataArrInSheet(String rngName,
            XSpreadsheet spreadsh, Object[][] dataArr)
            throws
            RangeHelpSetRngDataFormulaException {
        XCellRange rng = spreadsh.getCellRangeByName(rngName);
        int rngRows = getRngRowsAmount(rng);
        int rngCols = getRngColsAmount(rng);
        int dtaRows = dataArr.length;
        int dtaCols = dataArr[0].length;
        if (rngRows != dtaRows
                || rngCols != dtaCols) {
            String msg = "Range doesn't match data. "
                    + "Range[" + rngRows + "][" + rngCols + "]"
                    + " vs Data[" + dtaRows + "][" + dtaCols + "]";
            throw new RangeHelpSetRngDataFormulaException(msg);
        }
        XCellRangeData rngDta = (XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        rngDta.setDataArray(dataArr);
    }

    public void setRngDataArrInSheet(int startCol, int startRow,
            int endCol, int endRow, XSpreadsheet spreadsh, Object[][] dataArr)
            throws
            RangeHelpSetRngDataFormulaException,
            IndexOutOfBoundsException {
        XCellRange rng = spreadsh.getCellRangeByPosition(
                startCol, startRow, endCol, endRow);
        int rngRows = getRngRowsAmount(rng);
        int rngCols = getRngColsAmount(rng);
        int dtaRows = dataArr.length;
        int dtaCols = dataArr[0].length;
        if (rngRows != dtaRows
                || rngCols != dtaCols) {
            String msg = "Range doesn't match data. "
                    + "Range r[" + rngRows + "]:c[" + rngCols + "]"
                    + " vs Data r[" + dtaRows + "]:c[" + dtaCols + "]";
            throw new RangeHelpSetRngDataFormulaException(msg);
        }
        XCellRangeData rngDta = (XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        rngDta.setDataArray(dataArr);
    }

    public void setRngDataArrInSheet(String rngName,
            String sheetName, Object[][] dataArr)
            throws
            RangeHelpSetRngDataFormulaException,
            SheetHelpSheetExistsException {
        SheetHelp sheetH = new SheetHelp(m_xContext);
        XSpreadsheet sheet;
        sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        XCellRange rng = sheet.getCellRangeByName(rngName);
        XCellRangeData rngDta =
                (com.sun.star.sheet.XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        int rngRows = getRngRowsAmount(rng);
        int rngCols = getRngColsAmount(rng);
        int dtaRows = dataArr.length;
        int dtaCols = dataArr[0].length;
        if (rngRows != dtaRows
                || rngCols != dtaCols) {
            String msg = "Range doesn't match data. "
                    + "Range[" + rngRows + "][" + rngCols + "]"
                    + " vs Data[" + dtaRows + "][" + dtaCols + "]";
            throw new RangeHelpSetRngDataFormulaException("Range doesn't match data.");
        }
        rngDta.setDataArray(dataArr);
    }

    public void setRngFormulaArrInSheet(String rngName,
            XSpreadsheet sprsh, String[][] formulaArr)
            throws RangeHelpSetRngDataFormulaException {
        XCellRange rng = sprsh.getCellRangeByName(rngName);
        int rngRows = getRngRowsAmount(rng);
        int rngCols = getRngColsAmount(rng);
        int dtaRows = formulaArr.length;
        int dtaCols = formulaArr[0].length;
        if (rngRows != dtaRows
                || rngCols != dtaCols) {
            String msg = "Range doesn't match data. "
                    + "Range[" + rngRows + "][" + rngCols + "]"
                    + " vs Data[" + dtaRows + "][" + dtaCols + "]";
            throw new RangeHelpSetRngDataFormulaException(msg);
        }
        XCellRangeFormula rngFormula =
                (com.sun.star.sheet.XCellRangeFormula) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeFormula.class, rng);
        rngFormula.setFormulaArray(formulaArr);
    }

    public void setRngFormulaArrInSheet(int startCol, int startRow,
            int endCol, int endRow,
            XSpreadsheet sprsh, String[][] formulaArr)
            throws
            RangeHelpSetRngDataFormulaException,
            IndexOutOfBoundsException {
        XCellRange rng = sprsh.getCellRangeByPosition(startCol, startRow,
                endCol, endRow);
        int rngRows = getRngRowsAmount(rng);
        int rngCols = getRngColsAmount(rng);
        int dtaRows = formulaArr.length;
        int dtaCols = formulaArr[0].length;
        if (rngRows != dtaRows
                || rngCols != dtaCols) {
            String msg = "Range doesn't match data. "
                    + "Range[" + rngRows + "][" + rngCols + "]"
                    + " vs Data[" + dtaRows + "][" + dtaCols + "]";
            throw new RangeHelpSetRngDataFormulaException(msg);
        }
        XCellRangeFormula rngFormula =
                (com.sun.star.sheet.XCellRangeFormula) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeFormula.class, rng);
        rngFormula.setFormulaArray(formulaArr);
    }

    public void setRngFormulaArrInSheet(String rngName,
            String sheetName,
            String[][] formulaArr)
            throws
            SheetHelpSheetExistsException,
            RangeHelpSetRngDataFormulaException {
        SheetHelp sheetH = new SheetHelp(m_xContext);
        XSpreadsheet sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        XCellRange rng = sheet.getCellRangeByName(rngName);
        int rngRows = getRngRowsAmount(rng);
        int rngCols = getRngColsAmount(rng);
        int dtaRows = formulaArr.length;
        int dtaCols = formulaArr[0].length;
        if (rngRows != dtaRows
                || rngCols != dtaCols) {
            String msg = "Range doesn't match data. "
                    + "Range[" + rngRows + "][" + rngCols + "]"
                    + " vs Data[" + dtaRows + "][" + dtaCols + "]";
            throw new RangeHelpSetRngDataFormulaException(msg);
        }
        XCellRangeFormula rngFormula =
                (com.sun.star.sheet.XCellRangeFormula) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeFormula.class, rng);
        rngFormula.setFormulaArray(formulaArr);
    }

    public Object[][] getRngDataArrInSheet(String rngName, XSpreadsheet spreadsh) {
        XCellRange rng = null;
        XCellRangeData rngDta = null;
        rng = spreadsh.getCellRangeByName(rngName);
        rngDta = (com.sun.star.sheet.XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        return rngDta.getDataArray();
    }

    public XCellRange getRngFromRngSheeNames(String rngName, String sheetName)
            throws RangeHelpException, SheetHelpSheetExistsException {
        XCellRange rng = null;
        XSpreadsheet sheet = null;
        SheetHelp sheetH = new SheetHelp(m_xContext);
        sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        rng = sheet.getCellRangeByName(rngName);
        return rng;
    }

    public CellRangeAddress getRngAddr(XCellRange rng) {
        CellRangeAddress addr = null;
        com.sun.star.sheet.XCellRangeAddressable rngAddr =
                (com.sun.star.sheet.XCellRangeAddressable) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeAddressable.class, rng);
        try {
            addr = rngAddr.getRangeAddress();
        } catch (java.lang.NullPointerException ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return addr;
    }

    public CellRangeAddress getRngAddrFromRngSheetNames(String rngName,
            String sheetName) throws RangeHelpException, SheetHelpSheetExistsException {
        XCellRange rng = null;
        XSpreadsheet sheet = null;
        CellRangeAddress addr = null;
        SheetHelp sheetH = new SheetHelp(m_xContext);
        sheet = sheetH.getSheetByNameInActiveDoc(sheetName);
        rng = sheet.getCellRangeByName(rngName);
        com.sun.star.sheet.XCellRangeAddressable rngAddr =
                (XCellRangeAddressable) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeAddressable.class, rng);
        addr = rngAddr.getRangeAddress();
        return addr;
    }

    public int getRngColsAmount(XCellRange rng) {
        com.sun.star.table.XColumnRowRange colRowRng =
                (com.sun.star.table.XColumnRowRange) UnoRuntime.queryInterface(
                com.sun.star.table.XColumnRowRange.class, rng);
        return colRowRng.getColumns().getCount();
    }

    public int getRngRowsAmount(XCellRange rng) {
        com.sun.star.table.XColumnRowRange colRowRng =
                (com.sun.star.table.XColumnRowRange) UnoRuntime.queryInterface(
                com.sun.star.table.XColumnRowRange.class, rng);
        return colRowRng.getRows().getCount();
    }

    public Object[][][] joinColDataArr(Object arr1[][],
            Object arr2[][],
            Object arr3[][]) {
        Object join[][][] = new Object[3][][];
        join[0] = arr1;
        join[1] = arr2;
        join[2] = arr3;
        return join;
    }

    public void setRngDataArr(XCellRange rng, Object[][] dataArr) {
        XCellRangeData rngDta = null;
        rngDta = (com.sun.star.sheet.XCellRangeData) UnoRuntime.queryInterface(
                com.sun.star.sheet.XCellRangeData.class, rng);
        try {
            rngDta.setDataArray(dataArr);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public XCellRange getSelectedRangeFromActiveSheet() {
        DocHelp shtDoc = new DocHelp(m_xContext);
        XDesktop desk = shtDoc.getDesktop();
        XModel model = desk.getCurrentFrame().getController().getModel();
        XInterface interf = null;
        try {
            interf = (XInterface) UnoRuntime.queryInterface(
                    XInterface.class, model.getCurrentSelection());
        } catch (NullPointerException e) {
            TraceMyStack.print(e.getStackTrace(), e.getMessage(), 3);
        }
        XCellRange range = (XCellRange) UnoRuntime.queryInterface(
                XCellRange.class, interf);

        return range;
    }

    public String getAddressStringFromRange(XCellRange range) {
        String strAddr = null;
        XPropertySet ps =
                (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, range);
        try {
            strAddr = ps.getPropertyValue("AbsoluteName").toString();
        } catch (UnknownPropertyException ex) {
            Logger.getLogger(RangeHelp.class.getName()).log(Level.SEVERE, null, ex);
        } catch (WrappedTargetException ex) {
            Logger.getLogger(RangeHelp.class.getName()).log(Level.SEVERE, null, ex);
        }
        return strAddr;
    }

    public String getColumnNameFromIdx(int colIdx) throws IndexOutOfBoundsException {
        String strName = null;
        XSpreadsheet curSheet = null;
        XCellRange range = null;
        SheetHelp shtH = new SheetHelp(m_xContext);
        curSheet = shtH.getActiveSheet();
        try {
            range = curSheet.getCellRangeByPosition(colIdx, 0, colIdx, 0);
        } catch (IndexOutOfBoundsException ex) {
            Logger.getLogger(RangeHelp.class.getName()).log(Level.SEVERE, null, ex);
        }
        XColumnRowRange colRowRng =
                (XColumnRowRange) UnoRuntime.queryInterface(
                XColumnRowRange.class, range);
        XTableColumns tableCols = colRowRng.getColumns();
        XNameAccess nameAcc = (XNameAccess) UnoRuntime.queryInterface(
                XNameAccess.class, tableCols);
        if (nameAcc.hasElements()) {
            strName = nameAcc.getElementNames()[0];
        }
        return strName;
    }

    public static String getColumnNameFromIdx2(int colIdx) throws Exception {
        if (colIdx <= 0) {
            throw new Exception("Column index must be > 0.");
        }
        String strName = "";
        int mod = 0;
        while (colIdx > 0) {
            mod = colIdx % 26;
            mod = ((mod == 0) ? 26 : mod);
            strName += Character.toString((char) (64 + mod));
            colIdx -= mod;
            colIdx /= 26;
        }
        return new StringBuffer(strName).reverse().toString();
    }

    public static int getColumnIdxFromName2(String name) {
        name = name.toUpperCase();
        char nameChr[] = name.toCharArray();
        int col = 0;
        int last = name.length() - 1;
        int rowMax = name.length();
        int pow = 0;

        for (int i = 0; i < rowMax; ++i) {
            pow = last - i;
            nameChr[i] -= 64;
            col += nameChr[i] * Math.pow(26, pow);
        }
        col--;
        return col;
    }

    public String range2String(XCellRange rng) throws IndexOutOfBoundsException {
        if (rng == null) {
            throw new java.lang.NullPointerException("Empty cell cell range.");
        }
        String addrStr = null;
        CellRangeAddress address = getRngAddr(rng);
        int r0 = address.StartRow;
        int r1 = address.EndRow;
        int c0 = address.StartColumn;
        int c1 = address.EndColumn;
        String cs0 = getColumnNameFromIdx(c0);
        String cs1 = getColumnNameFromIdx(c1);
        r0++;
        r1++;
        addrStr = cs0 + r0 + ":" + cs1 + r1;
        return addrStr;
    }

    public String rngAddrIncreaseCol(XCellRange rng, int colIncr) {
        String addrStr = null;
        if (rng == null) {
            throw new java.lang.NullPointerException("Empty cell range address.");
        }
        CellRangeAddress address = getRngAddr(rng);
        int r0 = address.StartRow;
        int r1 = address.EndRow;
        int c0 = address.StartColumn;
        int c1 = address.EndColumn;
        colIncr++;
        r0++;
        r1++;
        c0 += colIncr;
        c1 += colIncr;
        String cs0 = null;
        String cs1 = null;
        try {
            cs0 = getColumnNameFromIdx2(c0);
            cs1 = getColumnNameFromIdx2(c1);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        addrStr = cs0 + r0 + ":" + cs1 + r1;
        return addrStr;
    }

    public int getSheetIdxFromRng(XCellRange rng) {
        CellRangeAddress address = getRngAddr(rng);
        return address.Sheet;
    }

    public class RangeHelpException extends Exception {

        public RangeHelpException(String msg) {
            super(msg);
        }
    }

    public class RangeHelpSetRngDataFormulaException extends Exception {

        public RangeHelpSetRngDataFormulaException(String msg) {
            super(msg);
        }
    }
}
