 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.ooohelp;

import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XNamed;
import com.sun.star.frame.XController;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XModel;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.sheet.XSpreadsheet;
import com.sun.star.sheet.XSpreadsheetDocument;
import com.sun.star.sheet.XSpreadsheetView;
import com.sun.star.sheet.XSpreadsheets;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.XProtectable;
import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.tracemystack.TraceMyStack;

public class SheetHelp {

    private XComponentContext context;

    public SheetHelp(XComponentContext m_xContext) {
        this.context = m_xContext;
    }

    public XSpreadsheet getSheetByIdxInActiveDoc(int idx) {
        DocHelp shtDoc = new DocHelp(context);
        XDesktop desk = shtDoc.getDesktop();
        XComponent component = desk.getCurrentComponent();
        XSpreadsheet spreadsh = null;
        XSpreadsheetDocument sheetDoc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, component);
        XSpreadsheets spreadsheets = sheetDoc.getSheets();
        XIndexAccess sheetsElem =
                (XIndexAccess) UnoRuntime.queryInterface(XIndexAccess.class, spreadsheets);
        try {
            spreadsh = (XSpreadsheet) UnoRuntime.queryInterface(
                    XSpreadsheet.class, sheetsElem.getByIndex(idx));
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return spreadsh;
    }

    public XSpreadsheet getSheetByNameInDocComponent(String name,
            XComponent component)
            throws SheetHelpSheetExistsException {
        XSpreadsheet spreadsh = null;
        XSpreadsheetDocument sheetDoc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, component);
        XSpreadsheets spreadsheets = sheetDoc.getSheets();
        try {
            spreadsh = (XSpreadsheet) UnoRuntime.queryInterface(
                    XSpreadsheet.class, spreadsheets.getByName(name));
        } catch (NoSuchElementException ex) {
            String msg =
                    "Sheet doesn't exitst: " + name;
            TraceMyStack.print(Thread.currentThread().getStackTrace(), msg, 3);
            throw new SheetHelpSheetExistsException(msg);
        } catch (WrappedTargetException ex) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(), ex.getMessage(), 3);
        }
        return spreadsh;
    }

    public XSpreadsheet getSheetByNameInActiveDoc(String name)
            throws SheetHelpSheetExistsException {
        DocHelp shtDoc = new DocHelp(context);
        XDesktop desk = shtDoc.getDesktop();
        XComponent component = desk.getCurrentComponent();
        XSpreadsheet spreadsh = null;
        XSpreadsheetDocument sheetDoc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, component);
        XSpreadsheets spreadsheets = sheetDoc.getSheets();
        try {
            spreadsh = (XSpreadsheet) UnoRuntime.queryInterface(
                    XSpreadsheet.class, spreadsheets.getByName(name));
        } catch (NoSuchElementException ex) {
            String msg =
                    "Sheet doesn't exitst: " + name;
            TraceMyStack.print(Thread.currentThread().getStackTrace(), msg, 3);
            throw new SheetHelpSheetExistsException(msg);
        } catch (WrappedTargetException ex) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(), ex.getMessage(), 3);
        }
        return spreadsh;
    }

    public void renameSheetInAcitiveDoc(XSpreadsheet sheet, String newName) {
        XNamed named = null;
        try {
            named = (XNamed) UnoRuntime.queryInterface(
                    XNamed.class, sheet);
            named.setName(newName);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public void delSheetByNameInActiveDoc(XComponent docCompon, String name) {
        XSpreadsheetDocument doc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, docCompon);
        XSpreadsheets sheets = doc.getSheets();
        try {
            sheets.removeByName(name);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
    }

    public XSpreadsheet getActiveSheet() {
        DocHelp shtDoc = new DocHelp(context);
        XDesktop desk = shtDoc.getDesktop();
        XComponent currCompon = null;
        XController spreadshCtrl = null;
        currCompon = desk.getCurrentComponent();
        XModel model =
                (XModel) UnoRuntime.queryInterface(XModel.class, currCompon);
        spreadshCtrl = model.getCurrentController();
        XSpreadsheetView spreadshView =
                (XSpreadsheetView) UnoRuntime.queryInterface(
                XSpreadsheetView.class, spreadshCtrl);
        return spreadshView.getActiveSheet();
    }

    public void protectActiveSheet(String passwd) {
        DocHelp shtDoc = new DocHelp(context);
        XDesktop desk = shtDoc.getDesktop();
        XComponent currCompon = null;
        currCompon = desk.getCurrentComponent();
        XController spreadshCtrl = null;
        currCompon = desk.getCurrentComponent();
        XModel model =
                (XModel) UnoRuntime.queryInterface(XModel.class, currCompon);
        spreadshCtrl = model.getCurrentController();
        XSpreadsheetView spreadshView = (XSpreadsheetView) UnoRuntime.queryInterface(XSpreadsheetView.class,
                spreadshCtrl);
        XSpreadsheet sheet = spreadshView.getActiveSheet();
        XProtectable sheetProtect =
                (XProtectable) UnoRuntime.queryInterface(XProtectable.class,
                sheet);
        if (sheetProtect.isProtected()) {
            try {
                sheetProtect.unprotect(passwd);
            } catch (Exception ex) {
                Logger.getLogger(SheetHelp.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            sheetProtect.protect(passwd);
        }
    }

    public XController getActiveSheetController() {
        DocHelp shtDoc = new DocHelp(context);
        XDesktop desk = shtDoc.getDesktop();
        XComponent currCompon = null;
        XController spreadshCtrl = null;
        currCompon = desk.getCurrentComponent();
        XModel model =
                (XModel) UnoRuntime.queryInterface(XModel.class, currCompon);
        return model.getCurrentController();
    }

    public XModel getComponentModel(XComponent compon) {
        return (XModel) UnoRuntime.queryInterface(XModel.class, compon);
    }

    public XController getComponentController(XComponent compon) {
        return getComponentModel(compon).getCurrentController();
    }

    public XSpreadsheetView getComponentView(XComponent compon) {
        XController ctrl = getComponentController(compon);
        return (XSpreadsheetView) UnoRuntime.queryInterface(
                XSpreadsheetView.class, ctrl);
    }

    public boolean checkSheetByNameInActiveDoc(String name) {
        boolean has = false;
        DocHelp doc = new DocHelp(context);
        try {
            XDesktop desktop = doc.getDesktop();
            Object component = desktop.getCurrentComponent();
            XSpreadsheetDocument sheetDoc =
                    (XSpreadsheetDocument) UnoRuntime.queryInterface(
                    XSpreadsheetDocument.class, component);
            XSpreadsheets spreadsheets = sheetDoc.getSheets();
            has = spreadsheets.hasByName(name);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return has;
    }

    public XSpreadsheet addNewSheetInActiveDoc(String name, short idx,
            XComponent xSheetComponent)
            throws SheetHelpSheetExistsException {
        XSpreadsheetDocument doc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, xSheetComponent);
        com.sun.star.sheet.XSpreadsheets sheets = doc.getSheets();
        com.sun.star.sheet.XSpreadsheet sheet = null;
        sheets.insertNewByName(name, idx);
        sheet = getSheetByNameInActiveDoc(name);
        return sheet;
    }

    public XSpreadsheet addNewSheetInDocComponent(String name, short idx,
            XComponent xSheetComponent)
            throws SheetHelpSheetExistsException {
        XSpreadsheetDocument doc =
                (XSpreadsheetDocument) UnoRuntime.queryInterface(
                XSpreadsheetDocument.class, xSheetComponent);
        com.sun.star.sheet.XSpreadsheets sheets = doc.getSheets();
        com.sun.star.sheet.XSpreadsheet sheet = null;
        sheets.insertNewByName(name, idx);
        sheet = getSheetByNameInDocComponent(name, xSheetComponent);
        return sheet;
    }

    public String[] getSheetsNamesInActiveDoc() {
        String names[] = null;
        try {
            XMultiComponentFactory xmcf = context.getServiceManager();
            Object desktopFrame =
                    xmcf.createInstanceWithContext(
                    "com.sun.star.frame.Desktop", context);
            XDesktop desktop =
                    (XDesktop) UnoRuntime.queryInterface(
                    XDesktop.class, desktopFrame);
            Object component = desktop.getCurrentComponent();
            XSpreadsheetDocument sheetDoc =
                    (XSpreadsheetDocument) UnoRuntime.queryInterface(
                    XSpreadsheetDocument.class, component);
            XSpreadsheets spreadsheets = sheetDoc.getSheets();
            names = spreadsheets.getElementNames();
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return names;
    }

    public String getSheetNameFromIdx(int idx) {
        XSpreadsheet spreadsh = null;
        XNamed named = null;
        spreadsh = this.getSheetByIdxInActiveDoc(idx);
        named = (XNamed) UnoRuntime.queryInterface(XNamed.class, spreadsh);
        return named.getName();
    }

    public class SheetHelpSheetExistsException extends Exception {

        public SheetHelpSheetExistsException(String msg) {
            super(msg);
        }
    }

    public Object callSpreadshFunction(String fName, Object[] fArgs) {
        XMultiComponentFactory xmcf = context.getServiceManager();
        Object aFuncInst = null;
        Object result = null;
        try {
            aFuncInst = xmcf.createInstanceWithContext(
                    "com.sun.star.sheet.FunctionAccess", context);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        com.sun.star.sheet.XFunctionAccess xFuncAcc =
                (com.sun.star.sheet.XFunctionAccess) UnoRuntime.queryInterface(
                com.sun.star.sheet.XFunctionAccess.class, aFuncInst);
        try {
            result = xFuncAcc.callFunction(fName, fArgs);
        } catch (Exception ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        return result;
    }
}
