 /* * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    Copyright 2011 by:
 *    Marcin Gutman
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *    If you have any questions or comments
 *    please e-mail:
 *
 *    Marcin Gutman: mgutman@op.pl
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * */
package name.mgplldz.spline;

import java.util.logging.Level;
import java.util.logging.Logger;
import name.mgplldz.tracemystack.TraceMyStack;
import org.apache.commons.math.FunctionEvaluationException;
import org.apache.commons.math.MathException;
import org.apache.commons.math.analysis.UnivariateRealFunction;
import org.apache.commons.math.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math.analysis.interpolation.UnivariateRealInterpolator;
import org.apache.commons.math.exception.NonMonotonousSequenceException;

public class Spline {

    public static double[] xNew(double x[], int more)
            throws SplineExceptionMorePoints {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        int row = x.length;
        if (row < 3) {
            throw new SplineExceptionMorePoints(
                    "Need at least 3 points.");
        }
        int rowMore = row + (row - 1) * more;
        double xNew[] = new double[rowMore];
        double step = 0.0;

        for (int i = 0, j = 0; i < (row - 1); ++i) {
            xNew[j] = x[i];
            step = (x[i + 1] - x[i]) / (double) (more + 1);
            for (int k = 0; k < more; ++k) {
                xNew[++j] = x[i] + step * (double) (k + 1);
            }
            ++j;
        }
        xNew[rowMore - 1] = x[row - 1];
        return xNew;
    }

    public static double[] yNew(double x[], double y[], int more,
            double xNew[]) throws SplineExceptionDuplicates {
        TraceMyStack.print(Thread.currentThread().getStackTrace(), "", 3);
        int row = x.length;
        int rowMore = xNew.length;
        double yNew[] = new double[rowMore];
        UnivariateRealInterpolator interpolator = new SplineInterpolator();
        UnivariateRealFunction function = null;
        try {
            function = interpolator.interpolate(x, y);
        } catch (MathException ex) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    ex.getMessage(), 3);
        } catch (NonMonotonousSequenceException ex) {
            TraceMyStack.print(Thread.currentThread().getStackTrace(),
                    ex.getMessage(), 3);
            throw new SplineExceptionDuplicates("Remove duplicates.");
        }

        for (int i = 0, j = 0; i < (row - 1); ++i) {
            yNew[j] = y[i];
            for (int k = 0; k < more; ++k) {
                try {
                    yNew[++j] = function.value(xNew[j]);
                } catch (FunctionEvaluationException ex) {
                    TraceMyStack.print(Thread.currentThread().getStackTrace(),
                            ex.getMessage(), 3);
                }
            }
            ++j;
        }
        yNew[rowMore - 1] = y[row - 1];
        return yNew;
    }

    public static class SplineExceptionMorePoints extends Exception {

        public SplineExceptionMorePoints(String msg) {
            super(msg);
        }
    }

    public static class SplineExceptionDuplicates extends Exception {

        public SplineExceptionDuplicates(String msg) {
            super(msg);
        }
    }

    public static void main(String args[]) {
        double x[] = {1, 2, 5};
        double y[] = {5, 10, 15};
        int more = 2;
        double xNew[] = null;
        try {
            xNew = xNew(x, more);
        } catch (SplineExceptionMorePoints ex) {
            TraceMyStack.print(ex.getStackTrace(), ex.getMessage(), 3);
        }
        double yNew[] = null;
        try {
            yNew = yNew(x, y, more, xNew);
        } catch (SplineExceptionDuplicates ex) {
            Logger.getLogger(Spline.class.getName()).log(Level.SEVERE, null, ex);
        }
        for (int i = 0; i < xNew.length; ++i) {
            System.out.println(xNew[i] + ":" + yNew[i]);
        }
    }
}
